/* apollo-client.c
 *
 * $Id: apollo-client.c,v 1.1.2.1 2002/06/27 14:52:34 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

/*!
  \file apollo-client.c remote control client for apollo

  Apollo uses XMMSs remote control mechanism: this is the client part, a
  seperate program that sends commands to - and read information from - a
  running apollo using a simple binary protocol through a unix domain
  socket. 

  The code was copied from xmmsctrl by Alexandre David <adavid@docs.uu.se>,
  available at http://www.docs.uu.se/~adavid/utils/, and xmms.
*/
  
#include <stdio.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#define XMMS_PROTOCOL_VERSION 1

#include "../glibtypes.c"

#include "apollo-client.h" 

#ifdef __cplusplus
extern "C" {
#endif


/******************************************************************************************
************* functions from xmms *********************************************************
******************************************************************************************/

static gpointer remote_read_packet(gint fd, ServerPktHeader * pkt_hdr)
{
	gpointer data = NULL;

	if (read(fd, pkt_hdr, sizeof (ServerPktHeader)) == sizeof (ServerPktHeader))
	{
		if (pkt_hdr->data_length)
		{
			data = malloc(pkt_hdr->data_length);
			read(fd, data, pkt_hdr->data_length);
		}
	}
	return data;
}

static void remote_read_ack(gint fd)
{
	gpointer data;
	ServerPktHeader pkt_hdr;

	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
		free(data);

}

static void remote_send_packet(gint fd, guint32 command, gpointer data, guint32 data_length)
{
	ClientPktHeader pkt_hdr;

	pkt_hdr.version = XMMS_PROTOCOL_VERSION;
	pkt_hdr.command = command;
	pkt_hdr.data_length = data_length;
	write(fd, &pkt_hdr, sizeof (ClientPktHeader));
	if (data_length && data)
		write(fd, data, data_length);
}

static void remote_send_guint32(gint session, guint32 cmd, guint32 val)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, cmd, &val, sizeof (guint32));
	remote_read_ack(fd);
	close(fd);
}

static void remote_send_boolean(gint session, guint32 cmd, gboolean val)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, cmd, &val, sizeof (gboolean));
	remote_read_ack(fd);
	close(fd);
}

static void remote_send_gfloat(gint session, guint32 cmd, gfloat value)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, cmd, &value, sizeof (gfloat));
	remote_read_ack(fd);
	close(fd);
}

static void remote_send_string(gint session, guint32 cmd, gchar * string)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, cmd, string, string ? strlen(string) + 1 : 0);
	remote_read_ack(fd);
	close(fd);
}

static gboolean remote_cmd(gint session, guint32 cmd)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return FALSE;
	remote_send_packet(fd, cmd, NULL, 0);
	remote_read_ack(fd);
	close(fd);

	return TRUE;
}

static gboolean remote_get_gboolean(gint session, gint cmd)
{
	ServerPktHeader pkt_hdr;
	gboolean ret = FALSE;
	gpointer data;
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return ret;
	remote_send_packet(fd, cmd, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		ret = *((gboolean *) data);
		free(data);
	}
	remote_read_ack(fd);
	close(fd);

	return ret;
}

static guint32 remote_get_gint(gint session, gint cmd)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd, ret = 0;

	if ((fd = connect_to_session(session)) == -1)
		return ret;
	remote_send_packet(fd, cmd, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		ret = *((gint *) data);
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return ret;
}

static gfloat remote_get_gfloat(gint session, gint cmd)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd;
	gfloat ret = 0.0;

	if ((fd = connect_to_session(session)) == -1)
		return ret;
	remote_send_packet(fd, cmd, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		ret = *((gfloat *) data);
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return ret;
}



gchar *remote_get_string(gint session, gint cmd)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return NULL;
	remote_send_packet(fd, cmd, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	remote_read_ack(fd);
	close(fd);
	return data;
}

gchar *remote_get_string_pos(gint session, gint cmd, guint32 pos)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return NULL;
	remote_send_packet(fd, cmd, &pos, sizeof (guint32));
	data = remote_read_packet(fd, &pkt_hdr);
	remote_read_ack(fd);
	close(fd);
	return data;
}

gint connect_to_session(gint session)
{
	gint fd;
	uid_t stored_uid, euid;
	struct sockaddr_un saddr;
	static char *app_name_for_socket = "xmms";
	struct passwd *pwd;
	char *user_name;	

	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) != -1)
	{
		saddr.sun_family = AF_UNIX;
		stored_uid = getuid();
		euid = geteuid();
		setuid(euid);
		pwd = getpwuid(getuid());
		user_name = pwd ? pwd->pw_name : "wtf?"; /* failure should be logged */
		sprintf(saddr.sun_path, "%s/%s_%s.%d", g_get_tmp_dir(), app_name_for_socket, user_name, session);
		setreuid(stored_uid, euid);
		if (connect(fd, (struct sockaddr *) &saddr, sizeof (saddr)) != -1)
			return fd;
	}
	close(fd);
	return -1;
}

void remote_playlist(gint session, gchar ** list, gint num, gboolean enqueue, gboolean playfirstp)
{
	gint fd, i;
	gchar *data, *ptr;
	gint data_length;
	guint32 len;
	gchar **resolved_list;

	if (!enqueue)
		remote_playlist_clear(session);

	resolved_list = malloc(num*sizeof(gchar *));
	for (i = 0; i < num; i++) {
		resolved_list[i] = malloc(PATH_MAX*sizeof(char));
		if (realpath(list[i], resolved_list[i]) == NULL) {
			fprintf(stderr, "Resolve filename error: %s", strerror(errno));
			exit(1);
		}
	}

	for (i = 0, data_length = 0; i < num; i++) {
		data_length += (((strlen(resolved_list[i]) + 1) + 3) / 4) * 4 + 4;
	}
	if (data_length)
	{
		data_length += 4;
		data = (gchar*) malloc(data_length);
		for (i = 0, ptr = data; i < num; i++)
		{
			len = strlen(resolved_list[i]) + 1;
			*((guint32 *) ptr) = len;
			ptr += 4;
			memcpy(ptr, resolved_list[i], len);
			ptr += ((len + 3) / 4) * 4;
		}
		*((guint32 *) ptr) = 0;
		if ((fd = connect_to_session(session)) == -1)
			return;
		remote_send_packet(fd, playfirstp ? CMD_PLAYFIRST_ADD : CMD_PLAYLIST_ADD, data, data_length);
		remote_read_ack(fd);
		close(fd);
		free(data);
	}

	if (!enqueue)
		remote_play(session);
}

void remote_play_files(gint session, GList * list)
{
	remote_playlist_clear(session);
	remote_playlist_add(session, list);
	remote_play(session);
}

void remote_playlist_add(gint session, GList * list)
{
	gchar **str_list;
	GList *node;
	gint i, num;

	num = g_list_length(list);
	str_list = (gchar**) malloc(num * sizeof (gchar *));
	for (i = 0, node = list; i < num && node; i++, node = g_list_next(node))
	{
		str_list[i] = node->data;
	}
	remote_playlist(session, str_list, num, TRUE, FALSE);
	free(str_list);
}

void remote_playlist_delete(gint session,gint pos)
{
	remote_send_guint32(session,CMD_PLAYLIST_DELETE,pos);
}

void remote_play(gint session)
{
	remote_cmd(session, CMD_PLAY);
}

void remote_pause(gint session)
{
	remote_cmd(session, CMD_PAUSE);
}

void remote_stop(gint session)
{
	remote_cmd(session, CMD_STOP);
}

gboolean remote_is_playing(gint session)
{
	return remote_get_gboolean(session, CMD_IS_PLAYING);
}

gboolean remote_is_paused(gint session)
{
	return remote_get_gboolean(session, CMD_IS_PAUSED);
}

gint remote_get_playlist_pos(gint session)
{
	return remote_get_gint(session, CMD_GET_PLAYLIST_POS);
}

void remote_set_playlist_pos(gint session, gint pos)
{
	remote_send_guint32(session, CMD_SET_PLAYLIST_POS, pos);
}

gint remote_get_playlist_length(gint session)
{
	return remote_get_gint(session, CMD_GET_PLAYLIST_LENGTH);
}

void remote_playlist_clear(gint session)
{
	remote_cmd(session, CMD_PLAYLIST_CLEAR);
}

gint remote_get_output_time(gint session)
{
	return remote_get_gint(session, CMD_GET_OUTPUT_TIME);
}

void remote_jump_to_time(gint session, gint pos)
{
	remote_send_guint32(session, CMD_JUMP_TO_TIME, pos);
}

void remote_get_volume(gint session, gint * vl, gint * vr)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, CMD_GET_VOLUME, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		*vl = ((guint32 *) data)[0];
		*vr = ((guint32 *) data)[1];
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
}

gint remote_get_main_volume(gint session)
{
	gint vl, vr;

	remote_get_volume(session, &vl, &vr);

	return (vl > vr) ? vl : vr;
}

gint remote_get_balance(gint session)
{
	return remote_get_gint(session, CMD_GET_BALANCE);
}

void remote_set_volume(gint session, gint vl, gint vr)
{
	gint fd;
	guint32 v[2];

	if (vl < 0)
		vl = 0;
	if (vl > 100)
		vl = 100;
	if (vr < 0)
		vr = 0;
	if (vr > 100)
		vr = 100;

	if ((fd = connect_to_session(session)) == -1)
		return;
	v[0] = vl;
	v[1] = vr;
	remote_send_packet(fd, CMD_SET_VOLUME, v, 2 * sizeof (guint32));
	remote_read_ack(fd);
	close(fd);
}

void remote_set_main_volume(gint session, gint v)
{
	gint b, vl, vr;

	b = remote_get_balance(session);

	if (b < 0)
	{
		vl = v;
		vr = (v * (100 - abs(b))) / 100;
	}
	else if (b > 0)
	{
		vl = (v * (100 - b)) / 100;
		vr = v;
	}
	else
		vl = vr = v;
	remote_set_volume(session, vl, vr);
}

void remote_set_balance(gint session, gint b)
{
	gint v, vl, vr;

	if (b < -100)
		b = -100;
	if (b > 100)
		b = 100;

	v = remote_get_main_volume(session);

	if (b < 0)
	{
		vl = v;
		vr = (v * (100 - abs(b))) / 100;
	}
	else if (b > 0)
	{
		vl = (v * (100 - b)) / 100;
		vr = v;
	}
	else
		vl = vr = v;
	remote_set_volume(session, vl, vr);
}

gchar *remote_get_skin(gint session)
{
	return remote_get_string(session, CMD_GET_SKIN);
}

void remote_set_skin(gint session, gchar * skinfile)
{
	remote_send_string(session, CMD_SET_SKIN, skinfile);
}

gchar *remote_get_playlist_file(gint session, gint pos)
{
	return remote_get_string_pos(session, CMD_GET_PLAYLIST_FILE, pos);
}

gchar *remote_get_playlist_title(gint session, gint pos)
{
	return remote_get_string_pos(session, CMD_GET_PLAYLIST_TITLE, pos);
}

gint remote_get_playlist_time(gint session, gint pos)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd, ret = 0;
	guint32 p = pos;

	if ((fd = connect_to_session(session)) == -1)
		return ret;
	remote_send_packet(fd, CMD_GET_PLAYLIST_TIME, &p, sizeof (guint32));
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		ret = *((gint *) data);
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return ret;
}

void remote_get_info(gint session, gint * rate, gint * freq, gint * nch)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, CMD_GET_INFO, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		*rate = ((guint32 *) data)[0];
		*freq = ((guint32 *) data)[1];
		*nch = ((guint32 *) data)[2];
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
}

void remote_get_eq_data(gint session)
{
	/* write me! */
}

void remote_set_eq_data(gint session)
{
	/* write me! */
}

void remote_pl_win_toggle(gint session, gboolean show)
{
	remote_send_boolean(session, CMD_PL_WIN_TOGGLE, show);
}

void remote_eq_win_toggle(gint session, gboolean show)
{
	remote_send_boolean(session, CMD_EQ_WIN_TOGGLE, show);
}

void remote_main_win_toggle(gint session, gboolean show)
{
	remote_send_boolean(session, CMD_MAIN_WIN_TOGGLE, show);
}

gboolean remote_is_main_win(gint session)
{
	return remote_get_gboolean(session, CMD_IS_MAIN_WIN);
}

gboolean remote_is_pl_win(gint session)
{
	return remote_get_gboolean(session, CMD_IS_PL_WIN);
}

gboolean remote_is_eq_win(gint session)
{
	return remote_get_gboolean(session, CMD_IS_EQ_WIN);
}

void remote_show_prefs_box(gint session)
{
	remote_cmd(session, CMD_SHOW_PREFS_BOX);
}

void remote_toggle_aot(gint session, gboolean ontop)
{
	remote_send_boolean(session, CMD_TOGGLE_AOT, ontop);
}

void remote_show_about_box(gint session)
{
	remote_cmd(session, CMD_SHOW_ABOUT_BOX);
}

void remote_playlist_prev(gint session)
{
	remote_cmd(session, CMD_PLAYLIST_PREV);
}

void remote_playlist_next(gint session)
{
	remote_cmd(session, CMD_PLAYLIST_NEXT);
}

void remote_playlist_add_url_string(gint session, gchar * string)
{
	remote_send_string(session, CMD_PLAYLIST_ADD_URL_STRING, string);
}

gboolean remote_is_running(gint session)
{
	return remote_cmd(session, CMD_PING);
}

void remote_toggle_repeat(gint session)
{
	remote_cmd(session, CMD_TOGGLE_REPEAT);
}

void remote_toggle_shuffle(gint session)
{
	remote_cmd(session, CMD_TOGGLE_SHUFFLE);
}

gboolean remote_is_repeat(gint session)
{
	return remote_get_gboolean(session, CMD_IS_REPEAT);
}

gboolean remote_is_shuffle(gint session)
{
	return remote_get_gboolean(session, CMD_IS_SHUFFLE);
}

void remote_get_eq(gint session, gfloat *preamp, gfloat **bands)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;

	if (preamp)
		*preamp = 0.0;

	if (bands)
		*bands = NULL;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, CMD_GET_EQ, NULL, 0);
	data = remote_read_packet(fd, &pkt_hdr);
	if (data)
	{
		if (pkt_hdr.data_length >= 11 * sizeof(gfloat))
		{
			if (preamp)
				*preamp = *((gfloat *) data);
			if (bands)
				*bands = (gfloat *) g_memdup((gfloat *)data + 1, 10 * sizeof(gfloat));
		}
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
}

gfloat remote_get_eq_preamp(gint session)
{
	return remote_get_gfloat(session, CMD_GET_EQ_PREAMP);
}

gfloat remote_get_eq_band(gint session, gint band)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;
	gfloat val = 0.0;

	if ((fd = connect_to_session(session)) == -1)
		return val;
	remote_send_packet(fd, CMD_GET_EQ_BAND, &band, sizeof(band));
	data = remote_read_packet(fd, &pkt_hdr);
	if (data) {
		val = *((gfloat *) data);
		free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return val;
}

void remote_set_eq(gint session, gfloat preamp, gfloat *bands)
{
	gint fd, i;
	gfloat data[11];

	if ((fd = connect_to_session(session)) == -1)
		return;
	data[0] = preamp;
	for(i = 0; i < 10; i++)
		data[i + 1] = bands[i];
	remote_send_packet(fd, CMD_SET_EQ, data, sizeof(data));
	remote_read_ack(fd);
	close(fd);
}

void remote_set_eq_preamp(gint session, gfloat preamp)
{
	remote_send_gfloat(session, CMD_SET_EQ_PREAMP, preamp);
}

void remote_set_eq_band(gint session, gint band, gfloat value)
{
	gint fd;
	gchar data[sizeof(gint) + sizeof(gfloat)];

	if ((fd = connect_to_session(session)) == -1)
		return;
	*((gint *) data) = band;
	*((gfloat *) (data + sizeof(gint))) = value;
	remote_send_packet(fd, CMD_SET_EQ_BAND, data, sizeof(data));
	remote_read_ack(fd);
	close(fd);
}

void remote_quit(gint session)
{
	gint fd;

	if ((fd = connect_to_session(session)) == -1)
		return;
	remote_send_packet(fd, CMD_QUIT, NULL, 0);
	remote_read_ack(fd);
	close(fd);
}

void remote_eject(gint session)
{
	remote_cmd(session, CMD_EJECT);
}

gint remote_get_version(gint session)
{
	return remote_get_gint(session, CMD_GET_VERSION);
}

/******************************************************************************************
************* end of functions from xmms **************************************************
******************************************************************************************/

/* returns true if xmms is playing a stream */
static gboolean is_stream_playing(gint);

/* simple commands, argument is the session number */
static void print_current_title(gint); /* print the title of the current playing song */ 
static void print_playlist(gint);      /* print all titles in the play list */
static void print_playfiles(gint);     /* print all files in the play list */
static void remove_from_playlist(gint);/* removes mp3 from playlist currently playing  */
static void print_current(gint);       /* print the current playing song */
static void print_help(gint);          /* print help   */
static void print_volume(gint);        /* print volume */
static void print_time(gint);          /* print played time */
static void print_current_pos(gint);   /* print current song position in the play list */
static void play_prev(gint);           /* play the previous track wrapping to the last from the first */

/* commands needing an argument, arguments are session number and the read string argument */
static void set_track(gint, char*);    /* set the current playing song */
static void set_vol(gint, char*);      /* set the volume */
static void set_time(gint, char*);     /* set time position */
static void set_dir(gint, char*);      /* set playing list to directory/file/device */

/* commands using the rest of argv (as file names) */
static void add_to_playlist(gint session, gint from_arg_no, int argc, char *argv[]); 
static void add_to_playfirst(gint session, gint from_arg_no, int argc, char *argv[]); 
static void add_to_playlist_and_playfirst(gint session, gint from_arg_no, int argc, char *argv[]); 

/* type for simple commands */
typedef struct {
	const char *name;               /* command line argument function */
	void (*command)(gint);          /* xmms API function to use       */
	const char *help;               /* help for this command          */
} Command;


/* simple command list */
Command com[]={
	{
		"cur", 
		print_current,
		"print the current mp3 song file"
	},
	{
		"eject",
		remote_eject,
		"open xmms \"Load file(s)\" dialog window"
	},
	{
		"getpos", 
		print_current_pos,
		"print the current mp3 song position in the play list"
	},
	{
		"gettime",
		print_time,
		"print the current song's playback time in seconds"
	},
	{
		"getvol",
		print_volume,
		"print the master volume value"
	},
	{
		"help",
		print_help,
		"print this help message"
	},
	{
		"next", 
		remote_playlist_next,
		"xmms next song command, go to the next song"
	},
	{
		"pause",
		remote_pause,
		"xmms pause command, pause the playing song"
	},
	{
		"play",
		remote_play,
		"xmms play command, play the current song"
	},
	{
		"playlist",
		print_playlist,
		"print the play list songs"
	},
	{
		"playfiles",
		print_playfiles,
		"print the play list files"
	},
	{
		"pref",
		remote_show_prefs_box,
		"open xmms preference window (as Ctrl-P)"
	},
	{
		"prev",
		remote_playlist_prev,
		"xmms previous song command, go to the previous song"
	},
	{
		"previous",
		play_prev,
		"go to the previous song and wrap round to last if position is first"
	},
	{
		"remove",
		remove_from_playlist,
		"removes mp3 currently playing from playlist"
	},
	{
		"clear",
		remote_playlist_clear,
		"clears the playlist"
	},
	{
		"repeat", 
		remote_toggle_repeat,
		"toggle xmms repeat flag"
	},
	{
		"shuffle", 
		remote_toggle_shuffle,
		"toggle xmms shuffle flag"
	},
	{
		"stop",
		remote_stop,
		"xmms stop command, stop playing"
	},
	{
		"title",
		print_current_title,
		"print the current mp3 song title"
	},
	{
		"quit", 
		remote_quit,
		"terminate xmms"
	},
	{
		"--help",
		print_help,
		"print this help message"
	}
};


/* type for test commands */
typedef struct {
	const char *name;
	gboolean (*test)(gint);
	const char *help;
} Test;


/* test command list */
Test test[]={
	{
		"paused", 
		remote_is_paused,
		"returns OK if xmms is paused"
	},
	{
		"playing", 
		remote_is_playing,
		"returns OK if xmms is playing a song"
	},
	{
		"is_equalizer", 
		remote_is_eq_win,
		"returns OK if xmms has its equalizer window open"
	},
	{
		"is_main", 
		remote_is_main_win,
		"returns OK if xmms has its main window open"
	},
	{
		"is_play_list", 
		remote_is_pl_win,
		"returns OK if xmms has its playing list window open"
	},
	{
		"is_stream",
		is_stream_playing,
		"returns OK if xmms is playing a stream (http://somewhere)"
	},
	{
		"running", 
		remote_is_running,
		"returns OK if xmms is running"
	}
};


/* type for toggle commands */
typedef struct {
	const char *name;
	void (*command)(gint,gboolean);
	const char *help;
} ToggleCommand;


/* toggle command list */
ToggleCommand toggle[]={
	{
		"equalizer",
		remote_eq_win_toggle,
		"hide/show xmms equalizer window"
	},
	{
		"main", 
		remote_main_win_toggle,
		"hide/show xmms main window"
	},
	{
		"play_list", 
		remote_pl_win_toggle,
		"hide/show xmms playing list window"
	}
};


/* type for commands wanting an argument */
typedef struct {
	const char *name;
	void (*command)(gint, char*);
	const char *help;
} ArgCommand;


/* one-argument-command list */
ArgCommand argcom[]={
	{
		"dir",
		set_dir,
		"  dir <name> : clear the play list and load the (hopefully) directory\n"
		"              <name> as the play list. This should work with devices\n"
		"              like /dev/cdrom to handle music CDs."
	},
	{
		"time",
		set_time,
		"  time [+|-|/]seconds :\n"
		"\t seconds : set the playback time to seconds\n"
		"\t+seconds : advance the playback by seconds\n"
		"\t-seconds : rewind the playback by seconds\n"
		"\t/seconds : set the playback time to seconds from the end\n"
		"\t           of the song\n"
		"    examples : xmmsctrl time 30, xmmsctrl time +10"
	},
	{
		"track",
		set_track,
		"  track <n> : set the n'th track in the play list as the current track"
	},
	{
		"vol",
		set_vol,
		"  vol [+|-]percent, with the following effects\n"
		"\t percent : set the volume to percent\n"
		"\t+percent : increase the volume with percent\n"
		"\t-percent : decrease the volume with percent\n"
		"  examples : xmmsctrl vol 40, xmmsctrl vol +5, xmmsctrl vol -5"
	}
};


/* type for commands wanting many arguments */
typedef struct {
	const char *name;
	void (*command)(gint, gint, gint, char **);
	const char *help;
} ArgsCommand;

/* many-arguments-command list */
ArgsCommand argscom[]={
	{
		"add",
		add_to_playlist,
		"  add <names> : adds mp3s, directories, playlists to the playlist"
	},
	{
		"add-to-playfirst",
		add_to_playfirst,
		"  add-to-playfirst <names> : adds mp3s, directories, playlists to the playfirst list"
	},
	{
		"add-to-playlist-and-playfirst",
		add_to_playlist_and_playfirst,
		"  add-to-playlist-and-playfirst <names> : adds mp3s, directories, playlists to \n"
		"                                          the playlist and playfirst"
	}
};


/* sizes of the lists */
#define NCOM (sizeof(com)/sizeof(Command))
#define NTST (sizeof(test)/sizeof(Test))
#define NTOG (sizeof(toggle)/sizeof(ToggleCommand))
#define NARG (sizeof(argcom)/sizeof(ArgCommand))
#define NARGS (sizeof(argscom)/sizeof(ArgsCommand))

/*! wrap function to print the current played mp3 file */
static void print_current(gint session)
{
	printf("%s\n", remote_get_playlist_file(session, remote_get_playlist_pos(session)));
}


/*! wrap function to print the current master volume */
static void print_volume(gint session)
{
	printf("%d\n", remote_get_main_volume(session));
}


/*! wrap function to print the current playback time in seconds */
static void print_time(gint session)
{
	printf("%d\n", remote_get_output_time(session) / 1000);
}


/*! wrap function to print the current played mp3 title  */
static void print_current_title(gint session)
{
	printf("%s\n", remote_get_playlist_title(session, remote_get_playlist_pos(session)));
}  


/*! returns OK if xmms is playing a stream */
static gboolean is_stream_playing(gint session)
{
	return !strncmp("http://", remote_get_playlist_file(session, remote_get_playlist_pos(session)), 7);
}


/*!
 * go to previous song, wrap to the last song if the position
 * in the play list is the first. Note that in case of CD,
 * the first position is 1 otherwise it is 0.
 */
static void play_prev (gint session)
{
	const gchar DEV[] = "/dev/";
	gint first = 0;

	if (!strncmp(DEV, remote_get_playlist_file(session, remote_get_playlist_pos(session)), 5))
		first = 1;

	if (first != remote_get_playlist_pos (session))
		remote_playlist_prev(session);
	else
		remote_set_playlist_pos(session, remote_get_playlist_length(session) - 1);
}


/*! list the song names of the mp3 songs in the playlist */
static void print_playlist(gint session)
{
	int n = remote_get_playlist_length(session);
	int i = 0;
	while (i < n) 
	{
		char *name = remote_get_playlist_title(session, i);

		/* name == NULL should not happen.
		 * The occurence would be a bug from xmms.
		 * If it happens, it is not dangerous here since (null)
		 * will be output and at least one will see the problem

		 if ( name == NULL ) break; */

		printf("%d\t%s\n", ++i, name);
	}
}


/*! list the song files of the mp3 songs in the playlist */
static void print_playfiles(gint session)
{
	int n = remote_get_playlist_length(session);
	int i = 0;

	while (i < n)
	{
		char *name = remote_get_playlist_file(session, i);

		/* name == NULL should not happen.
		 * The occurence would be a bug from xmms.
		 * If it happens, it is not dangerous here since (null)
		 * will be output and at least one will see the problem

		 if ( name == NULL ) break; */

		printf("%d\t%s\n", ++i, name);
	}
}


/*! wrap function to print the current play list position */
static void print_current_pos(gint session)
{
	/* print position + 1 to match the play list output */
	printf("%d\n", remote_get_playlist_pos(session)+1);
}


/*! wrap function to remove currently playing mp3 from playlist */
static void remove_from_playlist(gint session)
{
	remote_playlist_delete(session, remote_get_playlist_pos(session));
}


/*! track command: needs a track number */
static void set_track(gint session, char *arg)
{
	int pos = atoi(arg);

	if (pos > 0)
		remote_set_playlist_pos(session, pos-1);
	else
		fprintf(stderr, "Invalid play list position (%d), must be >= 1\n", pos);
}


/*! vol command: needs number | +number | -number  (0-100)*/
static void set_vol(gint session, char *arg)
{
	gint vol = remote_get_main_volume( session );

	switch (arg[0]) {

		case '+': /* argument is positive */
		case '-': /* argument is negative */
			/* no test on the validity of the argument,
			 * not critical: in the worst case 0 is returned */
			vol += atoi(arg);
			break;

		default:
			vol = atoi(arg);
	}

	/* check bounds */
	if (vol < 0)
		vol = 0;
	else if (vol > 100)
		vol = 100;
  
	remote_set_main_volume( session, vol );
}


/*! time command: needs number | +number | -number | /number */
static void set_time(gint session, char *arg)
{
	gint ptime = remote_get_output_time( session );
	gint pos = remote_get_playlist_pos( session );
	gint length = remote_get_playlist_time( session, pos );
	
	switch (arg[0])
	{
		case '+': /* argument is positive */
		case '-': /* argument is negative */
			/* no test on the validity of the argument,
			 * not critical: in the worst case 0 is returned */
			ptime += atoi(arg) * 1000;
			break;

		case '/':
			/* if arg[i] is a recognized string argument, by
			 * definition it has at least one character, therefore
			 * argv[i]+1 is at worst \0 and atoi returns in the
			 * worst case 0 */
			ptime = length - atoi(arg+1) * 1000;
			break;

		default:
			ptime = atoi(arg) * 1000;
	}

	/* check bounds */
	if (ptime < 0)
		ptime = 0;
	else if (ptime > length)
		ptime = length;
  
	remote_jump_to_time( session, ptime );
}


/*! dir command: needs a string as argument  */
static void set_dir(gint session, char *arg)
{
	remote_playlist_clear(session);
	remote_playlist_add_url_string(session, arg);
}


/*! add-to-playlist command: swallows the rest of argv */
static void add_to_playlist(gint session, gint from_arg_no, int argc, char **argv)
{
	remote_playlist(session, argv+from_arg_no, argc-from_arg_no, TRUE, FALSE);
}


/*! add-to-playfirst command: swallows the rest of argv */
static void add_to_playfirst(gint session, gint from_arg_no, int argc, char **argv)
{
	remote_playlist(session, argv+from_arg_no, argc-from_arg_no, TRUE, TRUE); 
}


/*! add-to-playlist-and-playfirst command, swallows the rest of argv */
static void add_to_playlist_and_playfirst(gint session, gint from_arg_no, int argc, char **argv)
{
	remote_playlist(session, argv+from_arg_no, argc-from_arg_no, TRUE, FALSE);
	remote_playlist(session, argv+from_arg_no, argc-from_arg_no, TRUE, TRUE);
}


/*!
 * print xmmsctrl help
 * the dummy variable is used just for convenience
 */
static void print_help(__attribute__ ((unused)) gint dummy) {
	unsigned int i;

	/* The string is cut to conform to ISO C89 */
	puts("apollo-client, derived from xmmsctrl by Alexandre David <adavid@docs.uu.se>:\n"
		 "apollo-client is a simple tool designed to be used at the shell level,\n"
		 "typically in a small shell script associated to a keyboard shortcut. There\n"
		 "are 4 different command types:\n"
		 " - simple commands: apollo-client play\n"
		 "   perform a simple task\n"
		 " - commands with a flag argument: apollo-client main 1\n" 
		 "   set a particular state"); 
	puts(" - condition testing: apollo-client playing\n"
		 "   can be used inif statements in shells. Something to notice: this was designed to be\n"
		 "   used simply, which is, directly in if statements: if <command>; then\n"
		 "   <command>; else <command>; fi. There you put directly \"apollo-client playing\"\n"
		 "   to test if xmms is playing. Notice how the if statement works: if the\n"
		 "   command succeeds, it returns a 0, which means OK, otherwise it returns\n"
		 "   an error code.\n"
		 " - more specific commands with particular arguments");

	/* simple commands, 2 special and the rest from the list */
	puts("\n"
		 "The simple commands are\n"
		 "  launch       : launch a xmms instance if none is running\n"
		 "  not          : negate the next condition test");
	for (i=0; i<NCOM; i++)
		printf("  %-12s : %s\n", com[i].name, com[i].help);

	/* toggle commands from the list */
	puts("\n"
		 "The flag setting commands are used with 0 or 1");
	for (i=0; i<NTOG; i++)
		printf("  %-12s : %s\n", toggle[i].name, toggle[i].help);

	/* test commands from the list */
	puts("\n"
		 "The condition testing commands are");
	for (i=0; i<NTST; i++)
		printf("  %-12s : %s\n", test[i].name, test[i].help);

	/* argument commands, one special and the rest from the list */
	puts("\n"
		 "The other specific commands are\n"
		 "  session number : use the session number 'number', apollo-client looks\n"
		 "                   automatically for the first working session.\n");
	for (i=0; i<NARG; i++)
		printf("  %s\n\n", argcom[i].help); /* special format here */

	/* multiple argument commands from the list */
	for (i=0; i<NARGS; ++i)
		printf(" %s\n\n", argscom[i].help); /* special format here */
		 

	/* examples */
	puts("\n"
		 "Examples of shell scripts to define simple functions:\n"
		 "  Play/Stop :\n"
		 " \tsh -c \"if apollo-client playing;\\\n"
		 " \t       then apollo-client stop;\\\n"
		 " \t       else apollo-client play; fi\"\n"
		 "  Play/Pause :\n"
		 " \tsh -c \"if apollo-client playing;\\\n"
		 " \t       then apollo-client pause;\\\n"
		 " \t       else apollo-client play; fi\"\n");
}


/*!
 * launch a new apollo and return the session number
 * exit if error
 */
static gint launch_apollo(void) {
	gint session;
	unsigned int tries;

	switch (fork()) {

		case -1:
			perror("fork");
			exit(1);

		case 0:
			execlp("apollo", "apollo", "-vanilla", NULL);
			fprintf(stderr, "apollo not found!\n");
			exit(0);

		default:
			for (tries = 0 ; tries < 10 ; tries++ ) {
				usleep( 500000 ); /* in usec */
				for (session = 0 ; session < 16 ; session++ )
					if ( remote_is_running( session ) )
						return session;
			}
			exit(1); /* if no session found, abort */
	}
}


int main( int argc, char *argv[] ) {
	int i = 1;
	unsigned int negate = 0;
	gint session;

	if (argc == 1) {
		print_help(0);
		return 1;
	}

	/* try to find automatically xmms session. */
	for (session = 0 ; session < 16 ; session++)
		if (remote_is_running(session))
			break;

	if (session == 16) {  /* no session found     */
		if (strcmp( argv[1], "launch" ))
			return 1;         /* error return = false */
		else {
			i++;
			session = launch_apollo();
		}
	}

	for (; i < argc ; i++) {

		/* special command tests first */
		/* use a given session number */
		if ( !strcmp( argv[i], "session" ) ) {

			/* if argument left */
			if (++i < argc)
				session = atoi( argv[i] );
			else /* no argument left */
				fprintf(stderr, "Command usage: session number\n");

		}
		/* negation handling */
		else if (!strcmp(argv[i], "not"))
			negate ^= 1;
		/* handle generic commands if the command is not launch */
		else if (strcmp( argv[i], "launch")) {
			unsigned int j;
			/* I don't need this, but this improves readability since
			 * I avoid 3 nested if statements with it */
			int matched = 0;

			/* test functions */
			for (j=0; j<NTST; j++)
				if (!strcmp(argv[i], test[j].name))
					return (!test[j].test(session))^negate;

			/* simple commands, we are here only if no test was found */
			for (j=0; j<NCOM; j++)
				if (!strcmp(argv[i], com[j].name)) {

					com[j].command(session);

					matched = 1;
					break; /* command matched -> stop for loop */
				}

			if (!matched)
				/* toggle commands */
				for (j=0; j<NTOG; j++) {
					if (!strcmp(argv[i], toggle[j].name)) {

						if (++i < argc) /* if argument left */
							toggle[j].command(session, atoi(argv[i]));
						else
							fprintf(stderr, "Command usage: %s 0|1\n", toggle[j].name);

						matched = 1;
						break; /* command matched -> stop for loop */
					}
				}

			if (!matched)
				/* argument commands */
				for (j=0; j<NARG; j++) {
					if (!strcmp(argv[i], argcom[j].name)) {

						if (++i < argc) /* if argument left */
							argcom[j].command(session, argv[i]); /* i < argc here implies argv[i] != NULL */
						else
							fprintf(stderr, "Command %s needs an argument. Usage:\n%s\n",
									argcom[j].name, argcom[j].help);

						matched = 1;
						break;
					}
				}

			if (!matched)
				/* swallow rest of args commands */
				for (j=0; j<NARGS; j++)
					if (!strcmp(argv[i], argscom[j].name)) {
						argscom[j].command(session, i+1, argc, argv);
						/* could replace return with: match=1; i=argc; */
						return 0;
					}

			if (!matched)
				fprintf(stderr, "Invalid command '%s' is ignored\n", argv[i]);
		}
	}
  
	return 0; /* OK result */
}

#ifdef __cplusplus
};
#endif
