#!/bin/sh
#
# Configures Apollo for compilation
#
# The Apollo Team <team@apolloplayer.org>

PROJECT="Apollo"
CACHE=".cache"
LOGFILE=".log"
TMAKE_DEF="tmake"
QMAKE_DEF="qmake"
TMAKE=$TMAKE_DEF
PRO_FILE="apollo.pro"
MAKE_CLEAN="true"

BUILD="release"

# Read previous settings if any
if [ -f $CACHE ]; then
    echo "Reading cached settings"
    echo 
    . $CACHE
fi

# Check parameters
for arg in $*; do
    case $arg in
	--help|-h)
	    echo "Usage: $0 [options]"
	    echo
	    echo "Options: -h"
	    echo "         --help              This message"
	    echo "         --release           Build for release(default)"
	    echo "         --debug             Build for debug"
	    echo "         --make-clean        Always run \"make clean\"(default)"
	    echo "         --no-make-clean     Do not run \"make clean\""
	    echo "         --with-tmake        Use TMake for making the Makefile (default)"
	    echo "         --with-qmake        Use QMake for making the Makefile (Qt3.0++)"
	    echo "         --with-qt-dir=DIR   Where the root of Qt is installed (default "'$QTDIR'")"
	    echo "         --with-stl[=DIR]    Add STL support with optional dir for include"
	    echo "         --detect-stl        Detect if STL can be used"
	    echo "         --without-stl       Do not add STL support"
	    echo "         --with-kde2[=DIR]   Add KDE 2 support with optional dir for include"
	    echo "         --without-kde2      Do not add KDE 2 support"
	    echo "         --detect-kde2       Detect whether KDE 2 can be used (default)"
	    echo "         --enable-buffer     Enable buffer setting for mpg123"
	    echo "         --disable-buffer    Disable buffer setting for mpg123 (default)"
	    echo "         --with-id3=DIR      Set id3lib directory"
	    echo "         --with-mad=DIR      Enable mad"
	    exit 1
	    ;;
	--release)
	    BUILD="release"
	    ;;
	--debug)
	    BUILD="debug"
	    ;;
	--make-clean)
	    MAKE_CLEAN="true"
	    ;;
	--no-make-clean)
	    MAKE_CLEAN="false"
	    ;;
	--with-tmake)
            TMAKE_SET=1
	    ;;
	--with-qmake)
            TMAKE_SET=2
	    ;;
	--with-qt-dir*)
	    if echo $arg | grep -e "--with-qt-dir=" >/dev/null; then
		QTDIR=`echo $arg | sed 's/--with-qt-dir=/\1/'`
	    else
		echo "$arg: requires a directory"
	    fi
	    ;;
	--with-stl*)
	    if echo $arg | grep -e "--with-stl=" >/dev/null; then
		STL=`echo $arg | sed 's/--with-stl=/\1/'`
	    else
		STL="true"
	    fi
	    unset NO_STL
	    ;;
	--detect-stl)
	    unset STL
	    ;;
	--without-stl)
	    NO_STL="true"
	    ;;
	--with-kde2*)
	    if echo $arg | grep -e "--with-kde2=" >/dev/null; then
		KDE=`echo $arg | sed 's/--with-kde2=/\1/'`
	    else
		KDE="true"
	    fi
	    unset NO_KDE
	    ;;
	--detect-kde2)
	    unset KDE
	    unset NO_KDE
	    ;;
	--without-kde2)
	    NO_KDE="true"
	    ;;
	--enable-buffer)
	    USE_BUFFER="true"
	    ;;
	--disable-buffer)
	    unset USE_BUFFER
	    ;;
	--with-id3*)
	    if echo $arg | grep -e "--with-id3=" >/dev/null; then
		ID3DIR=`echo $arg | sed 's/--with-id3=//'`
		if test ! -d $ID3DIR ; then
		    echo "id3 directory $ID3DIR does not exist"
		    unset ID3DIR
		fi
	    else
		unset ID3DIR
	    fi
	    ;;
	--with-mad*)
	    if echo $arg | grep -e "--with-mad=" >/dev/null; then
		MADDIR=`echo $arg | sed 's/--with-mad=//'`
		if test ! -d $MADDIR ; then
		    echo "mad directory $MADDIR does not exist"
		    MADDIR=""
		fi
	    else
		echo "$arg: requires a directory"
	    fi
	    ;;
	*)
	    echo "$arg: unkown option"
            $0 -h
	    exit 1
	    ;;
    esac;
done

case $TMAKE_SET in
    1)
	TMAKE=$TMAKE_DEF
	;;
    2)
	TMAKE=$QMAKE_DEF
	;;
esac

# Store the settings
if [ -f $CACHE ]; then
    rm $CACHE
fi
echo "BUILD=$BUILD" >> $CACHE
echo "MAKE_CLEAN=$MAKE_CLEAN" >> $CACHE
if [ ! -z $QTDIR ]; then
    echo "QTDIR=$QTDIR" >> $CACHE
fi
if [ ! -z $STL ]; then
    echo "STL=$STL" >> $CACHE
fi
if [ ! -z $NO_STL ]; then
    echo "NO_STL=$NO_STL" >> $CACHE
fi
if [ ! -z $KDE ]; then
    echo "KDE=$KDE" >> $CACHE
fi
if [ ! -z $NO_KDE ]; then
    echo "NO_KDE=$NO_KDE" >> $CACHE
fi
if [ ! -z $USE_BUFFER ]; then
    echo "USE_BUFFER=$USE_BUFFER" >> $CACHE
fi
if [ ! -z $TMAKE_SET ]; then
    echo "TMAKE_SET=$TMAKE_SET" >> $CACHE
fi

# Setup tmake settings
TMAKE_WARNING="warn_off"
# if [ "$BUILD" == "debug" ]; then
#     TMAKE_WARNING="warn_on"
# fi
TMAKE_CONF="CONFIG += $BUILD $TMAKE_WARNING"

DEFTMAKEPATH="/usr/share/tmake"
DEFLOCALTMAKEPATH="/usr/local/share/tmake"
DEFTMAKELIBPATH="/usr/lib/tmake"
DEFTMAKELOCALLIBPATH="/usr/local/lib/tmake"
DEFTMAKESUSEPATH="/usr/lib/tmake/linux-g++/"
DEFTMAKEMANDRAKEPATH="/usr/lib/tmake/lib/linux-g++/"

TMAKE_FOUND=
QMAKE_PATH=
unset QMAKE_QMAKE

which $TMAKE &>/dev/null
TMAKE_SEARCH_RESULT=$?
if [ $TMAKE_SEARCH_RESULT -eq 1 -a "$TMAKE" == "$QMAKE_DEF" ]; then
    PATH=$QTDIR/bin:$PATH which $TMAKE &>/dev/null
    TMAKE_SEARCH_RESULT=$?
    if [ $TMAKE_SEARCH_RESULT -eq 0 ]; then
        echo "It seems that $TMAKE is not present in the default path,"
	echo "however it was found among the Qt installation."
	echo "$0 will add $QTDIR/bin"
	echo "to the current path while building."
	echo
	echo "It's recommened that you add this to your path"
	echo "by editing your shell config."
	echo
	echo "For bash or zsh do:"
	echo "export \$PATH=\$PATH:$QTDIR/bin"
	echo "For csh do:"
	echo "setenv \$PATH \$PATH:$QTDIR/bin"
	echo
	QMAKE_QMAKE=$QTDIR/bin/$QMAKE_DEF
	QMAKE_PATH=$QTDIR/bin
    fi
fi

if [ $TMAKE_SEARCH_RESULT -eq 1 ]; then
    if [ $TMAKE = $TMAKE_DEF ]; then
	which $QMAKE_DEF &>/dev/null
	if [ $? -eq 0 ]; then
	    TMAKE_FOUND=true
  	    TMAKE=$QMAKE_DEF
	    echo "TMake not found, switching to QMake"
	fi
    else
	which $TMAKE_DEF &>/dev/null
	if [ $? -eq 0 ]; then
	    TMAKE_FOUND=true
  	    TMAKE=$TMAKE_DEF
	    echo "QMake not found, switching to TMake"
	fi
    fi
else
    TMAKE_FOUND=true
fi

# Check for tmake
if [ "$TMAKE" == "$TMAKE_DEF" ]; then
    if [ -z $TMAKEPATH ]; then
	if [ ! -d "$DEFTMAKEPATH" -a ! \
	       -d "$DEFLOCALTMAKEPATH" -a ! \
	       -d "$DEFTMAKELIBPATH" -a ! \
	       -d "$DEFTMAKELOCALLIBPATH" -a ! \
	       -d "$DEFTMAKEMANDRAKEPATH" -a ! \
               -d "$DEFTMAKESUSEPATH"] ; then
	    echo
	    echo
	    echo '   The enviroment variable $TMAKEPATH is not set and the default'
	    echo "   tmake dirs $DEFTMAKEPATH, $DEFLOCALTMAKEPATH, $DEFTMAKELOCALLIBPATH or "
	    echo "   $DEFTMAKELIBPATH cannot be found."
	    echo '   This means that tmake is not installed or not properly setup.'
	    echo '   TMake requires this to be set to create the Makefile.'
	    echo '   You can get tmake from'
	    echo '    http://www.trolltech.com/products/download/freebies/tmake.html'
	    echo
	    echo
	    exit 1
	elif [ ! -f "$DEFTMAKEPATH/tmake.conf" -a ! \
	         -f "$DEFLOCALTMAKEPATH/tmake.conf" -a ! \
		 -f "$DEFTMAKELIBPATH/tmake.conf" -a ! \
		 -f "$DEFTMAKELOCALLIBPATH/tmake.conf" -a ! \
	         -f "$DEFTMAKEMANDRAKEPATH" -a ! \
                 -f "$DEFTMAKESUSEPATH"] ; then
	    echo
	    echo    
	    echo '   The enviroment variable $TMAKEPATH is not set, but one of the default'
	    echo "   tmake dirs $DEFTMAKEPATH, $DEFLOCALTMAKEPATH, $DEFTMAKELOCALLIBPATH, "
	    echo "   $DEFTMAKELIBPATH, $DEFTMAKEMANDRAKEPATH, or $DEFTMAKESUSEPATH was found."
	    echo "   Did NOT however find a tmake.conf, the tmake configuration file.  It might"
	    echo "   be in a platform dependent subdirectory such as linux-g++.  Find the"
	    echo "   directory, set $TMAKEPATH and run configure.sh again." 
	    echo "   (export TMAKEPATH=/usr/lib/tmake/linux-g++/)." 
	    echo
	    echo
	    exit 1
	fi
    fi
fi

if [ -z $TMAKE_FOUND ]; then
    echo
    echo
    echo "   TMake cannot be found in binary path, please make sure it's properly installed"
    echo '   You can get tmake from'
    echo '   http://www.trolltech.com/products/download/freebies/tmake.html'
    echo
    echo
    exit 1
fi

# Check for Qt enviroment variables
if [ -z $QTDIR ]; then
    echo
    echo
    echo '   The enviroment variable $QTDIR is not set, and the --with-qt-dir=DIRPATH'
    echo '   switch is not used.  '$PROJECT' (and Qt programs) requires this to be set'
    echo '   to compile. Read the Qt documentation for more information.'
    echo
    echo
    exit 1
fi

# Find a make command
if ( which gmake &>/dev/null )
then
    MAKE=gmake
else
    if ( which make &>/dev/null )
    then
	MAKE=make
    else
	echo "You don't seem to have 'make' or 'gmake' in your PATH."
	echo "Cannot proceed."
	exit 1
    fi
fi

echo "Configuring $PROJECT:"
echo "Building for "$BUILD

rm -rf $LOGFILE
rm -rf $LOGFILE-kde
rm -rf $LOGFILE-stl
rm -rf $LOGFILE-qt
rm -rf $LOGFILE-id3

CHECK="Checking whether"
CHECK_FOR="Checking for"

INCLUDES=""
DEFINES=""
SOURCES=""
HEADERS=""
LIBRARIES=""
LIBDIRS=""

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS=""

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS conftest.c $LIBS &>$LOGFILE-stl"

# Check STL support
if [ -z $NO_STL ]; then
    echo -n $CHECK "STL can be used... "
    if [ ! -z $STL ]; then
	if [ $STL != "true" ]; then
	    echo "Using STL in include path $STL"
	    INCLUDES="$INCLUDES $STL"
	else
	    echo "Using STL in standard include path"
	fi
	DEFINES="$DEFINES APOLLO_USE_STL"
    else
	cat > conftest.c <<EOF
#line 336 "configure.sh"
#include <memory>

int main( int, char ** )
{
    std::auto_ptr<int> tmp;
    tmp.reset( new int( 1 ) );
    int *ptr = tmp.get();
    return 0;
}
EOF
	eval $CONF_LINK
	if [ $? -eq 0 ]; then
	    DEFINES="$DEFINES APOLLO_USE_STL"
	    echo "STL support added"
	else
	    echo "STL cannot be used, using custom classes."
	    echo "It seems that you have not got an up-to-date STL version installed."
	    echo "If you wish to use STL in Apollo we recommend you get STL from"
	    echo "http://www.stlport.org"
	fi
	rm -rf conftest*
    fi
else
    echo "STL support disabled"
fi

QTLINK="-lqt"
CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS="-L$QTDIR/lib $QTLINK"
INCS="-I$QTDIR/include"

CONF_LINK="QTDIR=$QTDIR g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-qt"

# Check Qt support
echo -n $CHECK_FOR "Qt... "
cat > conftest.c <<EOF
#line 374 "configure.sh"
#include <qapplication.h>

#if QT_VERSION < 200
#error "Apollo requires Qt version 2.00 or higher"
#endif

int main( int argc, char **argv ) {
    QApplication app( argc, argv );
    return 0;
    }
EOF
eval $CONF_LINK
if [ $? -eq 0 ]; then
    echo -n "yes, in $QTDIR"
else
    QTLINK="-lqt-mt"
    LIBS="-L$QTDIR/lib $QTLINK"
    CONF_LINK="QTDIR=$QTDIR g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-qt"
    eval $CONF_LINK
    if [ $? -eq 0 ]; then
	echo -n "yes, in $QTDIR"
    else	
        echo "no, Apollo cannot compile without Qt"
        echo "Please download Qt from http://www.trolltech.com before running $0 again"
        echo "Consult the file INSTALL for more information"
        exit 1
    fi
fi
echo $CONF_LINK >> $LOGFILE-qt
rm -rf conftest*

# Check Qt 3.0 support
cat > conftest.c <<EOF
#line 400 "configure.sh"
#include <qapplication.h>

#if QT_VERSION < 300
#error "Qt 3.0 build for Apollo is disabled"
#endif

int main( int argc, char **argv ) {
    QApplication app( argc, argv );
    return 0;
    }
EOF
eval $CONF_LINK
if [ $? -eq 0 ]; then
    echo " (Building for Qt 3.x)"
    DEFINES="$DEFINES USING_QT3"
else
    echo " (Building for Qt 2.x)"
fi
echo $CONF_LINK >> $LOGFILE-qt
rm -rf conftest*

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS="-L$KDEDIR/lib -L$QTDIR/lib -lkdecore -lksystemtrayapplet -lDCOP $QTLINK"
INCS="-I$KDEDIR/include -I$KDEDIR/include/kde -I$QTDIR/include"

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-kde"

# Check KDE support
if [ -z $NO_KDE ]; then
    echo -n $CHECK "KDE can be used... "
    if [ ! -z $KDE ]; then
	if [ $KDE != "true" ]; then
	    echo "Using KDE installed at $KDE"
	    INCLUDES="$INCLUDES $KDE"
	    KDEDIR=$KDE
	else
	    echo "Using KDE in include path $KDEDIR"
	fi
	USE_KDE="true"
    else
	if [ -z $KDEDIR ]; then
	    KDEDIR="/usr/local/kde"
	fi
	cat > conftest.c <<EOF
#line 447 "configure.sh"
#include <stdio.h>
#include <kapp.h>
#include <ksystemtray.h>

class QmSystrayKDE : public KSystemTray
{
public:
	QmSystrayKDE(QWidget *parent = 0, const char *name = 0) : KSystemTray( parent, name ) {}
	virtual ~QmSystrayKDE() {}
};

int main() {
    KApplication app;
    if (KDE_VERSION_MAJOR != 2)
	return 1;
    else
	return 0;
}
EOF
	eval $CONF_LINK
	if [ $? -eq 0 ]; then
	    USE_KDE="true"
	    echo "KDE support added"
	else
	    echo "KDE support omitted"
	fi
	echo $CONF_LINK >> $LOGFILE-kde
  	rm -rf conftest*
    fi
else
    echo "KDE support disabled"
fi


# Check for id3lib
LIBS="-lz -lid3"
INCS=""
if [ ! -z "$ID3DIR" ]; then
    LIBS="-L$ID3DIR/lib $LIBS"
    INCS="-I$ID3DIR/include $INCS"
fi

cat > conftest.c <<EOF
#line 480 "configure.sh"
#include <id3/tag.h>

int main(int argc, char *argv[])
{
	ID3_Tag *test = new ID3_Tag();
	if ( ID3LIB_MAJOR_VERSION >= 3 && ID3LIB_MINOR_VERSION >= 8 )
		return 0;
	else
		return 1;
}
EOF

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-id3lib"

echo -n $CHECK_FOR "id3lib... "
eval $CONF_LINK
if test $? -eq 0 ; then
    ENABLE_ID3="true"
    echo -n "yes"
    LDPATH="/usr/local/lib"
    if [ ! -z "$ID3DIR" ]; then
	LDPATH="$LDPATH:$ID3DIR/lib"
    fi
    LDPATH="LD_LIBRARY_PATH=$LDPATH:$LD_LIBRARY_PATH"
    PROGRUN="$LDPATH ./conftest"
    eval $PROGRUN
    if test $? -eq 0 ; then
	echo ""
    else
	echo " but insufficient version"
	echo
	echo
	echo '   ID3lib was found but had an insufficient version'
	echo '   The required version is 3.8.0 or higher.'
	echo '   You can get id3lib from'
	echo '    http://id3lib.sourceforge.net/'
	echo
	echo
	exit 1
    fi
else
    unset ENABLE_ID3
    echo "no"
    echo
    echo
    if [ ! -z "$ID3DIR" ]; then
	echo '   ID3lib was not found using the default include and library path.'
    else
	echo "   ID3lib was not found in $ID3DIR."
    fi
    echo '   You can get id3lib from'
    echo '    http://id3lib.sourceforge.net/'
    echo
    echo
    exit 1
fi
rm -rf conftest*

# Checks done

if [ ! -z $USE_KDE ]; then
    DEFINES="$DEFINES APOLLO_USE_KDE"
    SOURCES="$SOURCES kde/qmsystray_kde.cpp kde/qmapplication_kde.cpp"
    HEADERS="$HEADERS kde/qmsystray_kde.h kde/qmapplication_kde.h"
    LIBRARIES="$LIBRARIES -lkdecore -lDCOP -lksystemtrayapplet"
    LIBDIRS="$LIBDIRS -L$KDEDIR/lib"
    INCLUDES="$INCLUDES . $KDEDIR/include $KDEDIR/include/kde"
else
    SOURCES="$SOURCES qt/qmapplication_qt.cpp"
    HEADERS="$HEADERS qt/qmapplication_qt.h"
fi

if [ ! -z $USE_BUFFER ]; then
    DEFINES="$DEFINES ENABLE_MPG123_BUFFER"
fi

if test ! -z "$MADDIR" ; then
    INCLUDES="$INCLUDES $MADDIR/include"
    DEFINES="$DEFINES ENABLE_MAD"
    LIBRARIES="$LIBRARIES -lmad"
    LIBDIRS="$LIBDIRS -L$MADDIR/lib"
fi 

if test ! -z "$ENABLE_ID3" ; then 
    DEFINES="$DEFINES ENABLE_ID3"
    LIBRARIES="$LIBRARIES -lid3"
    if test ! -z "$ID3DIR" ; then
	INCLUDES="$INCLUDES $ID3DIR/include"
	LIBDIRS="$LIBDIRS -L$ID3DIR/lib"
    fi
fi

XMAKE_INCDIR_QT="TMAKE_INCDIR_QT"
XMAKE_CXXFLAGS="TMAKE_CXXFLAGS"
if [ "$TMAKE" == "$QMAKE_DEF" ]; then
    XMAKE_INCDIR_QT="QMAKE_INCDIR_QT"
    if [ "$BUILD" == "debug" ]; then
	XMAKE_CXXFLAGS="QMAKE_CXXFLAGS_WARN_ON"
    else
	XMAKE_CXXFLAGS="QMAKE_CXXFLAGS_WARN_OFF"
    fi
fi

echo 

TMAKE_INCLUDES="unix:$XMAKE_INCDIR_QT += $INCLUDES"
TMAKE_DEFINES="DEFINES += $DEFINES"
TMAKE_LIBS="LIBS += $LIBRARIES $LIBDIRS"
TMAKE_SOURCES="SOURCES += $SOURCES"
TMAKE_HEADERS="HEADERS += $HEADERS"
if [ "$BUILD" == "debug" ]; then
    TMAKE_CXXFLAGS="unix:$XMAKE_CXXFLAGS += -Wall -Wno-ctor-dtor-privacy"
else
    TMAKE_CXXFLAGS="unix:$XMAKE_CXXFLAGS += -Wno-ctor-dtor-privacy"
fi
# We need this dummy setting since qmake chokes on empty "" entries
TMAKE_QMAKE="NON_SETTING=unset"
if [ ! -z $QMAKE_QMAKE ]; then
    TMAKE_QMAKE="QMAKE_QMAKE = $QMAKE_QMAKE"
fi

TMAKE_AFTER_CMD=""
if [ "$TMAKE" == "$QMAKE_DEF" ]; then
   TMAKE_AFTER_CMD="-after"
fi

# Create the Makefile
echo -n "Creating Makefile"
if PATH=$QMAKE_PATH:$PATH $TMAKE -o Makefile $PRO_FILE $TMAKE_AFTER_CMD "$TMAKE_CONF" "$TMAKE_CXXFLAGS" "$TMAKE_INCLUDES" "$TMAKE_DEFINES" "$TMAKE_LIBS" "$TMAKE_SOURCES" "$TMAKE_HEADERS" "$TMAKE_QMAKE" &>.log-tmake; then
    # re-run configure when apollo.pro changes
    sed  -e "s,	tmake apollo.pro -o Makefile,	./configure.sh $*," < Makefile > Makefile.tmp
    (echo "export QTDIR=$QTDIR" && cat Makefile.tmp) > Makefile
    sed -e 's,^all: \(.*\),all: \1 apollo-client/apollo-client,' < Makefile > Makefile.tmp
    mv Makefile.tmp Makefile
    if [ "$MAKE_CLEAN" == "true" ]; then
 	$MAKE clean 2>&1 > /dev/null
    fi
    echo 
    echo 
    echo $PROJECT is now configured for building. Just run $MAKE.
    echo
else
    echo " failed"
    echo "$TMAKE failed with error:"
    cat .log-tmake
fi
