/* qmconfig.cpp
 *
 * $Id: qmconfig.cpp,v 1.54.2.3 2002/10/11 06:39:03 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmconfig.h"

#include "qmrecoverymanager.h"

#include <qdir.h>
#include <qdom.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qtextstream.h>

#include <iostream>

#include <unistd.h>

/**
 * \file qmconfig.cpp
 * \brief Reads, writes and gives access to the configuration
*/

QmAutoPtr<QmConfig> QmConfig::s_pInstance;

/*!
  \class QmConfig qmconfig.h
  \brief Provides functionality for persistently storing values in XML file.

  This class is used for persistently storing data.  The class provides the
  usual get() and set() methods to locate the values.  The values are identified
  in <group, key> pairs.  Groups can be assigned to a parent group so that the
  XML tree can be built.
  
  Remember that only one instance of this class is allowed, this means
  that you cannot create this on your own, instead use
  QmConfig::instance() to obtain the pointer.
*/


/*!
  Creates a config object with default data.
*/

QmConfig::QmConfig()
	: m_LoadConfig(true),
	  m_LoadPlayList(true),
	  m_LoadPlayListTree(true),
	  m_LoadDirectoryTree(true),
	  m_LoadBadlist(true)
{
	m_pDefaultsRoot = new QDomDocument("apollo");
	m_pFileRoot 	= new QDomDocument("apollo");
	
	m_pRoot = m_pDefaultsRoot;
	
	m_pRoot->setNodeValue("apollo");
	// Create the root document element.  This is necessary if we build our own tree.
	m_pRoot->appendChild(m_pRoot->createElement("apollo"));

	m_LazySetMode	= false;
	m_SilentMode	= true;
	mergeDefaults();
	m_SilentMode	= false;

//	QTextStream conf_stream(stdout, IO_WriteOnly); // Debug output
//	m_pDefaultsRoot->save(conf_stream, 0);	
}


/*!
  
*/

QmConfig::~QmConfig()
{
    delete m_pDefaultsRoot;
    delete m_pFileRoot;
}


/*!
  Returns the only instance of the QmConfig class. Uses QmAutoPtr for storing the object
  so it is automaticly destroyed when quitting.
*/

QmConfig *
QmConfig::instance()
{
    if ( QmConfig::s_pInstance.get() == 0 )
    {
        s_pInstance.reset( new QmConfig );
    }
	
    return reinterpret_cast<QmConfig *>(QmConfig::s_pInstance.get());
}


/*!
  Merges the loaded settings with the defaults.  The default values cannot
  be stored in the same QDomDocument as the QDomDocument populated from the
  configuration file because the default values will be lost, even if there
  is no value defined in the configuration file to replace the default.

  So, the defauls are stored in a separate QDomDocument than the configuration
  file.  This function will merge these two documents.  If a value is present
  in the configuration file, the default will be ignored.  If, however, there
  is a default value for something not present in the configuration file, this
  default value will be used.

  This function is typically only used once, and that is after the configuration
  file is loaded.
 */

void
QmConfig::mergeDefaults()
{
	// *** BUG: Everything will be lost if a config file exists so the values below only
	// work when there's no config file.  Fix this.

    // Set default data
	assign("main-window");
    set( "main-window", "geometry",			QRect( 0, 0, 400, 400 ) );
    set( "main-window", "scroll-pos",		QPoint( -1, -1 ) );
    set( "main-window", "playing-song",		"" );
    set( "main-window", "browser-width",	100 );
    set( "main-window", "playfirst-heigt",	100 );
    set( "main-window", "playlist-width",	300 );
    set( "main-window", "playlist-height",	300 );
	set( "main-window", "browser-position",	0 ); // 0 = left, 1 = right, 2 = top, 3 = bottom
	set( "main-window", "show-browser",		0 ); // 0 = hide, 1 = show
	set( "main-window", "full-browser",		0 ); // 0 = not full, 1 = full
	set( "main-window", "browser-mode",		0 ); // 0 = dir, 1 = playlist
    set( "main-window", "playlist",         "playlist.xml");
    set( "main-window", "playlisttree",     "playlisttree.xml");

    assign( "file-dialog" );
    set( "file-dialog", "geometry",			QRect( 0, 0, 600, 400 ) );

	assign("path");
    set( "path", "music",                   QDir::homeDirPath() );
    set( "path", "playlist",                QDir::homeDirPath() );
    set( "path", "playlisttree",            QDir::homeDirPath() );
    set( "path", "startup-playlist",	    QDir::homeDirPath() + "/.apollo/playlist.xml"); 

	assign("mpg123");
    set( "mpg123", "buffer",				0 );
    set( "mpg123", "path",					"/usr/bin/mpg123" );
    set( "mpg123", "channels",              "stereo" );
    set( "mpg123", "downsample",            0 );

    assign("stuff");
    set( "stuff", "encoding",               "utf-8");
    set( "stuff", "packed",                 false);
	set( "stuff", "display-format",         "%p  %a  %2n %t");
	set( "stuff", "display-format-multi",   "Various  %a  %2n %p  %t");	
	set( "stuff", "random",					false);
	set( "stuff", "loop",					false);
	set( "stuff", "loop-one",               false); 

	assign("font");

	assign("key"); // values: next, prev, play, stop,
	assign("shortcut-keys"); 
}


/*!
  Loads the configuration file from ~/.apollo/.  If the folder does not
  exist, it will attempt to create it.
*/

void
QmConfig::load()
{
	if(QmRecoveryManager::instance()->loadingEnabled(Config) == false)
		return;

	QDir dir( configPath() );

	if( ! dir.exists() )
	{
		if ( dir.mkdir( configPath() ) )
        {
            QMessageBox::information(
				0, QObject::tr("Welcome to Apollo"),
				QObject::tr("It seems this is the first time you run Apollo.<br>"
							"Welcome!  A '.apollo' folder has been created in<br>"
							"your home folder in which Apollo will store its<br>"
							"configuration."));
        }
        else
        {
            QMessageBox::warning(
				0, QObject::tr("Welcome to Apollo"),
				QObject::tr("READ THIS: Apollo needs a directory named `.apollo'<br>"
							"in your home directory to store its configuration.<br>"
							"Apollo was unable to create it - perhaps you could help?<br>"
							"Type `chmod u+w ~ && mkdir ~/.apollo && chmod u+w ~/.apollo && chmod u-w ~`<br>"
							"in a shell.<br>"));
        }
		return;
	}

    QFile conf_file( configPath() + configFileName() );
	
    if ( ! conf_file.open( IO_ReadOnly ) )
    {
        std::cerr << "Failed to open configuration file \"" << conf_file.name() << "\" for reading." << std::endl
				  << "Default settings will be used." << std::endl;
        return;
    }
	
    m_pFileRoot->setContent( &conf_file );
    QDomElement root = m_pFileRoot->documentElement();
    if ( root.nodeName() != "apollo" )
    {
		std::cerr << "The configuration file appears to be corrupt.  The root node of the XML file is " << std::endl
				  << "to be \"apollo\", but instead \"" << root.nodeName() << "\" was found.  Default " << std::endl
				  << "values will be used." << std::endl;

		m_pRoot = m_pDefaultsRoot;
        mergeDefaults();
        return;
    }
	
	m_pRoot = m_pFileRoot;
	
	m_LazySetMode = true;
	mergeDefaults();
	m_LazySetMode = false;
}


/*!
  Saves the configurtion as XML to ~/.apollo/config.
*/

void
QmConfig::save()
{
	QDir dir( configPath() );

	if( ! dir.exists() )
	{
		if ( ! dir.mkdir( configPath() ) )
        {
			std::cerr << "Failed to create the directory \"" << configPath() << "\".  Configuration "
					  << "will not be saved." << std::endl;
            return;
        }
	}
	
    QFile conf_file( configPath() + configFileName() );
	
    if ( ! conf_file.open( IO_WriteOnly ) )
    {
		std::cerr << "Failed to write configuration to \"" << conf_file.name() << "\".  Configuration "
				  << "will not be saved." << std::endl;
        return;
    }

	
    QTextStream conf_stream( &conf_file );
//	QTextStream conf_stream(stdout, IO_WriteOnly); // Debug output
	m_pRoot->save(conf_stream, 0);
}


/*!
  Creates a string which contains the correct indentation for a given
  level, each level adds 4 spaces to the string. Level 0 is an empty string.

  This is used for correctly indenting the output of the config, each
  time a sub function is called the level is increased by one.
*/

QString
QmConfig::indent( int level )
{
    QString tmp;
    for ( ; level > 0; --level )
        tmp += "    ";
    return tmp;
}


/*!
  Returns the path of the config file. This is not including the filename.
*/

QString
QmConfig::configPath() const
{
    return QString( QDir::homeDirPath() + "/.apollo/");
}


/*!
  \return The path for \a f.
 */

QString
QmConfig::path(
	ConfigFile f) const
{
	switch(f)
	{
	case Config:
		return configPath() + configFileName();
		break;

	case PlayList:
		return configPath() + "playlist.xml";
		break;

	case PlayListTree:
		return configPath() + "playlisttree";
		break;

	case DirectoryTree:
		return configPath() + "directorytree";
		break;

	case Badlist:
		return configPath() + "badlist";
		break;

	default:
		break;
	}

	return QString("");
}


/*!
  hmm
 */

void
QmConfig::setLoading(
	ConfigFile f,
	bool e)
{
	switch(f)
	{
	case Config:
		m_LoadConfig = e;
		break;

	case PlayList:
		m_LoadPlayList = e;
		break;

	case PlayListTree:
		m_LoadPlayListTree = e;
		break;

	case DirectoryTree:
		m_LoadDirectoryTree = e;
		break;

	case Badlist:
		m_LoadBadlist = e;
		break;

	default:
		qWarning("QmConfig::setLoading(): Unknown file.");
		break;
	}
}


/*!
  hmm
 */

bool
QmConfig::loadingEnabled(
	ConfigFile f) const
{
	bool e;
	switch(f)
	{
	case Config:
		e = m_LoadConfig;
		break;

	case PlayList:
		e = m_LoadPlayList;
		break;

	case PlayListTree:
		e = m_LoadPlayListTree;
		break;

	case DirectoryTree:
		e = m_LoadDirectoryTree;
		break;

	case Badlist:
		e = m_LoadBadlist;
		break;

	default:
		qWarning("QmConfig::loadingEnabled(): Unknown file.");
		e = false;
		break;
	}

	return e;
}



/*!
  Returns the file name of the config file. This is not including the file path.
*/

QString
QmConfig::configFileName() const
{
    return QString( "config" );
}


/*!
  Assigns the group \a group as a child of \a parent.  If \a parent is null
  (default), \a group will be assigned as a child to the root.  If \a group
  already exist \e anywhere in the tree, no changes will be made so make sure
  the groupname is unique.

  If \a parent does not exist a warning will be printed and \a group will be
  assigned as a child to root.

  The \a group name must be different from \a parent name.  The \a group name cannot
  be blank.

  For example, if 'apollo' is root, and we do:

  \code
  assign("player");
  assign("mpg123", "player");
  \endcode

  will create a tree like this:

  \verbatim
  <apollo>
      <players>
          <mpg123>
          </mpg123>
	  </players>
  </apollo>
  \endverbatim

  Note that the group name \e must be unique in the entire tree.  So, this is not
  possible:

  \verbatim
  <apollo>
     <foobar>
	 </foobar>
	 <foo>
	     <bar>
		     <foobar>
			 </foobar>
		 </bar>
	 </foo>
  </apollo>
  \endverbatim

  Here we have two 'foobar' groups.  Since variables are assigned to groupnames
  only, they must be unique.  This limitation is for convenience.
 */

void
QmConfig::assign(
	const QString &group,
	const QString &parent)
{
	if(group.isEmpty())
	{
		qWarning("QmConfig::assign(): The group name cannot be empty.");
		return;
	}
	
	if(group == parent)
	{
		qWarning("QmConfig::assign(): The group name and parent name cannot be equal (\'%s\').", group.latin1());
		return;
	}

	if(findNode(group).isNull() == false)
	{
		if(m_SilentMode == false && m_LazySetMode == false)
			qWarning("QmConfig::assign(): The group \'%s\' already exists.", group.latin1());
		return;
	}

	QDomNode parent_node = findNode(parent);
	if(parent_node.isNull())
	{
		if( ! parent.isEmpty())
		{
			qWarning("QmConfig::assign(): Could not locate the parent node \'%s\'.  Assigning \'%s\' to root.",
					 parent.latin1(), group.latin1());
		}

		parent_node = m_pRoot->documentElement();
	}

	parent_node.appendChild( m_pRoot->createElement(group) );
}



/*!
  Find the node with name \a name.  There can be at most \e one node with the
  given name.

  \return The node found, or a null node if not found.
 */

QDomNode
QmConfig::findNode(
	const QString &name) const
{
	if(name == QString::null)
		return m_pRoot->documentElement();
	else
		return findNode(m_pRoot->documentElement(), name);
}


/*!
  Searches for the node with name \a name among the children of \a start.
  Note that the function will traverse down the tree of all children of \a
  start.
  
  Only elements will be considered, i.e. QDomElement objects.

  \return The node found, or a null node if not found.
*/

QDomNode
QmConfig::findNode(
	const QDomNode &start,
	const QString &name) const
{
	QDomNode n = start.firstChild();

	while ( ! n.isNull() )
	{		
		if(n.firstChild().isNull() == false)
		{
			QDomNode t = findNode(n, name);

			if( ! t.isNull())
				n = t;
		}

		if(n.isElement() && n.nodeName() == name)
			break;
			
		n = n.nextSibling();
	}

	return n;
}


/*!
  Retrieves the key node \a key in \a group or a null node if not found.
  For example, in a tree like this:

  \verbatim
  <group>
      <integer>42</integer>
  </group>
  \endverbatim

  the returned node will be the <integer> node, so in order to get the
  actual value (42) you need to get the first child of the <integer> node.

  \return The key node or a null node if not found.
*/

QDomNode
QmConfig::findKeyNode(
	const QString &group,
	const QString &key) const
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::findKeyNode(): Group \'%s\' not found.  The value for \'%s\' will "
				 "not be retrieved.", group.latin1(), key.latin1());
		return group_node;
	}

	QDomNode n = group_node.namedItem(key);

//  rk: that's ok, the "getter" can deal with it.
//      if it's usefull for debugging we could add a debug variable to test on
// 	if(n.isNull())
// 	{
// 		qWarning("QmConfig::findKeyNode(): The key \'%s\' was not found in the group \'%s\'.  "
// 				 "No value will be returned.", key.latin1(), group.latin1());
// 	}

	return n;
}

void
QmConfig::clear(
	const QString &group,
	const QString &key)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::clear(): Group \'%s\' not found.  The key \'%s\' will not be cleared.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
        group_node.removeChild( n );
	}
}

/*! \name Set functions
  Use these functions to persistently store values.  Each set() function has
  a corresponding get() function to retrieve the value.
 */

/*@{*/

/*!
  Assigns \a i as the value of \a key and adds this key to \a group.  If
  \a group does not exist, a warning will be printed, but the key and its
  value will not be stored.
  
  For example:

  \code
  set("mpg123", "buffer", 1024);
  \endcode

  will produce:

  \verbatim
  <mpg123>
      <buffer>1024</buffer>
  </mpg123>
  \endverbatim

  \warning The result is undefined if you try to set a key of one type to
  an already existing key of another type.  For example, if a rectangle
  exists with name 'size', then trying to set the contents of that key
  with a string will give undefined behaviour.
 */

void
QmConfig::set(
	const QString &group,
	const QString &key,
	int i)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).

		if( ! m_LazySetMode)
		{
			n = n.firstChild();
			n.setNodeValue(QString::number(i));
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		n = m_pRoot->createElement(key);
		n.appendChild( m_pRoot->createTextNode( QString::number(i) ) );
		group_node.appendChild(n);
	}
}

void
QmConfig::set(
	const QString &group,
	const QString &key,
	bool b)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).

		if( ! m_LazySetMode)
		{
			n = n.firstChild();
            n.setNodeValue(b ? "true" : "false");
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		n = m_pRoot->createElement(key);
		n.appendChild( m_pRoot->createTextNode( b ? "true" : "false" ) );
		group_node.appendChild(n);
	}
}


/*!
  \overload
*/

void
QmConfig::set(
	const QString &group,
	const QString &key,
	const QRect &rect)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).
		
		if( ! m_LazySetMode)
		{
			QDomNode w = n.namedItem("width").firstChild();
			QDomNode h = n.namedItem("height").firstChild();
			QDomNode x = n.namedItem("x").firstChild();
			QDomNode y = n.namedItem("y").firstChild();
		
			w.setNodeValue(QString::number(rect.width()));
			h.setNodeValue(QString::number(rect.height()));
			x.setNodeValue(QString::number(rect.x()));
			y.setNodeValue(QString::number(rect.y()));
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		QDomNode r = m_pRoot->createElement(key);
		QDomNode w = m_pRoot->createElement("width");
		QDomNode h = m_pRoot->createElement("height");
		QDomNode x = m_pRoot->createElement("x");
		QDomNode y = m_pRoot->createElement("y");
		w.appendChild(m_pRoot->createTextNode( QString::number(rect.width())));
		h.appendChild(m_pRoot->createTextNode( QString::number(rect.height())));
		x.appendChild(m_pRoot->createTextNode( QString::number(rect.x())));
		y.appendChild(m_pRoot->createTextNode( QString::number(rect.y())));
		r.appendChild(w);
		r.appendChild(h);
		r.appendChild(x);
		r.appendChild(y);
		group_node.appendChild(r);
	}
}


/*!
  \overload
*/

void
QmConfig::set(
	const QString &group,
	const QString &key,
	const QPoint &pos)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).

		if( ! m_LazySetMode)
		{
			QDomNode x = n.namedItem("x").firstChild();
			QDomNode y = n.namedItem("y").firstChild();
		
			x.setNodeValue(QString::number(pos.x()));
			y.setNodeValue(QString::number(pos.y()));
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		QDomNode p = m_pRoot->createElement(key);
		QDomNode x = m_pRoot->createElement("x");
		QDomNode y = m_pRoot->createElement("y");
		x.appendChild(m_pRoot->createTextNode( QString::number(pos.x())));
		y.appendChild(m_pRoot->createTextNode( QString::number(pos.y())));
		p.appendChild(x);
		p.appendChild(y);
		group_node.appendChild(p);
	}
}


/*!
  \overload
*/

void
QmConfig::set(
	const QString &group,
	const QString &key,
	const QString &text)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).
		
		if( ! m_LazySetMode)
		{
			n = n.firstChild();
			n.setNodeValue(text);
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		n = m_pRoot->createElement(key);
		n.appendChild( m_pRoot->createCDATASection(text));
		group_node.appendChild(n);
	}
}

void
QmConfig::set(
	const QString &group,
	const QString &key,
	const char *text)
{
	QDomNode group_node = findNode(group);

	if(group_node.isNull())
	{
		qWarning("QmConfig::set(): Group \'%s\' not found.  The key \'%s\' will not be set.",
				 group.latin1(), key.latin1());
		return;
	}

	QDomNode n = group_node.namedItem(key);

	if( ! n.isNull())
	{
		// n will be the key element (<key> ... </key>).  The first child will be the
		// text (value).
		
		if( ! m_LazySetMode)
		{
			n = n.firstChild();
			n.setNodeValue(text);
		}
	}
	else
	{
		// Create an element for the key (<key> ... </key>) and a text node within the
		// key (<key> textnode goes here </key>).
		
		n = m_pRoot->createElement(key);
		n.appendChild( m_pRoot->createCDATASection(text));
		group_node.appendChild(n);
	}
}


/*@}*/ // End of Doxygen group tag.


/*!
  Returns true if the \a key in \a group exists, false otherwise.
*/

bool
QmConfig::has(
	const QString &group,
	const QString &key) const
{
	QDomNode key_node = findKeyNode(group, key).firstChild();

    return !key_node.isNull();
}

/** \name Get functions
  Use these functions to retrieve persistently stored values.  Each get() function has
  a corresponding set() function to set the value.
 */

/*@{*/

/*!
  Retrieves the value of \a key in \a group.  If \a key or \a group could not
  be found, a warning will be printed and \a i will remain unchanged.
 */

void
QmConfig::get(
	const QString &group,
	const QString &key,
	int &i) const
{
	QDomNode key_node = findKeyNode(group, key).firstChild();

	if( ! key_node.isNull())
		i = key_node.nodeValue().toInt();
}


void
QmConfig::get(
	const QString &group,
	const QString &key,
	bool &b) const
{
	QDomNode key_node = findKeyNode(group, key).firstChild();

	if( ! key_node.isNull())
        b = key_node.nodeValue() == "true";
    else
        b = false;
}

/*!
  \overload
*/

void
QmConfig::get(
	const QString &group,
	const QString &key,
	QRect &rect) const
{
	QDomNode key_node = findKeyNode(group, key);

	if( ! key_node.isNull())
	{
		QDomNode x = key_node.namedItem("x").firstChild();
		QDomNode y = key_node.namedItem("y").firstChild();
		QDomNode w = key_node.namedItem("width").firstChild();
		QDomNode h = key_node.namedItem("height").firstChild();

		rect.setRect(x.nodeValue().toInt(),
					 y.nodeValue().toInt(),
					 w.nodeValue().toInt(),
					 h.nodeValue().toInt());
	}
}


/*!
  \overload
*/

void
QmConfig::get(
	const QString &group,
	const QString &key,
	QPoint &pos) const
{
	QDomNode key_node = findKeyNode(group, key);

	if( ! key_node.isNull())
	{
		QDomNode x = key_node.namedItem("x").firstChild();
		QDomNode y = key_node.namedItem("y").firstChild();

		pos.setX(x.nodeValue().toInt());
		pos.setY(x.nodeValue().toInt());
	}
}


/*!
  \overload
*/

void
QmConfig::get(
	const QString &group,
	const QString &key,
	QString &text) const
{
	QDomNode key_node = findKeyNode(group, key).firstChild();

	if( ! key_node.isNull())
		text = key_node.nodeValue();
}


/*!
  \return The rectangle \a group, \a key or an invalid rectangle if not found.
*/

QRect
QmConfig::getRect(
	const QString &group,
	const QString &key) const
{
    QRect rect;
    get( group, key, rect );
    return rect;
}


/*!
  \return The point \a group, \a key or (0, 0) if not found.
*/

QPoint
QmConfig::getPoint(
	const QString &group,
	const QString &key ) const
{
    QPoint pos;
    get( group, key, pos );
    return pos;
}


/*!
  \return The string \a group, \a key or null string if not found.
*/

QString
QmConfig::getString(
	const QString &group,
	const QString &key ) const
{
    QString text;
    get( group, key, text );
    return text;
}

/*!
  \return The integer \a group, \a key or 0 if not found.
*/

int
QmConfig::getInt(
	const QString &group,
	const QString &key ) const
{
    int val;
    get( group, key, val );
    return val;
}

/*@}*/ // End of Doxygen group tag.


