/* qmlistview.cpp
 *
 * $Id: qmlistview.cpp,v 1.46.2.3 2002/10/11 06:39:03 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmlistview.h"
#include "qmlistviewitem.h"
#include "qmmainwindow.h"
#include "message.h"

#include <iostream>

#include <qdom.h>
#include <qdragobject.h>
#include <qheader.h>
#include <qpoint.h>
#include <qtextstream.h>
#include <qtimer.h>

#include "qmdropindicator.h"
#include "qmdiritem.h"

/* Disables false warning in Visual Studio about dynamic_cast */

#ifdef _WS_WIN_
#pragma warning(disable: 4541)
#endif


/*!
  \file qmlistview.cpp
  Enhanced list view with drag'n drop support.
*/

/*!
  \class QmListView qmlistview.h
  \brief Enhanced list view with drag'n drop support.

  This class is an extension to QListView.  It adds features to QListView
  that can be useful to list views, such as reorganizability.  With this class,
  the user can rearrange the order of the items through drag'n drop operations.

  This class supports persistent storage.  By calling save(), the listview will
  be written in XML form.  If you plan to use the class for persistent storage,
  you should give the object a name!  The name of the object will be included in
  the XML output.  If the name is 'foo', the output will be as follows:

  \verbatim
  <apollo-foo>
  ...
  </apollo-foo>
  \endverbatim

  In order for the save functionality to work, all items in the listview must
  either be QmListViewItem objects.

  \sa QmDropIndicator, QmListViewItem
*/


/*!
  Creates a new listview.  \a parent and \a name is passed to the QListView
  constructor.
*/

QmListView::QmListView(
    QWidget *parent,
    const char *name)
	: QListView(parent, name),
	  m_Reorganizable(false)
{
	m_pDropIndicator = new QmDropIndicator(viewport());
	m_pDropIndicator->hide();
    m_pScrollTimer = new QTimer( this, "ScrollTimer" );
    connect( m_pScrollTimer, SIGNAL( timeout() ),
             this, SLOT( slotScrollView() ) );
}

QmListView::~QmListView() {}


/*!
  This function will find the listview item at the position \a pos.
  The header will be taken into account (unlike QListView::itemAt()).
  
  \return The item at \a pos, or null if none.
*/

QListViewItem*
QmListView::at(
	const QPoint &pos) const
{
	int xpos = pos.x();
	int ypos = pos.y() - header()->height() - itemMargin();
	return itemAt( QPoint(xpos, ypos) );
}


/*!
  Calculates the position (upper left corner) the drop indicator should have
  according to the mouse position \a pos.

  The y position is calculated like this:
  If the mouse is over the upper half of an item, the position calculated will
  be in between the item above and the one the mouse is over.  If the mouse is
  over the lower half of an item, the position calculated will be in between
  the item below and the one the mouse is over.

  The x position is calculated like this:
  The depth of the item will indicate the x position.  This will make the
  drop indicator left aligned at the depth where the item will be inserted.

  Ugly hack:
  This function will also set m_pItemAbove, which is the one we will use as
  to determine where to insert the dropped item(s).

  \return The indicator position.
  \todo This code isn't very pretty, and neither is its usage.  Refactor.
 */

QPoint
QmListView::indicatorPos(
	const QPoint &pos)
{
	if( ! m_Reorganizable)
		return QPoint(0, 0);
	
	QListViewItem *item = at(pos);

    m_InsertPos = Below;

	if(item == 0)
	{
		// Check wether the mouse is below the last item in the list view.

		QListViewItem *l = lastItem();

		// If the list is empty, place drop indicator at the top
		
		if(l == 0)
			return QPoint(0, 0);

		int ypos = l->itemPos() + l->height();

		if(pos.y() >= ypos)
        {
            m_pItemAbove  = lastChild();
			return QPoint(0, ypos);
        }
		else
			return QPoint(0, 0);
	}
	
	int depth;
	int itemypos = itemPos(item);
	int ypos 	 = pos.y() - header()->height() - itemMargin() + contentsY();
    m_pItemAbove = item;

	// Is the mouse on the bottom half of the item?
	
	if(ypos > itemypos + (item->height() / 2))
	{
		// Yes.  That means that the indicator will be shown
		// below this item, which again means that this item
		// is the item above the drop position.
		
		m_pItemAbove  = item;
		depth 		  = item->depth();		
		itemypos 	 += item->height();

		if(isFolder(item))
        {
            m_InsertPos = Child;
			depth++;
        }
	}
	else
	{
		// No.  That means that the indicator will be shown
		// above this item, which again means that this item
		// is the item below the drop position.

        // An simple implementation of previousSibling(), this is needed
        // since QListViewItem does not have one.
        QListViewItem *cur = item->parent();
        QListViewItem *above = 0;
        if ( !cur )
            cur = firstChild();
        else
            cur = cur->firstChild();
        while ( cur && cur != item )
        {
            above = cur;
            cur = cur->nextSibling();
        }

        // If we found a previous sibling use it, if not use the parent item.
        if ( above )
            m_pItemAbove = above;
        else
            m_pItemAbove = item->parent();

		if(m_pItemAbove)
			depth = m_pItemAbove->depth();
		else
			depth = 0;

		if(m_pItemAbove == item->itemAbove() && isFolder(m_pItemAbove))
        {
            m_InsertPos = Child;
            depth++;
        }
	}

	int	itemxpos = depth * (16 + itemMargin());	
	return QPoint(itemxpos, itemypos);
}


/*!
  Called when a drag is in progress and the mouse enters the widgets and
  whenever it moves within the widget.

  This function will show a drop indicator in between the lines where the
  dragged items may be dropped.  The function will make sure the drop is
  legal.

  \sa isDroppable()
  \todo Refactor; see indicatorPos().
*/

bool
QmListView::canDecode(QDragMoveEvent *e)
{
	return QTextDrag::canDecode(e);
}

void
QmListView::dragEnterEvent(QDragEnterEvent *event)
{
	event->accept(canDecode(event));
	addChild(m_pDropIndicator);	
}

/*!
  Hides the drop indicator and stops auto-scrolling (if active).
 */

void
QmListView::dragLeaveEvent(
	QDragLeaveEvent *e)
{
	stopDragging();
	QListView::dragLeaveEvent(e);
}

// void
// QmListView::dragLeaveEvent(QDragLeaveEvent *event)
// {
// 	stopDragging();
// }

void
QmListView::dragMoveEvent(
	QDragMoveEvent *e)
{
	if( ! m_Reorganizable)
		return;
	
	if( canDecode(e) )
	{
		// It's the right format.  Make sure the item above is not a child
		// of the one we're moving.

		indicatorPos(e->pos()); // nasty hack;  this function will update m_pItemAbove used below:
		QListViewItem *item = m_pItemAbove;

		// Check whether the dragged items are droppable beneath the given
		// item.  If item is null (i.e. is at top), the items are droppable.
		
		if( ! isDroppable(item))
		{
			e->ignore();
			m_pDropIndicator->hide();
		}
		else
		{
            /// \todo Can this be removed safely?
// 			if(item)
// 			{
// 				e->accept(); 
// 				m_pDropIndicator->show();
// 				QPoint pos = indicatorPos(e->pos());
// 				int indicatorwidth = header()->sectionSize(0) - pos.x();
// 				m_pDropIndicator->setFixedWidth(indicatorwidth);	
// 				moveChild(m_pDropIndicator, pos.x(), pos.y());
// 			}
            e->accept();
            moveIndicator( e->pos() );
            QPoint relpos( e->pos().x(), e->pos().y() - header()->height() );
            if ( atTopMargin( relpos ) || atBottomMargin( relpos ) )
            {
                m_DragPos = relpos;
                m_PointerPos = e->pos();
                if ( !m_pScrollTimer->isActive() )
                    m_pScrollTimer->start( 1000/(10), false );
            }
            else
                m_pScrollTimer->stop();
		}
	}
}


/*!
  Takes care of drop events.

  If the drop is accepted, the item(s) will be inserted at drop position
  as indicated by the drop indicator given in dragMoveEvent().

  \todo Currently only drops from items within the list itself are supported.
 */

void
QmListView::dropEvent(
	QDropEvent *e)
{
	if( ! m_Reorganizable)
		return;

    QString text;

    if( QTextDrag::decode(e, text) )
	{
		if( e->source() == this )
		{
			e->acceptAction();

			// We are to move items within the playlist.
			QList<QListViewItem> *selected = selectedItems();
			QListIterator<QListViewItem> it(*selected);
			
			for(it.toFirst(); it.current(); ++it)
			{
				reposition(it.current(), m_pItemAbove);
				m_pItemAbove = it.current();
			}
			
			delete selected;
		}
    }

	stopDragging();
}


/*!
  Start a drag operation.  Creates the necessary drag object, starts the drag
  as a 'Move' and activates the drop indicator as used in dragMoveEvent().

  \sa stopDragging()
  \todo Replace QTextDrag with QUriDrag or something.
 */

void
QmListView::startDragging()
{
	if( ! m_Reorganizable)
		return;
    m_pScrollTimer->stop();

	// Just put some dummy text in
	QDragObject *d = new QTextDrag( "<Apollo Playlist Drag'n Drop>", this );
	d->dragMove();
	addChild(m_pDropIndicator);
	
	// Note: d is NOT to be deleted.  Will be handeled by Qt.
}


/*!
  Stop the drag operation.  Disables the drop indicator as started through
  startDragging().

  \sa startDragging()
*/

void
QmListView::stopDragging()
{
	if( ! m_Reorganizable)
		return;
	
	m_pDropIndicator->hide();
	removeChild(m_pDropIndicator);
	if (m_pScrollTimer->isActive())
		m_pScrollTimer->stop();
}

/*!
  Record the position of where the left mouse button was clicked.  This will
  be used to determine when a drag is to be started, i.e. when there's a
  sufficient move from where the left mouse button was first clicked.

  \sa contentsMouseMoveEvent()
*/

void
QmListView::contentsMousePressEvent(
	QMouseEvent *e)
{
	QListView::contentsMousePressEvent(e);
	if(e->button() & LeftButton)
		m_Click = e->pos();
}

/*!
  Activates a drag action if the left mouse button is pressed and
  the movement threshold is large enough.

  \sa contentsMousePressEvent()
*/

void
QmListView::contentsMouseMoveEvent(
	QMouseEvent *e)
{
	if( ! m_Reorganizable)
		return;
	
	if(e->state() & LeftButton)
	{
		QPoint vector = e->pos() - m_Click;

		if(vector.manhattanLength() > DRAG_THRESHOLD)
		{
			if(hasSelectedItems())
				startDragging();
		}
	}
}


/*!
  Moves the item \a moveitem to be positioned below \a itemabove.  If \a itemabove
  is zero, \a moveitem will be made the top-most item in the list.

  The items can be anywhere within the list.  It does not have the
  limits of QListView::moveItem() and friends.

  \sa take(), insertBelow()
*/

void
QmListView::reposition(
	QListViewItem *moveitem,
	QListViewItem *itemabove)
{
	CHECK_PTR(moveitem);

	take(moveitem);
	insertBelow(moveitem, itemabove);
}


/*!
  Takes the item \a item out of the list regardless of its position.

  This function does not have the limits of QListView::takeItem().

  \sa insertBelow()
 */

void
QmListView::take(
	QListViewItem *item)
{
	CHECK_PTR(item);
	
	QListViewItem *parent = item->parent();
	
	if(parent)
		parent->takeItem(item);
	else
		takeItem(item);	
}


/*!
  Inserts \a insertitem below \a itemabove.  If \a itemabove is null,
  \a insertitem will become a "root" item.  \a insertitem will also
  become a "root" item if \a itemabove has no parent.

  Make sure to take() the item first.

  \warning Do not attempt to insert a (parent) item as a child to itself.
  \sa take(), isDroppable()
*/

void
QmListView::insertBelow(
	QListViewItem *insertitem,
	QListViewItem *itemabove)
{
	CHECK_PTR(insertitem);
	
	// First, insert the item so that it has the same parent (if any)...

	if(itemabove == 0)
    {
		insertItem(0, insertitem, 0);
    }
	else
	{
        if(m_InsertPos == Child)
        {
            insertItem(itemabove, insertitem, 0);
        }
        else
        {
            insertItem(itemabove->parent(), insertitem, itemabove);
        }
	}
}

/*!
  Helper function for inserting items.
  Inserts \a item so that the item \a parent is it's parent
  and makes sure it's placed after the item \c after.
  If parent is 0 it is placed at toplevel.
  If after is 0 it is placed at the top of the current level.
*/

void
QmListView::insertItem( QListViewItem *parent,
                        QListViewItem *item,
                        QListViewItem *after )
{
    if ( parent )
    {
        if ( !after )
            parent->insertItem( item );
        else
        {
            parent->insertItem( item );
            item->moveItem( after );
        }
    }
    else
    {
        if ( !after )
            QListView::insertItem( item );
        else
        {
            QListView::insertItem( item );
            item->moveItem( after );
        }
    }
}


/*!
  \return True if \a parent is the parent of \a item at \e any level,
  that is, \a parent does not have to be the immediate parent of \a item
  for this function to return true.  If \a parent and \a item is not
  connected at all, false is returned.  \a parent can be null, in which this
  function will always return false, but not \a item can never be null.
*/

bool
QmListView::isParent(
	QListViewItem *parent,
	QListViewItem *item) const
{
	CHECK_PTR(item);

	if(parent == 0)
		return false;
	
	QListViewItem *temp = item->parent();

	while(temp)
	{
		if(temp == parent)
			return true;

		temp = temp->parent();
	}

	return false;
}


/*!
  \return The save filename.
 */

const QString &
QmListView::fileName() const
{
	return m_FileName;
}


/*!
  This function checks whether the dragged items can be dropped below
  \a itemabove, that is, whether the dragged items can have the parent
  of \a itemabove as a parent.

  Usually that would be legal, but it is not if \a itemabove is a child
  of any of the items being dragged.  For example, if you have a list
  like this:

  <ul>
  <li>Folder</li>
  <ul>
  <li>Item 1</li>
  <li>Item 2</li>
  <li>Item 3</li>
  </ul>
  </ul>

  If you drag 'Folder', it cannot be placed somewhere in between Item 1,
  2, or 3.

  This function will use selectedItems() to get the list of items being
  dragged.

  A drop is also considered "illegal" if the item above itself is
  selected.  This is because the item above will be used to place the
  dropped items, and if the item above is removed, there drop position
  is undefined.

  \return True if the selected items can be inserted below \a itemabove,
  false otherwise.  If \a itemabove is null, true will be returned.
 */

bool
QmListView::isDroppable(
	QListViewItem *itemabove)
{
	if( ! m_Reorganizable)
		return false;
	
	// If there's no item above, the dragged item(s) is/are to inserted at
	// the top of the list, that is, they are droppable, so return true.
	
	if(itemabove == 0)
		return true;
	
	QList<QListViewItem> *selected = selectedItems();
	QListIterator<QListViewItem> it(*selected);

	bool rc = true;
	
	for(it.toFirst(); it.current(); ++it)
	{
		if(itemabove == it.current())
		{
			rc = false;
			break;
		}
		else if(isParent(it.current(), itemabove))
		{
			rc = false;
			break;
		}
	}

	delete selected;
	return rc;
}


/*!
  This function will build a list of selected items.  The term
  'selected' here has a different meaning than is normal:

  The returned list will contain the minimum set of items.  For
  example, if a folder and children of it are selected, only the folder
  item will be in the list.  When a folder is selected, all it's
  children are implicitly considered to be selected, although this is
  not shown visibly to the user.

  A different case would be to have children of a folder selected, but
  not the folder itself.  In this case, all the children will be added
  to the list.

  By defining selected as this, it's is possible to e.g. safely delete
  all items in the list and therefore also from the listview.  If
  children of a parent had been included, these children would
  automatically be deleted if the parent was deleted, and therefore
  raise a segmentation fault.  \return A list of the selected items.  An
  empty list will be returned if no items are selected, but not null.

  \warning The caller is responsible for deleting the returned list!
*/

QList<QListViewItem>*
QmListView::selectedItems()
{
	QList<QListViewItem> *selected = new QList<QListViewItem>;

	for( QListViewItemIterator it(this); it.current(); ++it )
	{
		if( it.current()->isSelected() )
		{
			selected->append( it.current() );

			// If the item has children, we will skip checking them
			// whether they are selected or not because they are
			// assumed to be selected when their parent is selected.

			if( it.current()->childCount() > 0 )
			{				
				int depth = it.current()->depth();
				++it;
				
				while(it.current() != 0)
				{
					if(it.current()->depth() > depth)
						++it;
					else
					{
						// We need to jump back one item so we don't skip
						// one item because of the ++it in the outermost loop.
						
						--it;
						break;
					}
				}
			}
		}
    }

	return selected;
}


/*!
  Removes the selected items
*/

void
QmListView::removeSelectedItems()
{
	QList<QListViewItem> *selected = selectedItems();
	selected->setAutoDelete(true);
	delete selected;
}


/*!
  Calls QListView::clear()
*/

void
QmListView::clear()
{
	QListView::clear();
}


/*!
  \return True if there are selected item(s), false otherwise.
  \warning This function is rather slow.  It's a convenience function around selectedItems().
  \todo Should be const, but selectedItems() is not const so.
*/

bool
QmListView::hasSelectedItems()
{
	QList<QListViewItem> *selected = selectedItems();

	bool rc;
	
	if(selected->count() > 0)
		rc = true;
	else
		rc = false;

	delete selected;
	return rc;
}



/*!
  Akin to firstChild() but instead returns the last item in the listview.
  
  \sa firstChild()
*/

QListViewItem*
QmListView::lastChild() const
{
    QListViewItem *found = 0;
    QListViewItem *item = firstChild();
    while ( item )
    {
        found = item;
        item = item->nextSibling();
    }
    return found;
}


/*!
  Similar to lastChild(), except this one returns the absolute bottommost
  item in the list while lastChild() returns the last item that is directly
  a child of the list view (view may not be the bottommost as that item
  can have children).

  \sa lastChild()
*/

QListViewItem*
QmListView::lastItem() const
{
    QListViewItem *found = 0;
    QListViewItem *item = lastChild();
    while ( item )
    {
        found = item;
        item = item->itemBelow();
    }
    return found;	
}


/*!
  Determines whether \a item is a folder or not.

  \return True if \a item is a folder, false otherwise or if \a item is null.
*/

bool
QmListView::isFolder(
	QListViewItem *item)
{
	if(dynamic_cast<QmDirItem*>(item) == 0)
		return false;
	else
		return true;
}


/*!
  Call this function with a filename, and the object will save itself to
  that file.

  \sa disableSelfSave()
*/

void
QmListView::enableSelfSave(
	const QString &filename)
{
	m_FileName = filename;
}


/*!
  Disables self saving.

  \sa enableSelfSave()
*/

void
QmListView::disableSelfSave()
{
	m_FileName = QString::null;
}



/*!
  Self saves, if enabled.

  \sa enableSelfSave(), disableSelfSave()
 */

void
QmListView::save()
{
	if(m_FileName.isEmpty())
		return;
	
    QFile save_file( m_FileName );
	
    if ( !save_file.open( IO_WriteOnly ) )
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                        "Error: could not write to %s ",
                                                        m_FileName.latin1()));
        return;
    }
	
    QTextStream save_stream( &save_file );
    save_stream.setEncoding(QTextStream::UnicodeUTF8);
    save_stream << "<?xml version='1.0'?>\n" 
                << "<!DOCTYPE APOLLO-PLAYLIST>\n";
	save(save_stream);
}


/*!
  Saves the object to \a out in XML format.

  \sa save()
 */

void
QmListView::save(
	QTextStream &out)
{
	QListViewItemIterator it(this);

	out << "<apollo-" << name() << ">\n";

	// Save the FIRST item in the list.  This item is then responsible for saving
	// the rest of the items in whatever fashion it desires.  It's necessary to let
	// this be the reponsibility of the item and not the listview.  The reason is that
	// in some cases, only root items, for example, needs to be saved.  Other times,
	// the entire tree needs to be saved.  Doing a simple iteration from the listview's
	// perspective will not be sufficient.
	
	if(it.current() != 0)
	{
		QmListViewItem *item = dynamic_cast<QmListViewItem*>(it.current());
	
		if(item == 0)
			qWarning("QmListView::save() - Unable to save item not derived from QmListViewItem.");
		else
			item->save(out);
	}

	out << "</apollo-" << name() << ">\n";
}


/*!
  Set \a r to true to enable reorganizability, e.g. drag and drop operations.
  False to disable.
 */

void
QmListView::reorganizable(
	bool r)
{
	m_Reorganizable = r;
}


/*!
  Slot which is called by the scroll timer.
  It will scroll the view upwards if the pointer is at the top margin,
  or down if at the bottom margin.

  It also makes sure the indicator marker is updated.
  \sa moveIndicator(), atTopMargin(), atBottomMargin()
*/

void
QmListView::slotScrollView()
{
    int movement = 15;
    if ( m_DragPos.y() < 0 || m_DragPos.y() > viewport()->height() )
        movement *= 2;
    if ( atTopMargin( m_DragPos ) )
    {
        scrollBy( 0, -movement );
    }
    else if ( atBottomMargin( m_DragPos ) )
    {
        scrollBy( 0, movement );
    }
    moveIndicator( m_PointerPos );
}

/*!
  Returns true if the position is inside the top margin,
  which is the 24 top pixels.
  \sa atBottomMargin()
*/

bool
QmListView::atTopMargin( const QPoint &pos )
{
    return pos.y() < 24;
}

/*!
  Returns true if the position is inside the bottom margin,
  which is the 24 bottom pixels.
  \sa atTopMargin()
*/

bool
QmListView::atBottomMargin( const QPoint &pos )
{
    return  pos.y() > viewport()->height() - 24;
}

/*!
  Moves the indicator to the place specified by \a mousepos.
*/

void
QmListView::moveIndicator( const QPoint &mousepos )
{
    m_pDropIndicator->show();
    QPoint pos = indicatorPos(mousepos);
    int indicatorwidth = header()->sectionSize(0) - pos.x();
    m_pDropIndicator->setFixedWidth(indicatorwidth);
    int ypos = QMIN(pos.y(),
                    contentsY() + viewport()->height() - m_pDropIndicator->height() );
    moveChild(m_pDropIndicator, pos.x(), ypos );
}
