/* qmmainwindow.cpp
 *
 * $Id: qmmainwindow.cpp,v 1.209.2.1 2002/09/23 19:26:51 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmmainwindow.h"

#include "message.h"
#include "qmaccelmanager.h"
#include "qmbrowser.h"
#include "qmconfig.h"
#include "qmcontrolbar.h"
#include "qmmarkdialog.h"
#include "qmmarkmanager.h"
#include "qmminibuffer.h"
#include "qmnullplayer.h"
#include "qmpixmapsupplier.h"
#include "qmplayfirst.h"
#include "qmplaylist.h"
#include "qmpositionbar.h"
#include "qmremotecontrol.h"
#include "qmrecoverymanager.h"
#include "qmsongitem.h"
#include "qmstatusbar.h"
#include "qmvolumebar.h"
#include "qmsystray.h"
#include "version.h"

#ifdef ENABLE_MPG123
#include "qmmpg123player.h"
#endif // ENABLE_MPG123

#ifdef ENABLE_MAD
#include "qmmadplayer.h"
#endif // ENABLE_MAD

#ifdef ENABLE_XMP
#include "qmxmpplayer.h"
#endif // ENABLE_XMP

#include <iostream>
#include <time.h>
#include <cassert>

#include <qaccel.h>
#include <qapplication.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qfont.h>
#include <qheader.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qmainwindow.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qwidgetstack.h>
#include <qtextview.h>
#include <qpalette.h>

/* Disables false warning in Visual Studio about dynamic_cast */

#ifdef _WS_WIN_
#pragma warning(disable: 4541)
#endif

/*!
  \file qmmainwindow.cpp
  The main Apollo window - control center for the whole app.
 */

/*!
  \class QmMainWindow qmmainwindow.h
  \brief The main Apollo window - control center for the whole app.

  The main window is where it's all at.  Most communication within apollo
  goes through QmMainWindow, providing loose coupling among the different
  parts.
*/

const bool QmMainWindow::debug;
const unsigned int QmMainWindow::MESSAGE_LOG_SIZE;

/*!
  Constructs a new Apollo main window.
*/

QmMainWindow::QmMainWindow(
    QWidget *parent,
    const char *name)
    : QMainWindow(parent, name),
      m_pPlayer(0),
      m_ContinueHerep(false),
      m_pContinueHere(0),
      m_ContinuePlayList(""),
      m_ContinueSong(""),
	  m_State( QmMainWindow::Ready ),
	  m_PlayState( QmMainWindow::Stopped ),
	  m_StopClicked( false ),
      m_Randomp( false ),
	  m_Loopp( false ),
	  m_pMarkDialog(0),
	  m_Position( QmMainWindow::LeftBrowser )
{
    setCaption("Apollo");
	assert(mainwin == 0); 
    mainwin = this;

	// needs to get initialized before any status messages are emitted ...
	m_pMiniBuffer = new QmMiniBuffer(this);
	m_pMiniBuffer->setHorizontalStretchable(true);

    QmConfig *conf = QmConfig::instance();
    conf->load();

//    qDebug( "Pos: j" );
	QFont font;
    if ( conf->has( "font", "name" ) )
    {
        QString fontname = conf->getString( "font", "name" );
        if (!fontname.isEmpty())
            font.setFamily(fontname);
    }
    if ( conf->has( "font", "size" ) )
    {
        int size = conf->getInt( "font", "size" );
        if (size)
            font.setPointSize(size);
    }
    if ( conf->has( "font", "weight" ) )
    {
        int weight = conf->getInt( "font", "weight" );
        if (weight)
            font.setWeight(weight);
    }
    if ( conf->has( "font", "italic" ) )
    {
        bool italic;
        conf->get( "font", "italic", italic );
        font.setItalic(italic);
    }
//    qDebug( "Pos: k" );

	qApp->setFont(font, true);

	QmAccelManager::instance();
	
	startPlayer();
	
    QmPixmapSupplier *sup = QmPixmapSupplier::instance();

    setIcon( sup->pixmap( "icons-apollo-mini" ) );

    resize( conf->getRect( "main-window", "geometry" ).size() );
    move( conf->getRect( "main-window", "geometry" ).topLeft() );

    m_pControlBar = new QmControlBar(this, "control-bar");
	addToolBar(m_pControlBar);
	
    m_pStatusBar = new QmStatusBar(this, m_pPlayer);
	addToolBar(m_pStatusBar, Top, true); 

	m_pListBrowserSplitter = new QSplitter(this);
    
	m_pBrowser = new QmBrowser(this, m_pListBrowserSplitter);

	// The use of layout below is somewhat unusual, but perfectly legal and
	// given as an example in the Qt documentation (Layout overview documentation).
	// We want a 1 pixel border around the playlist, and the only practical way I
	// know how to do that is by using a layout and set spacing in it.  The problem
	// is that we have a splitter which needs to be the parent.  A layout is not
	// a widget, but letting the splitter be the parent of both the layout and the
	// the playlist doesn't work (very well).
	//
	// The solution is as given below.
	//
	// Note that the holder, m_pHolder, is now the widget connected to the splitter
	// and it is this widget that needs to be moved with the splitter functions.
    // update: there is now two holders connected to a splitter connected to the
    // splitter.

    m_pListsSplitter = new QSplitter(m_pListBrowserSplitter);
    m_pListsSplitter->setOrientation(Vertical);

    m_pPlayFirstHolder = new QWidget(m_pListsSplitter);
    m_pPlayFirst = new QmPlayFirst(m_pPlayFirstHolder, "PlayFirst");
	m_pPlayFirst->initMenu();
    m_pPlayFirst->addColumn(tr("PlayFirst: play these before going on"), 300);
    m_pPlayFirst->addColumn(tr("Length"), 50);
    m_pPlayFirst->setColumnAlignment(1, AlignRight);
    m_pPlayFirst->setColumnWidthMode(1, QListView::Maximum);
    m_pPlayFirst->setAllColumnsShowFocus(true);
 	m_pPlayFirst->setRootIsDecorated(false);
    m_pPlayFirst->setItemMargin(3);
 	m_pPlayFirst->setSorting(-1);
	QPalette pfp(QColor(238, 232, 170)); // pale golden rod (maybe it should be colorGroup().base())
	pfp.setColor(QColorGroup::Base, colorGroup().background());
	m_pPlayFirst->setPalette(pfp);
	QVBoxLayout *templayout = new QVBoxLayout(m_pPlayFirstHolder, 1);
	templayout->addWidget(m_pPlayFirst);
    m_pPlayFirstHolder->hide();
	
    m_pPlayListHolder = new QWidget(m_pListsSplitter); 
    m_pPlayList = new QmPlayList( m_pPlayListHolder, "playlist" );
	m_pPlayList->initMenu();
    connect(m_pMiniBuffer, SIGNAL(search(const QString&)), m_pPlayList, SLOT(search(const QString&)));
    
	m_pPlayList->enableSelfSave(QmConfig::instance()->configPath() + "playlist.xml");
    m_pPlayList->addColumn(tr("Name"), 300);
    m_pPlayList->addColumn(tr("Length"), 50);
    m_pPlayList->setColumnAlignment(1, AlignRight);
    m_pPlayList->setColumnWidthMode(1, QListView::Maximum);
    m_pPlayList->setAllColumnsShowFocus(true);
	m_pPlayList->setRootIsDecorated(false);
    m_pPlayList->setItemMargin(3);
	m_pPlayList->setSorting(-1);

	templayout = new QVBoxLayout(m_pPlayListHolder, 1);
	templayout->addWidget(m_pPlayList);

    QValueList<int> widths;
    widths.append( conf->getInt( "main-window", "browser-width" ) );
    widths.append( conf->getInt( "main-window", "playlist-width" ) );
    m_pListBrowserSplitter->setSizes( widths );
    
	setCentralWidget(m_pListBrowserSplitter);

    connect(m_pPlayList, SIGNAL(doubleClicked(QListViewItem*)),
			this, SLOT(selected(QListViewItem*)));
    connect(m_pPlayList, SIGNAL(returnPressed(QListViewItem*)),
			this, SLOT(selected(QListViewItem*)));
    connect(m_pPlayList, SIGNAL(currentChanged(QListViewItem*)),
			this, SLOT(current(QListViewItem*)));
// rk: why commented out?
//    connect(m_pPlayList, SIGNAL(statusUpdate(const QString &)),
//            this, SLOT(statusUpdate(const QString &)));

    connect(m_pPlayer, SIGNAL(secondsCount(int)), this, SLOT(updateSongLength(int)));
    connect(m_pPlayer, SIGNAL(bitrate(int)), this, SLOT(updateBitrate(int)));

    m_pPositionBar = new QmPositionBar(this, m_pStatusBar, m_pPlayer);
	addToolBar(m_pPositionBar, Bottom);

	m_pVolumeBar = new QmVolumeBar(this, m_pStatusBar);
	addToolBar(m_pVolumeBar, Bottom);

	QObject::connect(QmMarkManager::instance(), SIGNAL(songRemoved()), this, SLOT(updateItems()));
	
    m_pVolumeBar->updateVolume( false );
    m_pControlBar->updateWidgets();
    m_pControlBar->updateControls();

    m_pPlayList->setFocus();

    m_pStatusBar->showMessage( tr( "Welcome to Apollo!" ) );

	int browser_position;
	conf->get("main-window", "browser-position", browser_position);
	setBrowserPosition(static_cast<QmMainWindow::BrowserPosition>(browser_position));	

	int show_browser;
	conf->get("main-window", "show-browser", show_browser);

	if (show_browser == 0)
		hideBrowser();
	else
		showBrowser();

    widths.clear();
    widths.append( conf->getInt( "main-window", "playfirst-height" ) );
    widths.append( conf->getInt( "main-window", "playlist-height" ) );
    m_pListsSplitter->setSizes( widths );

	int full_browser;
	conf->get("main-window", "full-browser", full_browser);
	if (full_browser == 0) 
		m_pListsSplitter->show();
	else
		m_pListsSplitter->hide();

	// rk: delayed to get it at the bottom - is there a better way?
	addToolBar(m_pMiniBuffer, Bottom, true);

    QAccel *accel = new QAccel(this);
    accel->connectItem(accel->insertItem(CTRL + Key_F),
                       m_pMiniBuffer,
                       SLOT(activateSearch()));

	m_pRemoteControl = new QmRemoteControl();
	if (!m_pRemoteControl->setup_ctrlsocket())
		statusUpdate(new Message(Message::Error, "Remote control start failed"));

// 	if(full_browser == 0)
// 		m_pHolder->show();
// 	else
// 		m_pHolder->hide();

	m_pPlayerMessage = new QTextView;
	m_pPlayerMessage->setCaption(tr("Apollo - Player messages"));
}


/*!
  Destructs the main window.  Window position, size etc. will be
  saved.
*/

QmMainWindow::~QmMainWindow()
{
    saveState();
	delete m_pPlayer;

    QmConfig::instance()->save();

    for(std::deque<Message*>::iterator it = m_Messages.begin(); it != m_Messages.end(); it++)
    {
        delete *it;
    }
}


/*!
  Access point to the main window.
  \todo Replace with instance() (singleton pattern); consistent with the
   rest of the code.  rk: IMHO the singleton pattern as described in the
   gang of four book doesn't apply, or rather none of the benefits apply,
   it only makes the code look uglier.  Without it there is nothing.
*/
QmMainWindow *QmMainWindow::mainwin = 0;


/*!
  Adds a directory, mp3 or playlist to the playlist
  If it's a mp3 and \a playFirst is true, it adds it to playfirst as well.
  
  \return true iff successful
*/

bool
QmMainWindow::playListAdd(
    const QString &filename,
    bool playFirst)
{
	QFileInfo info(filename);
	if (!info.exists())
		return false;
	
	if (info.isDir())
		m_pPlayList->appendDir(filename);
	else if (info.isFile())
	{
		if (info.extension(false).lower() == "mp3") {
			m_pPlayList->appendSong(filename);
			if (playFirst)
				m_pPlayFirst->append(filename);
		}
		else
			m_pPlayList->appendList(filename);
	}
	else
		return false;

	return true;
}


/*!
  Appends directory (path) \a filename to the play list
*/

void
QmMainWindow::playListAppendDir(
    const QString &filename)
{
    m_pPlayList->appendDir(filename);
    m_pControlBar->updateControls();	
}


/*!
  Adds directory (path) \a filename to the beginning of the play list
 */

void
QmMainWindow::playListPrependDir(
    const QString &filename)
{
    m_pPlayList->prependDir(filename);
    m_pControlBar->updateControls(); 
}


/*!
  Appends a song to the play list
 */

void
QmMainWindow::playListAppendFile(
    const QString &filename)
{
	m_pPlayList->appendSong(filename);
    m_pControlBar->updateControls(); 
}


/*!
  Appends playlist file (path) \a filename to the play list
*/

void
QmMainWindow::playListAppendList(
    const QString &filename)
{
    m_pPlayList->appendList(filename);
	m_pControlBar->updateControls();
}


/*!
  Prepends play list file (path) \a filename to the play list
 */

void
QmMainWindow::playListPrependList(
    const QString &filename)
{
    m_pPlayList->prependList(filename);
	m_pControlBar->updateControls();
}


/*!
  \bug should put in event queue if Busy:
*/

void
QmMainWindow::addFilesDialog()
{
	m_pControlBar->addFiles(); 
}

void
QmMainWindow::configDialog()
{
	m_pControlBar->openConfigDialog();
}


void
QmMainWindow::toggleCompactWindow()
{
	m_pControlBar->toggleCompactWindow();
}

void
QmMainWindow::showAbout()
{
	m_pControlBar->about();
}


/*!
  Saves playlist, as m3u if the names ends with ".m3u", else in apollo format.
*/

void
QmMainWindow::savePlayList(
    const QString &filename)
{
    if (filename.findRev(".m3u", -1, false) == static_cast<int>(filename.length()) - 4)
        m_pPlayList->saveAsM3u(filename);
	else
		m_pPlayList->saveAsXml(filename); 
}


/*!
  \return wheter the play list is empty
 */

bool
QmMainWindow::playListIsEmpty() const
{
    return m_pPlayList->isEmpty();
}


/*!
  \return a pointer to the QmSongItem of the currently playing song
*/

QmSongItem*
QmMainWindow::playingSong() 
{
    return m_pPlayList->currentSong();
}


/*!
  \return a pointer to the QmSongItem for the song at position \a index in
  the currently active playlist, as counted by a depth-first traversal of
  non-bad songs.
 */

QmSongItem*
QmMainWindow::song(
    int index)
{
	return activePlayList()->song(index);
}


/*!
  \return wheter the play list is at the last position.  Undefined for empty play lists.
*/

bool
QmMainWindow::playListAtEnd() const
{
    return m_pPlayList->atEnd();
}


/*!
  \return wheter the play list is at the first position.  Undefined for empty play lists.
*/

bool
QmMainWindow::playListAtBeginning() const
{
    return m_pPlayList->atBeginning();
}


/*!
  Updates the playlist widget, used to toggle packed mode.
  \todo Find proper way.
*/

void
QmMainWindow::updatePlayList() const
{
// m_pPlayList->reconfigureItems(); m_pPlayFirst->reconfigureItems();
/*! \todo find the proper way to do this. */
    m_pPlayList->setFont(m_pPlayList->font());
    m_pPlayFirst->setFont(m_pPlayFirst->font());
}


/*!
  Update status message with msg (currently just display in bottom status bar)
  \todo implement show-message-log (dump to a text area widget for example).
*/

void
QmMainWindow::statusUpdate(
    Message *msg)
{
	if(msg == 0)
		m_pMiniBuffer->clear();
    else
    {
		if (msg->type() != Message::Update)
		{
			m_Messages.push_back(msg);
			if (m_Messages.size() > MESSAGE_LOG_SIZE)
			{
				delete m_Messages.front();
				m_Messages.pop_front();
			}
		}
		m_pMiniBuffer->message(msg->message());
		qApp->processEvents();
	}
}


/*!
  \todo rk: this is bogus: PlayFirst is hidden when it's playing the last song.
  and visible when it's not yet played a song
*/

QmPlayList*
QmMainWindow::activePlayList()
{
	// m_pPlayFirst->isEmpty() is no good eihter
	return m_pPlayFirstHolder->isVisible() ? m_pPlayFirst : m_pPlayList;
}

/*!
  Does what's needed to ContinueHere - just shared code between play() and next()
  \return whether it was successful
*/

bool
QmMainWindow::tryContinueHere()
{
	m_ContinueHerep = false;
	m_pContinueHere = dynamic_cast<QmSongItem*>(m_pContinueHere);
	bool ret = true;
	if (m_pContinueHere)
		selected(m_pContinueHere, m_pPlayList);
	else
	{
		/*! \todo make this configurable */
		m_pPlayList->clear();
		m_pPlayList->appendList(m_ContinuePlayList);
		if ((m_pContinueHere = m_pPlayList->findSong(m_ContinueSong)))
			selected(m_pContinueHere, m_pPlayList);
		else
			ret = false;
	}
	m_pContinueHere = 0;	
	return ret;
}

/*!
  \returns The number of the song currently playing, counting from 0. -1 if
  no song playing.
*/

int
QmMainWindow::playListPosition() 
{
	return activePlayList()->position();
}


/*!
  Deletes the song at the requested position if it exists, else ignores it.
*/

void
QmMainWindow::playListDeletePosition(
    int pos)
{
	activePlayList()->deletePosition(pos);
}


/*!
  Plays the song at position \a pos of the active play list.  Ignored if
  the position is out of range.
*/

void
QmMainWindow::playListPosition(
    int pos)
{
	QmSongItem *tsong = song(pos);
	if (tsong)
		selected(tsong, activePlayList());
}


/*!
  \return Number of songs in the active play list.
*/

int
QmMainWindow::playListEntries()
{
    return activePlayList()->noSongs();
}


/*!
  \return Number of seconds the current song has played, or -1 if nothing is
  playing.
*/

int
QmMainWindow::currentSongPlayedTime()
{
	return m_pStatusBar->secondsPlaying();
}


/*!
  \return the file name with path of the current play list.  The empty
  string there is no file name.
*/

const QString&
QmMainWindow::playListFile()
{
    return m_pPlayList->filename();
}


/*!
  \return the volume.  From linux it seems to be in the range 0 .. 127, possibly 0 .. 31
*/

int
QmMainWindow::volume() const
{
	return QmMixer::instance()->volume();
}


/*!
  Apollo doesn't support changing the balanse, but can read it (go figure).
  \return something weird: look at the code.
  \todo look at this
 */

int
QmMainWindow::volumeBalanse() const
{
	return QmMixer::instance()->balance();
}


/*!
  Session management.  Saves which play list is currently loaded, which song it's at,
  any play first songs, position and size of windows, etc.
  \todo Qt has special support for session management. May be something to look at (ms)
*/

void
QmMainWindow::saveState()
{
    QmConfig *conf = QmConfig::instance();

	// save playfirst.xml
	QString pf(QDir::homeDirPath() + "/.apollo/playfirst.xml");
	if (m_pPlayFirst->isEmpty())
	{
		QFileInfo fi(pf);
		if (fi.exists() && fi.isFile())
		{
			QDir dir(QDir::homeDirPath() + "/.apollo");
			dir.remove("playfirst.xml"); 
		}
	}
	else
	{
		m_pPlayFirst->saveAsXml(pf);
	}

    if (m_pPlayList->isDirty()) 
        saveStartupPlayList();
    else
        conf->set("path", "startup-playlist", m_pPlayList->filename());

    // Store current song being played, if any
	if (m_pPlayList->currentSong()) 
		conf->set( "main-window", "playing-song", m_pPlayList->currentSong()->filePath() );
	else
		conf->set( "main-window", "playing-song", "" );

    conf->set("path", "startup-playlisttree", m_pBrowser->playTreeFile());
    
	// save window/pane geometry
    conf->set( "main-window", "geometry", QRect( pos(), size() ) );
    conf->set( "main-window", "scroll-pos", QPoint( m_pPlayList->contentsX(), m_pPlayList->contentsY() ) );
	
    QValueList<int> widths = m_pListBrowserSplitter->sizes();
    if ( widths.count() != 2 )
        std::cerr << "QmMainWindow::saveState(): Not two elements in the main-window splitter?" << std::endl;
	else
    {
		if (m_Position == LeftBrowser || m_Position == TopBrowser)
		{
			conf->set( "main-window", "browser-width", *widths.at(0));
			conf->set( "main-window", "playlist-width", *widths.at(1));
		}
		else
		{		
			conf->set( "main-window", "browser-width", *widths.at(1));
			conf->set( "main-window", "playlist-width", *widths.at(0));
		}
    }
    m_pPlayFirstHolder->show(); 
    widths = m_pListsSplitter->sizes();
    conf->set( "main-window", "playfirst-height", *widths.at(0));
    conf->set( "main-window", "playlist-height", *widths.at(1));
    

	conf->set( "main-window", "browser-position", m_Position);
	conf->set( "stuff", "loop", m_Loopp );
	conf->set( "stuff", "loop-one", m_LoopOnep );	
	conf->set( "stuff", "random", m_Randomp );
	
	m_pControlBar->save();
}


/*!
  Session management.  Restores play mode, play list, playlist trees, directory tree,
  play first.
*/

void
QmMainWindow::restoreState()
{
	QmConfig *conf = QmConfig::instance();
	conf->get( "stuff", "random", m_Randomp );
 	conf->get( "stuff", "loop", m_Loopp );
 	conf->get( "stuff", "loop-one", m_LoopOnep ); 
	m_pControlBar->updateControls();

	// create the short-cut manager so the shortcuts are installed.
	QmAccelManager::instance();

	bool initvalSystray;
	conf->get( "stuff", "systray", initvalSystray );
    QmSystray::show( this, initvalSystray );

    QString tree = QmConfig::instance()->getString("path", "startup-playlisttree");
    if (!tree.isEmpty() && QFile::exists(tree))
        m_pBrowser->loadPlayListTree(tree);

	// read playfirst.xml
	QString pf(QDir::homeDirPath() + "/.apollo/playfirst.xml");
	if (QFile::exists(pf))
	{
		m_pPlayFirst->appendList(pf);
		showOrHidePlayFirst();
	}
    loadStartupPlayList();
}
  

/*!
  Used on startup to load the play list used on the previous shut-down,
  and restores the currently playing song.
*/

void
QmMainWindow::loadStartupPlayList()
{
	if(QmRecoveryManager::instance()->loadingEnabled(QmConfig::PlayList) == false)
		return;
		
    setState( Busy );

    // Restore playlist
    QmConfig *conf = QmConfig::instance();

	QFileInfo playlist(conf->getString("path", "startup-playlist"));

    if ( playlist.exists() )
	{
        m_pPlayList->loadPlayList( playlist.filePath() );
	
		// Restore last playing song
		QmSongItem *songitem = m_pPlayList->findSong(conf->getString( "main-window", "playing-song" ));
		if (songitem)
		{
			songitem->setPlaying(true);
			m_pPlayList->setCurrent(songitem);
			m_pPlayList->setSelected(songitem, true);
			m_pPlayList->setCurrentItem(songitem);
			m_pPlayList->ensureItemVisible(songitem);
		}
		else
		{
			QPoint scroll_pos = conf->getPoint( "main-window", "scroll-pos" );
			m_pPlayList->setContentsPos( scroll_pos.x(), scroll_pos.y() );
			m_pPlayList->setColumnWidth( 0, m_pPlayList->visibleWidth() - m_pPlayList->columnWidth(1) );
			m_pPlayList->triggerUpdate();
		}
	}

    setState( Ready );
    m_pControlBar->updateControls();
}


/*!
  Stores the current playlist in an internal file for playlist persistence
  and stores the current playing song in the config.
*/

void
QmMainWindow::saveStartupPlayList()
{
    setState( Busy );
	
    // Store playlist
    QmConfig *conf = QmConfig::instance();
    conf->set("path", "startup-playlist", QDir::homeDirPath() + "/.apollo/playlist.xml");

    m_pPlayList->saveAsXml(conf->getString("path", "startup-playlist"));

    setState( Ready );
}


/*!
  Pauses if already playing, else starts a new one from PlayFirst, Random,
  ContinueHere, selected playlist item or top of playlist; in that
  preference order.

  \sa stop(), pause()
*/

void
QmMainWindow::play()
{
	if (m_PlayState==Paused || m_pPlayer->isPlaying())
        pause();
    else if (!m_pPlayFirst->isEmpty())
        playAndUpdateControls(dynamic_cast<QmSongItem*>(m_pPlayFirst->pop()));
    else if (m_pPlayList->currentSong() == 0)
	{
		stop(); 
		QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, "No non-bad songs in playlist")); 
	}
	else if (m_Randomp)
        selected(m_pPlayList->randomSong(), m_pPlayList);
    else if (m_ContinueHerep)
	{
		if (!tryContinueHere()) 
			play(); 
    }
    else 
		selected(m_pPlayList->currentSong());
}
 

/*!
  Stops the music.

  \sa play(), pause()
*/

void
QmMainWindow::stop()
{
	m_StopClicked = true;
	m_pPlayer->stop();
    setPlayingState( QmMainWindow::Stopped );
	m_pControlBar->setPauseEnabled(false);
	m_pControlBar->setStopEnabled(false);
	m_pControlBar->updateControls();
}


/*!
  Toggle pause / play
  
  \todo blink play button or something 
  \sa play(), stop()
*/

void
QmMainWindow::pause()
{
	if (m_PlayState==Paused || m_PlayState==Playing)
	{
		m_pPlayer->pause();
		m_PlayState = m_PlayState==Paused ? Playing : Paused;
		m_pStatusBar->showSongInfo();
	}
	m_pControlBar->updateControls();
}


/*!
  Selectes the previous song item, if any, and plays it, unless there is
  items in PlayFirst.  Folder items and bad songs will be skipped.
  
  \sa next()
*/

void
QmMainWindow::prev()
{
    if (!m_pPlayFirst->isEmpty() || m_pPlayList->isEmpty() || m_Randomp)
	{
        statusUpdate(new Message(Message::Bug,
                                 "Can't use prev when there is songs in PlayFirst, "
                                 "PlayList is empty or random mode is on"));
        return;
    }

    if ( playingState() == QmMainWindow::Stopped )
		play();
    else
		selected(m_pPlayList->prevSong(m_Loopp));			
}


/*!
  Plays next song, if any, in this preference order
  - PlayFirst
  - Random
  - ContinueHere
  - PlayList

  \sa prev()
*/

void
QmMainWindow::next()
{
	if (m_PlayState == Stopped)
	{
		play();
		return;
	}
    
    QListViewItem* tmp = 0;
    QmSongItem* tmp2;

    if (!m_pPlayFirst->isEmpty() && (tmp=m_pPlayFirst->pop())) 
        playAndUpdateControls(dynamic_cast<QmSongItem*>(tmp));
	else if (m_LoopOnep && m_pPlayList->currentSong())
		selected(m_pPlayList->currentSong());
    else if (m_Randomp && !m_pPlayList->isEmpty()) 
        selected(m_pPlayList->randomSong(), m_pPlayList);
    else if (m_ContinueHerep)
	{
		if (!tryContinueHere()) 
			next(); 
    }
    else if ( playingState() != QmMainWindow::Stopped )
	{
		if ((tmp2 = m_pPlayList->nextSong(m_Loopp)))
			selected(tmp2);
		else
			stop();
	}
    else if (m_pPlayList->currentSong())
        selected(m_pPlayList->currentSong());
	else
		stop();
}


/*!
  The current random mode selects a song at random every time a new song
  is needed.  There is no history.
  
  \todo implement history
*/

void
QmMainWindow::toggleRandomMode()
{
    m_Randomp = !m_Randomp;
	m_pControlBar->updateControls();
}


/*!
  Toggles loop mode on/off.
 */

void
QmMainWindow::cycleLoopMode()
{
	if (m_Loopp)
	{
		m_Loopp = false;
		m_LoopOnep = true;
	}
	else if (m_LoopOnep)
		m_Loopp = m_LoopOnep = false;
	else
		m_Loopp = true;
	
	m_pControlBar->updateControls();
}


/*!
  Calls selected(item, playlist) with the play list as \a playlist.
 */

void
QmMainWindow::selected(
    QListViewItem *Item) 
{
    selected(Item, m_pPlayList);
}


/*!
  Invoked to play a song from the play list.  Called when an item has been
  selected in the play list (typically by double-clicking or pressing
  return), and from play(), and next().  
  If \a item is a:
  - Non-bad song it's meta info (id3 data for mp3s) is updated, and it's
    played, or put in the PlayFirst list if that is non-empty.
  - bad song, a message saying so is put on the status bar.
  - directory it is ignored.
*/

void
QmMainWindow::selected(
    QListViewItem *item,
    QmPlayList *playlist)
{
	assert(item && dynamic_cast<QmPlayListItem*>(item));
	if (!reinterpret_cast<QmPlayListItem*>(item)->isSong())
		return;
	
    QmSongItem *song = reinterpret_cast<QmSongItem*>(item);

    if (song->isBad())
	{
        statusUpdate(new Message(Message::Status, "%s is marked bad", song->text(0).latin1()));
        return;
    }

	if (song->artist().isEmpty())
		song->readInfo(QmConfig::instance()->getString("stuff", "display-format").latin1(),
					   QmConfig::instance()->getString("stuff", "display-format-multi").latin1());
    
    if (!m_pPlayFirst->isEmpty())
    {
        addToPlayFirst(song);
        return;
    }

    if (!QFile::exists(song->filePath()))
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "Song doesn't exist: %s",
                                                        song->filePath().latin1()));
        return;
    }
    
    playlist->setCurrent(song);
    playlist->setCurrentItem(song);

    playAndUpdateControls(song);
}


/*!
  hides PlayFirst if it's empty, called from PlayFirst
*/

void
QmMainWindow::showOrHidePlayFirst()
{
    if (m_pPlayFirst->isEmpty() == m_pPlayFirstHolder->isVisible())
	{
        if (m_pPlayFirst->isEmpty())
            m_pPlayFirstHolder->hide();
        else
        {
            // ms: Personally, I don't see a use for the playfirst to have focus.
            // Almost always, I find myself wanting to continue using the keyboard
            // in the playlist. So, set the focus back to the previous widget
            // after showing the playfirst list.
            
            QWidget *hadfocus = qApp->focusWidget();
            m_pPlayFirstHolder->show();

            if(hadfocus)
                hadfocus->setFocus();
        }
        
        m_pListsSplitter->refresh();
    }
}


/*!
  Plays the song and updates the gui controls accordingly.
  This should be the only way to get a song playing, and thus
  allow for other side-effects such as logging or informing other
  programs.  Called through selected.
*/

void
QmMainWindow::playAndUpdateControls(
    QmSongItem *song)
{
    CHECK_PTR(song);
    
	m_StopClicked = false;      // to distinguish end-of-song and click
	m_pStatusBar->clearName();
	m_pStatusBar->clearTime();
    m_pPlayer->play(song->filePath());
    setPlayingState( QmMainWindow::Playing );
    m_pControlBar->updateControls();
    m_pStatusBar->showSongInfo();
	m_pStatusBar->setSongTitle(song->displayString());
}


/*!
  Disables the play button if a folder is selected and we would have
  played the selected item (not in random mode, playfirst empty).
  \a item is a play list item.
*/

void
QmMainWindow::current(
	QListViewItem *item)
{
    QmPlayListItem *thing = dynamic_cast<QmPlayListItem*>(item);

    if ( thing && (m_Randomp || !m_pPlayFirst->isEmpty() || !thing->isBad()) )
		m_pControlBar->setPlayEnabled(true);
    else 
		m_pControlBar->setPlayEnabled(false);
}


/*!
  This will be called whenever the player stops playing.  This can happen
  three ways:

  <ul>
  <li> End of song reached.
  <li> User pushed stop, leading stop() to be called.
  <li> Stop is received through the remote-control socket, leading stop() to be called.
  </ul>

  In case the end of song was reached, we want to start loading the next
  song.  We do \e not want to do that if stop is pushed.  This function
  takes care of both these issues.

  \sa stop()
*/

void
QmMainWindow::playingStopped()
{  
	if ( m_StopClicked )
        setPlayingState( QmMainWindow::Stopped );
    else
		next();
}


/*!
  Clears the play list.
*/

void
QmMainWindow::clearPlayList()
{
    m_pContinueHere = 0;
	m_pPlayList->clear();
    m_pControlBar->updateControls();
}


/*!
  Remove(s) the current item(s), if any, from the play list.  If the
  current item is the playing item, it will stop playing and the
  next one, if any, will start playing.
*/

void
QmMainWindow::removeItem()
{
	if (m_pPlayList->hasFocus())
	{
		bool was_playing = m_pPlayer->isPlaying();
		bool stop_playing;
		QmSongItem *newsong = m_pPlayList->removeSelected(&stop_playing);
        
        if (m_pContinueHere == newsong)
            m_pContinueHere = 0;

		if (newsong != 0)
		{
			// The reason for this variable is to distinguish between user Stop action
			// and end of song Stop.

			m_StopClicked = false;
	
			m_pStatusBar->clearName();
			m_pStatusBar->clearTime();

			if (was_playing)
				selected(newsong);
		}
		else if (stop_playing)
			stop();

		m_pControlBar->updateControls();
	}
	else if (m_pBrowser->hasBrowserFocus())
	{
		m_pBrowser->removeItem();
	}
}


/*!
  Starts a player process and assigns this to m_pPlayer.
*/

void
QmMainWindow::startPlayer()
{
	bool reinit;
	
	if(m_pPlayer != 0)
	{
		reinit = true;
		delete m_pPlayer;
	}
	else
		reinit = false;

	int			rc = -1;
	QmPlayer	*p = 0;
	
#ifdef _WS_X11_	
#ifdef ENABLE_MPG123
	QmMpg123Player *p2 = new QmMpg123Player;
	rc = p2->init();
	p = p2;
#endif // mpg123
//	QmMadPlayer *p2 = new QmMadPlayer;
//	rc = 0;
//	p = p2;
//	QmXmpPlayer *p2 = new QmXmpPlayer;
//	rc = 0;
//	p = p2;
#endif	// x11

	if (rc < 0)
		m_pPlayer = QmNullPlayer::instance();
	else
		m_pPlayer = p;

	QObject::connect(m_pPlayer, SIGNAL(playingStopped()), this, SLOT(playingStopped()));
	QObject::connect(m_pPlayer, SIGNAL(playerMessage(const QString&)),
					 this, SLOT(playerMessage(const QString&)));

	if(reinit)
	{
		m_pPositionBar->initialize(m_pPlayer);
		m_pStatusBar->initialize(m_pPlayer);
	}
}


/*!
  Marks the current (focus) item to be played next (overrides any previous setting).
*/

void
QmMainWindow::continueHere(
    QmPlayListItem *here)
{
    CHECK_PTR(here);
    
    if (here->isSong())
    {
        m_ContinueHerep = true;
        m_pContinueHere = here;
        m_ContinuePlayList = m_pPlayList->filename();
        m_ContinueSong = static_cast<QmSongItem*>(here)->filePath();
    }
}


/*!
  Sets the current item(s) to be 'bad'.  Bad items will be skipped automatically
  when playing a play list.  Marking items as bad is useful if there are some
  files that the underlaying player cannot cope with, even though the files
  may be correct.

  \sa showMarkDialog()
*/

void
QmMainWindow::setMark()
{
	QListViewItem *item = m_pPlayList->firstChild();
	QmSongItem *song;

	while (item)
	{
		song = dynamic_cast<QmSongItem*>(item);

		if (song)
		{
			if (song->isSelected())
				song->setMark(true);
		}

		item = item->itemBelow();		
	}	

	m_pControlBar->updateControls();
	current(m_pPlayList->currentItem());
}


/*!
  Shows the mark dialog.
*/

void
QmMainWindow::showMarkDialog()
{
	if (m_pMarkDialog == 0)
		m_pMarkDialog = new QmMarkDialog(0, "markdialog");
	
	m_pMarkDialog->show();
	m_pMarkDialog->raise();
	m_pControlBar->updateControls();
	current(m_pPlayList->currentItem());
}


/*!
  Returns the main state of the program.

  \sa playingState()
*/

QmMainWindow::MainState
QmMainWindow::state() const
{
    return m_State;
}


/*!
  Sets the main state of the program and enables/disables the various
  parts of the GUI.
  
  \sa setPlayingState()
*/

void
QmMainWindow::setState(
    QmMainWindow::MainState state)
{
    if ( m_State == state )
        return;
	
    m_State = state;
    m_pControlBar->updateWidgets();
}


/*!
  Returns the play state of the program, this tells wheter a song is
  playing, paused or is stopped.
  
  \sa state()
*/

QmMainWindow::PlayState
QmMainWindow::playingState() const
{
    return m_PlayState;
}


/*!
  Sets the state of the player, this will update the various parts of the GUI
  by enabling/disabling buttons and text.
  
  \sa setState()
*/

void
QmMainWindow::setPlayingState(
    QmMainWindow::PlayState state)
{
    if ( state == m_PlayState )
        return;
	
    m_PlayState = state;

    if ( state == QmMainWindow::Stopped )
		m_pStatusBar->clearTime();

    m_pStatusBar->updateStatus();
    m_pStatusBar->updateCaption();
    m_pControlBar->updateControls();
}


/*!
  Updates the items in the list view.  This will typically happen when a
  mark is cleared from an item in the mark editor.
*/

void
QmMainWindow::updateItems()
{
	m_pPlayList->triggerUpdate();
	m_pControlBar->updateControls();	
}


/*!
  \return The playlist.
  \deprecated Only the main window should know about the playlist.
*/

QmPlayList *
QmMainWindow::playList()
{
	CHECK_PTR(m_pPlayList);
	return m_pPlayList;
}


/*!
  Hides/shows the play list.
*/

void
QmMainWindow::togglePlayList()
{
	if (m_pListsSplitter->isVisible())
		m_pListsSplitter->hide();
	else
		m_pListsSplitter->show();
}


/*!
  Flatten: remove all directories (but not the songs in them).
*/

void
QmMainWindow::flattenPlayList()
{
    m_pPlayList->flatten();
}


/*!
  \return The browser position.
  \sa setBrowserPosition(QmMainWindow::BrowserPosition)
 */

QmMainWindow::BrowserPosition
QmMainWindow::browserPosition() const
{
	return m_Position;
}


/*!
  Sets the integrated browser to be \a pos. If the requested position is the
  same as the current position, this function will do nothing.

  \sa browserPosition()
*/

void
QmMainWindow::setBrowserPosition(
	QmMainWindow::BrowserPosition pos)
{
	if (m_Position == pos)
		return;
	
	switch(pos)
	{
		case LeftBrowser:
			if (m_pListBrowserSplitter->orientation() == Vertical)
				m_pListBrowserSplitter->setOrientation(Horizontal);
		
			m_pListBrowserSplitter->moveToFirst(m_pBrowser);
			m_pListBrowserSplitter->moveToLast(m_pListsSplitter);
			m_pListBrowserSplitter->refresh();
			break;

		case RightBrowser:
			if (m_pListBrowserSplitter->orientation() == Vertical)
				m_pListBrowserSplitter->setOrientation(Horizontal);
		
			m_pListBrowserSplitter->moveToFirst(m_pListsSplitter);
			m_pListBrowserSplitter->moveToLast(m_pBrowser);
			m_pListBrowserSplitter->refresh();
			break;

		case TopBrowser:
			if (m_pListBrowserSplitter->orientation() == Horizontal)
				m_pListBrowserSplitter->setOrientation(Vertical);
		
			m_pListBrowserSplitter->moveToFirst(m_pBrowser);
			m_pListBrowserSplitter->moveToLast(m_pListsSplitter);
			m_pListBrowserSplitter->refresh();
			break;

		case BottomBrowser:
			if (m_pListBrowserSplitter->orientation() == Horizontal)
				m_pListBrowserSplitter->setOrientation(Vertical);
		
			m_pListBrowserSplitter->moveToFirst(m_pListsSplitter);
			m_pListBrowserSplitter->moveToLast(m_pBrowser);
			m_pListBrowserSplitter->refresh();
			break;

		default:
			qWarning("QmMainWindow::setBrowserPosition(): Invalid position %d.", m_Position);
			return;
			break;
	}

	m_Position = pos;
}

void QmMainWindow::setBrowserLeft()   { setBrowserPosition(LeftBrowser); }
void QmMainWindow::setBrowserRight()  { setBrowserPosition(RightBrowser); }
void QmMainWindow::setBrowserTop()    { setBrowserPosition(TopBrowser); }
void QmMainWindow::setBrowserBottom() { setBrowserPosition(BottomBrowser); }


/*!
  Hides the browser.

  \sa showBrowser()
*/

void
QmMainWindow::hideBrowser()
{
	m_pBrowser->hide();
}


/*!
  Shows the browser.

  \sa hideBrowser()
*/

void
QmMainWindow::showBrowser()
{
	m_pBrowser->show();
}

/*!
  Returns the browser object.
*/

QmBrowser *
QmMainWindow::browser() const
{
    return m_pBrowser;
}

/*!
  Returns the control bar object.
*/

QmControlBar *
QmMainWindow::controlBar() const
{
    return m_pControlBar;
}

/*!
  Shows the directory browser.
*/

void
QmMainWindow::showDirectoryBrowser()
{
    m_pBrowser->showDirectories();
}

/*!
  Shows the playlist browser.
*/

void
QmMainWindow::showPlaylistBrowser()
{
    m_pBrowser->showPlaylists();
}

/*!
  Toggles the browser on/off.

  \sa showBrowser(), hideBrowser()
*/

void
QmMainWindow::toggleBrowser()
{
	m_pControlBar->toggleBrowser();
}


/*!
  Updates the display format, currently passes it to the play list.
*/

void
QmMainWindow::setDisplayFormat()
{
    m_pPlayList->setDisplayFormat(QmConfig::instance()->getString("stuff", "display-format"),
								  QmConfig::instance()->getString("stuff", "display-format-multi"));
}


/*!
  Loads the playlist with the given name into the current playlist, places it
  below `above'.
  \param filename defaults to ~/.apollo/playlist
  \param above defaults to the bottom of the current list
*/
// void
// QmMainWindow::loadPlayList(const QString &filename = QString::null, QmPlayListItem *above = 0)
// {}

void
QmMainWindow::loadPlayListTree(
    const QString &filename)
{
    m_pBrowser->loadPlayListTree(filename);
}


/*!
  \todo
*/

void
QmMainWindow::savePlayListTree(
    const QString & /*filename = QString::null*/)
{
}


/*!
  Appends the currently selected playlist song to playfirst, if any.
*/

void
QmMainWindow::addToPlayFirst() 
{
    addToPlayFirst(0);
}


/*!
  Appends \a addme to playfirst. If \a addme is null, the current item of
  the playlist will be added if it's a non-bad song. If there is no current
  item, this function does nothing.

  \sa addToPlayFirst()
*/

void
QmMainWindow::addToPlayFirst(
    QmPlayListItem *addme)
{
    QmPlayListItem *tmp = addme != 0 ? addme : static_cast<QmPlayListItem *>(m_pPlayList->currentItem());

	if (tmp && tmp->isSong() && ! tmp->isBad())
        m_pPlayFirst->append(static_cast<QmSongItem *>(tmp));
}


/*!
  Replaces playListAtEnd() to enable/disable next button.
*/

bool
QmMainWindow::moreSongsp() const
{
    return !m_pPlayFirst->isEmpty() ||
        (!m_pPlayList->isEmpty() && (m_Loopp || m_LoopOnep || m_Randomp || !m_pPlayList->atEnd()));
}


/*!
  Replaces playListAtBeginning() to enable/disable prev button.
*/

bool
QmMainWindow::prevSongAvailablep() const
{
	if (m_Randomp || !m_pPlayFirst->isEmpty() || m_pPlayList->isEmpty())
		return false;
    
	return m_Loopp || m_LoopOnep || !m_pPlayList->atBeginning();
}


/*!
  Updates play length of currently playing song.
*/

void
QmMainWindow::updateSongLength(
    int length)
{
    QmSongItem *song = activePlayList()->currentSong();
    if (song && song->length()==0)
	{
        song->setLengthSeconds(length);
        activePlayList()->triggerUpdate();
    }
}


/*!
  Updates bitrate of currently playing song.
*/

void
QmMainWindow::updateBitrate(
    int bitrate)
{
    QmSongItem *song = activePlayList()->currentSong();
    if (song) 
        song->setBitrate(bitrate);
}


/*!
  Sets the volume to \a vol.
  \todo define range
*/

void
QmMainWindow::setVolume(
    int vol)
{
	QmMixer::instance()->setVolume(vol);
}


/*!
  Jumps \a seconds into the song. If no song is playing, this function does
  nothing.
 */

void
QmMainWindow::jumpToSeconds(
    int seconds)
{
	if (isPlaying())
		m_pPlayer->jump(seconds);
}

/*!
  Activates the search field in the minibuffer.
 */

void
QmMainWindow::activateSearch()
{
    m_pMiniBuffer->activateSearch();
}


/*!
  \todo 
*/

void
QmMainWindow::playerMessage(
	const QString &info)
{
	m_pPlayerMessage->append(info);
}


/*!
  \todo
 */

void
QmMainWindow::showPlayerMessages()
{
	m_pPlayerMessage->resize(400, 400);
	m_pPlayerMessage->show();
}
