/* qmmpg123player.cpp
 *
 * $Id: qmmpg123player.cpp,v 1.56.2.2 2002/10/11 06:39:03 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <errno.h>
#include <iostream>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>

#include <qfile.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qsocketnotifier.h>
#include <qtextstream.h>
#include <qtimer.h>

#include "qmmpg123player.h"
#include "qmconfig.h"

bool 	QmMpg123Player::s_OurTermSignal = false;
bool 	QmMpg123Player::s_FailFlag 		= false;

/**
 * @file qmmpg123player.cpp
 * @brief Interface to mpg123
*/

/*!
  \class QmMpg123Player qmmpg123player.h
  \brief Provides functionality for communicating with mpg123.

  This class is used to communicate with <a href="http://www.mpg123.com">mpg123</a>
  to play MP3 files.  The communication is done through pipes.  Apollo sends a set
  of commands to mpg123, and mpg123 provides Apollo with playing info.

  Some of the code in this class was taken from Andreas Neuhaus' IRMP3.
  
  \warning The current mpg123 version does not handle all MP3 files very well.  With
  some MP3 files, mpg123 will provide faulty playing info but play the song correctly.
  The user interface will try to detect this, but may still act a little weird.

  \warning mpg123 is very buggy with a buffer enabled.  Stop is not immediate (Pause
  seems to be, though), jump forward is not immediate (but jump backward seems to be)
  and not even quit is immediate.
*/


/*!
  Initializes the player with some variables.  The player is not started.  Call
  init() for that.
*/

QmMpg123Player::QmMpg123Player()
	: QmPlayer(0, 0),
	  m_ID3Format( "%1 - %2" ),
	  m_Playing( false ),
	  m_Paused( false ),
	  m_NewSong( false ),
	  m_GotNewSong( false ),
	  m_BufferLeftovers( 0 ),
	  m_SendFd( 0 ),
	  m_RecvFd( 0 ),
	  m_PlayerPid( 0 ),
      m_BufferPid( 0 ),
	  m_FrameCount( 0 ),
	  m_Seconds( 0 )
{
	s_OurTermSignal = false;
	s_FailFlag 		= false;
    m_pTimer 		= new QTimer( this );
    connect( m_pTimer, SIGNAL( timeout() ),
             this, SLOT( releaseTimingInformation() ) );
}


/*!
  Sends a termination signal to the mpg123 process.

  \sa terminate()
*/

QmMpg123Player::~QmMpg123Player()
{
	terminate();
}


/*!
  Terminates the player.

  \sa init()
 */

void
QmMpg123Player::terminate()
{
	s_OurTermSignal = true;
	strcpy(m_WriteBuffer, "QUIT");
	sendCommand( m_WriteBuffer );
//kill(m_PlayerPid, SIGQUIT);
//kill(m_PlayerPid, SIGTERM);
#ifdef ENABLE_MPG123_BUFFER
	kill(m_PlayerPid, SIGKILL);
#endif
	delete m_pReadNotifier;
	close(m_SendFd);
	close(m_RecvFd);
}


/*!
  Starts mpg123.  If a mpg123 process is already running, it will
  be terminated and another one restarted.

  \return -1 if failure, 0 otherwise.
  \sa terminate()
*/

int
QmMpg123Player::init()
{
	// check if player is already running
	if(m_PlayerPid != 0)
		terminate();

	// make socketpair to communicate with player
	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fd_send) < 0)
		return -1;
	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fd_recv) < 0)
		return -1;

	// handle signals
	signal(SIGPIPE, SIG_IGN);
	signal(SIGUSR1, QmMpg123Player::failHandler);

	struct sigaction *tmp = new struct sigaction;
 	tmp->sa_handler = QmMpg123Player::childHandler;
 	tmp->sa_flags = SA_NOCLDSTOP;
 	sigaction(SIGCHLD, tmp, 0);

	QmConfig *conf = QmConfig::instance();
	QString mpgpath = conf->getString( "mpg123", "path" );
	QFileInfo mpg123player(mpgpath);

 	if( ! (mpg123player.exists() && mpg123player.isExecutable()))
 	{
		QMessageBox::critical(
			0,
			QObject::tr("Apollo Error"),
			QObject::tr("Apollo tried to start the mpg123 player as:<br>"
						"<center>\"%1\"</center><br>"
						"but failed."
						"<p>"
						"Does the file exists and is it executable?"
						"<p>"
						"You can specify the mpg123 path in the configuration dialog.").arg(mpgpath));
		return -1;
	}
	
	m_PlayerPid = fork();
	
	if (m_PlayerPid == 0)
	{
		// pipe in/output through socket to parent
		dup2(fd_send[0], STDIN_FILENO);
		close(fd_send[0]);
		close(fd_send[1]);
		dup2(fd_recv[0], STDOUT_FILENO);
		dup2(fd_recv[0], STDERR_FILENO);
		close(fd_recv[0]);
		close(fd_recv[1]);

        QString buffer = conf->getString( "mpg123", "buffer" );
        int buffer_size = buffer.toInt()*1024; // Multiply to get Kb
        buffer.sprintf( "%d", buffer_size );
		int rc = 0;

        QString channels = conf->getString( "mpg123", "channels" );
        int downsample = conf->getInt( "mpg123", "downsample" );
        if ( downsample < 0 || downsample > 2 )
            downsample = 0;

		// Spawn player.  -R for starting mpg123 in remote mode (i.e. we're using
		// mpg123 through commands via pipe).  The last argument, '-', is necessary
		// otherwise mpg123 will quit immediately (dash is an alternative to an
		// URL or file name).

        const char *args[10];

        int i = 0;

		QString artsdsp = conf->getString("mpg123", "artsdsp");
		QFileInfo artsdspi(artsdsp);
		// debug: exists || warn user; executable || warn user
		if ( artsdspi.isExecutable() )
		{
			args[i++] = artsdsp.latin1(); 
			args[i++] = "-m";
			args[i++] = mpgpath.latin1();
		}
		else
			args[i++] = mpgpath.latin1();

        if ( buffer != "0" )
        {
            args[i++] = "-b";
            args[i++] = buffer.ascii();
        }
        if ( channels == "mono" )
            args[i++] = "-m";
        if ( downsample != 0 )
            args[i++] = downsample == 1 ? "-2" : "-4";
        args[i++] = "-R";
        args[i++] = "-";
        args[i++] = 0;

        rc = execvp(args[0], (char *const *)args);

		// Note that all std ouput is now piped to the parent.

//         const char *args[8];

//         int i = 0;
//         args[i++] = mpgpath.ascii();
//         if ( buffer != "0" )
//         {
//             args[i++] = "-b";
//             args[i++] = buffer.ascii();
//         }
//         if ( channels == "mono" )
//             args[i++] = "-m";
//         if ( downsample != 0 )
//             args[i++] = downsample == 1 ? "-2" : "-4";
//         args[i++] = "-R";
//         args[i++] = "-";
//         args[i++] = 0;

//         rc = execvp(mpgpath.ascii(), (char *const *)args);

		// never reached if exec was ok
		_exit(-1);
	}

	// parent (Apollo) continues here

	close(fd_send[0]);
	m_SendFd = fd_send[1];
	close(fd_recv[0]);
	m_RecvFd = fd_recv[1];
	
	m_pReadNotifier = new QSocketNotifier(m_RecvFd, QSocketNotifier::Read);
	QObject::connect(m_pReadNotifier, SIGNAL(activated(int)), this, SLOT(dataReceived()));
	
	FD_SET(m_RecvFd, &mpg123_fdset);

	s_OurTermSignal = false;

	// Check whether we have a child (player) process or not.
    int status;	
	sleep(1);
	if(waitpid(-1, &status, WNOHANG) == m_PlayerPid)
	{
		m_Message = "";
		QTextOStream ostr(&m_Message);
		ostr << "mpg123 process exited with status " << WIFEXITED(status) << ".\n";
		emit playerMessage(m_Message);
		m_PlayerPid = 0;
		return -1;
	}
	
	return 0;
}


/*!
  Plays the song \a filename.
*/

void
QmMpg123Player::play(
    const QString &filename)
{
    m_FrameCount	= -1;
    m_Seconds		= -1;
    m_TotalSeconds	= -1;
    m_NewSong  		= true;
    m_GotNewSong    = false;
    m_Filename 		= filename;
    QFileInfo		fi( m_Filename );
    m_FilePath 		= fi.absFilePath();
    m_FileExtension = fi.extension();
    sprintf(m_WriteBuffer, "LOAD %s", m_Filename.latin1());
	sendCommand( m_WriteBuffer );
    startTimer();
}


/*!
  Plays the current song from the beginning.

  \sa stop()
*/

void
QmMpg123Player::play()
{
    m_FrameCount	= -1;
    m_TotalSeconds	= m_Seconds = -1;
    m_NewSong		= true;
    m_GotNewSong	= false;
	restart();
    startTimer();
}


/*!
  Starts the timer used for emitting time values to clients.
*/

void
QmMpg123Player::startTimer()
{
    m_pTimer->start( 500, false );
}


/*!
  Stops the player.

  \sa play()
*/

void
QmMpg123Player::stop()
{
    m_FrameCount	= -1;
    m_Seconds		= m_TotalSeconds = -1;
	strcpy(m_WriteBuffer, "STOP");
	sendCommand( m_WriteBuffer );
	m_Playing		= false;
    m_pTimer->stop();
}


/*!
  Pauses the player.
*/

void
QmMpg123Player::pause()
{
	strcpy(m_WriteBuffer, "PAUSE");
	sendCommand( m_WriteBuffer );
}


/*!
  Restarts the song.

  \sa jump(int)
*/

void
QmMpg123Player::restart()
{
	jump(0);
}


/*!
  Jumps to a specific position in the song.
  @param pos seconds into song

  \sa restart()
*/

void
QmMpg123Player::jump(
	int pos )
{
	sprintf(m_WriteBuffer, "JUMP %d", pos * m_TotalFrames / m_TotalSeconds);
	sendCommand( m_WriteBuffer );
}


/*!
  Reads the response from mpg123.
*/

void
QmMpg123Player::dataReceived()
{
//	memset(m_ReadBuffer, 0, READ_BUF_SIZE);

	if(s_FailFlag)
		return;
	
    int bytes_read;
    
	do 
	{
        bytes_read = read(m_RecvFd,
                          m_ReadBuffer + m_BufferLeftovers,
                          READ_BUF_SIZE - m_BufferLeftovers );
        
		m_ReadBuffer[bytes_read + m_BufferLeftovers] = '\0';
//          cout << "$$ " << m_ReadBuffer << " $$";

        char *buffer_pointers[50];
        char **buffer_ptr = buffer_pointers;

        // Split lines into separate strings and store their pointers
        char *tmp = m_ReadBuffer;
        char *last_tmp = tmp;
		
        while ( *tmp != 0 )
        {
            if ( *tmp == '\n' )
            {
                *tmp = 0;
                *buffer_ptr = last_tmp;
                ++buffer_ptr;
                last_tmp = tmp + 1;
            }
            ++tmp;
        }

        *buffer_ptr			= 0;				// This ends the pointer list
        buffer_ptr			= buffer_pointers;	// Start from the top
        char *line_ptr		= m_ReadBuffer;
        int line_counter	= 1;
        while ( *buffer_ptr != 0 )
		{
            line_ptr = *buffer_ptr;
			
			if( line_ptr[0] == '@')
				parseCodes( line_ptr );
			else
			{
				m_Message = "";
				QTextOStream ostr(&m_Message);
				ostr << "dataReceived(): Received " << bytes_read
					 << " bytes, skipping line " << line_counter << ":\n"
					 << line_ptr << " - Filename: " << m_Filename << "\n";
				emit playerMessage(m_Message);
			}

            // Jump to next string, if any
            ++line_counter;
            ++buffer_ptr;
		}

        // There's some text without an endline
        if ( tmp <= last_tmp )
            m_BufferLeftovers = 0;
        else
        {
            // Read all available data but line still not terminated:
            if ( bytes_read != READ_BUF_SIZE - m_BufferLeftovers)
            {
				m_Message = "";
				QTextOStream ostr(&m_Message);
                ostr << m_BufferLeftovers << " bytes left over:\n"
                     << '#' << last_tmp << "#\n";
				emit playerMessage(m_Message);
            }

            int leftovers = tmp - last_tmp;
            m_BufferLeftovers = leftovers;
            
            // Copy it to the beginning of the buffer
            char *dst = m_ReadBuffer;
            for ( ; leftovers > 0; --leftovers )
            {
                *dst++ = *last_tmp++;
            }
        }
        
    } while( bytes_read == READ_BUF_SIZE - m_BufferLeftovers ); 
}


/*!
  Parses the response \a read_buffer from mpg123.
*/

void
QmMpg123Player::parseCodes(
	char *read_buffer )
{
    char code = read_buffer[1];
	
    switch(code)
    {
        case 'F':
            parseFrameInfo( read_buffer );
            break;

        case 'S':
            parseStatus( read_buffer );
            break;

        case 'I':
            parseInformation( read_buffer );
            break;

        case 'P':
            parsePlayStatus( read_buffer );
            break;

        case 'E':
            parseError( read_buffer );
            break;

        case 'B':
            sscanf(&read_buffer[3], "%d", &m_BufferPid);
            break;

        case 'R':
			if (strncmp("@R MPG123\n", read_buffer, 10) != 0)
				m_NeedNewline = true;
//			cout << "Version: " << read_buffer << endl;
            /*
			  For mpg123 0.59r, the output is:
			  
              @R MPG123

			  For mpg123 0.59s, the output is:

			  @R MPG123 0.59s-mh4
            */
            break;

        default:
			m_Message = "";
			QTextOStream ostr(&m_Message);
			ostr << "Unknown code: " << read_buffer << "\n";
			emit playerMessage(m_Message);
            break;
    }
}


/*!
  Status message during playing (frame info)
  @F <a> <b> <c> <d>
  a = framecount (int)
  b = frames left this song (int)
  c = seconds (float)
  d = seconds left (float)
  
  Examples:
	  
  @F 1249 9960 32.63 260.18
  @F 1250 9959 32.65 260.15
  @F 1251 9958 32.68 260.13
  @F 1252 9957 32.71 260.10

  The above is for mpg123 0.59r.  However, mpg123 0.59s changed and
  outputs one extra field:

  @F 20 3725 0.52 97.31 128000
  @F 21 3724 0.55 97.28 128000
  @F 22 3722 0.57 97.23 128000
  @F 23 3721 0.60 97.20 128000
*/

void
QmMpg123Player::parseFrameInfo(
	char *read_buffer )
{
	int framecount, framesleft;
	int bitrate;
	float seconds, secondsleft;
// 	sscanf(&read_buffer[3], "%d %d %f %f", &framecount,
// 		   &framesleft, &seconds, &secondsleft);
	sscanf(&read_buffer[3], "%d %d %f %f %d", &framecount,
		   &framesleft, &seconds, &secondsleft, &bitrate);

    bool release = false;

	if(m_GotNewSong)
	{
		m_Playing = true;
        m_NewSong = false;
        m_GotNewSong = false;
        m_TotalFrames = framesleft + framecount;
		emit frameCount( m_TotalFrames );
        emit secondsCount((int)( seconds + secondsleft ));
        m_TotalSeconds = (int)(seconds + secondsleft);
        release = true;
	}

	if(framesleft == 0)
		m_Playing = false;

    m_FrameCount = framecount;
    m_Seconds = (int)seconds;

	// release is true if this is the first time, if so we need to send
	// the information immediately
    if ( release )
        releaseTimingInformation();
}


/*!
  Emits the number of frames left and the number of seconds currently played.
*/

void QmMpg123Player::releaseTimingInformation()
{
	// Only emit while we are playing
    if ( m_Playing )
    {
        emit framesLeft(m_FrameCount);
        emit secondsPlaying(m_Seconds);
    }
}


/*!
  @S <a> <b> <c> <d> <e> <f> <g> <h> <i> <j> <k> <l>
  Status message after loading a song (stream info)
  a = mpeg type (string)
  b = layer (int)
  c = sampling frequency (int)
  d = mode (string)
  e = mode extension (int)
  f = framesize (int)
  g = stereo (int)
  h = copyright (int)
  i = error protection (int)
  j = emphasis (int)
  k = bitrate (int)
  l = extension (int)
*/

void
QmMpg123Player::parseStatus( char *read_buffer )
{
     char mpegType[32], mode[32];
     int layer, samplingFreq, modeExtension, frameSize, stereo, copyright, errorProtection;
     int emphasis, _bitrate, extension;

     int scanned = sscanf(read_buffer, "@S %31s %d %d %31s %d %d %d %d %d %d %d %d", mpegType, &layer,
                          &samplingFreq, mode, &modeExtension, &frameSize, &stereo, &copyright,
                          &errorProtection, &emphasis, &_bitrate, &extension);
     if (scanned == 12)
         emit bitrate(_bitrate);
     else
	 {
		 m_Message = "";
		 QTextOStream ostr(&m_Message);
		 ostr << "Failed to parse status from mpg123. Failed on: " << read_buffer << "\n";
		 emit playerMessage(m_Message);
	 }
}


/*!
  @I ID3:<a><b><c><d><e><f>
  Status message after loading a song (ID3 song info)
  a = title (exactly 30 chars)
  b = artist (exactly 30 chars)
  c = album (exactly 30 chars)
  d = year (exactly 4 chars)
  e = comment (exactly 30 chars)
  f = genre (string)

  @I <a>
  Status message after loading a song (no ID3 song info)
  a = filename without path and extension
*/

void
QmMpg123Player::parseInformation(
	char *read_buffer)
{
    m_NewSong = true;
    m_GotNewSong = true;

    if ( read_buffer[3] == 'I' &&
         read_buffer[4] == 'D' &&
         read_buffer[5] == '3' &&
         read_buffer[6] == ':' )
    {
        char *info = &read_buffer[7];
        char title[31], artist[31], album[31], year[5], comment[31];
        char *genre;
        title[30] = artist[30] = album[30] = year[4] = comment[30] = 0;

        strncpy( title, info, 30 );
        info += 30;
        strncpy( artist, info, 30 );
        info += 30;
        strncpy( album, info, 30 );
        info += 30;
        strncpy( year, info, 4 );
        info += 4;
        strncpy( comment, info, 30 );
        info += 30;
        genre = info;

        m_Title		= QString( title ).stripWhiteSpace();
        m_Artist	= QString( artist ).stripWhiteSpace();
        m_Album		= QString( album ).stripWhiteSpace();
        m_Year		= QString( year ).stripWhiteSpace();
        m_Comment	= QString( comment ).stripWhiteSpace();
        m_Genre		= QString( genre ).stripWhiteSpace();

        QString song_title = resolveTags();

        emit songTitle( song_title );
    }
    else
    {
        char *info = &read_buffer[3];

        emit songTitle( info );
    }
}


/*!
  Creates a string useable for showing the title of the current song.
  The string is created by replacing % tags in the m_ID3Format with song
  information.
*/

QString
QmMpg123Player::resolveTags()
{
    QString song_title;
    int old_pos = 0;
	
    while ( old_pos < (int)m_ID3Format.length() && old_pos >= 0 )
    {
        int pos = m_ID3Format.find( '%', old_pos );
        if ( pos != -1 )
        {
            if ( pos != (int)m_ID3Format.length() -1 )
            {
                song_title += m_ID3Format.mid( old_pos, pos - old_pos );
				QChar c = m_ID3Format[pos + 1];
				
				switch(c)
				{
				case '1':
					// Artist
                    song_title += m_Artist;
					break;

				case '2':
					// Title
                    song_title += m_Title;
					break;

				case '3':
					// Album
                    song_title += m_Album;
					break;

				case '4':
					// Year
                    song_title += m_Year;
					break;

				case '5':
					// Comment
                    song_title += m_Comment;
					break;

				case '6':
					// Genre
                    song_title += m_Genre;
					break;

				case '7':
					// File name
                    song_title += m_Filename;
					break;

				case '8':
					// File path
                    song_title += m_FilePath;
					break;

				case '9':
					// File extension
                    song_title += m_FileExtension;
					break;

				default:
                    song_title += QString( "%" ) + c;
				}
				
                old_pos = pos + 2;
            }
            else
            {
                song_title += m_ID3Format.mid( old_pos );
                old_pos = -1;
            }
        }
        else
        {
            song_title += m_ID3Format.mid( old_pos );
            old_pos = pos;
        }
    }
    return song_title;
}


/*!
  @P <a>
  Playing status
  a = 0: playing stopped
  a = 1: playing paused
  a = 2: playing unpaused
*/

void
QmMpg123Player::parsePlayStatus(
	char *read_buffer )
{
	int status;
	sscanf(&read_buffer[3], "%d", &status);

	switch(status)
	{
	case 0:
		emit playingStopped();
		break;

	case 1: //  		cout << "paused" << endl; 
		break;

	case 2: //  		cout << "unpaused" << endl;
		break;

	default:
		m_Message = "";
		QTextOStream ostr(&m_Message);
		ostr << "Unknown play status: " << read_buffer << "\n";
		emit playerMessage(m_Message);
		break;
	}
}


/*!
  @E <a>
  An error occured
  Errors may be also reported by mpg123 through
  stderr (without @E)
  a = error message (string)
*/

void
QmMpg123Player::parseError(
	char *read_buffer )
{
	m_Message = "";
	QTextOStream ostr(&m_Message);
	ostr << "Error: " << read_buffer << "\n";
	emit playerMessage(m_Message);
}


/*!
  \return True if playing, false otherwise.
 */

bool
QmMpg123Player::isPlaying() const
{
	return m_Playing;
}


/*!
  \return True if paused, false otherwise.
 */

bool
QmMpg123Player::isPaused() const
{
	return m_Paused;
}


/*!
  Sends the string \a write_buffer to the mpg123 process.  If
  the command was not sent successfully, a dialogbox will be
  shown.  This kind of failure will typically happen when the
  mpg123 process crashed, and the pipe is broken.

  \return True if successful, false otherwise.
*/

bool
QmMpg123Player::sendCommand(
	char *write_buffer )
{
	int len = strlen(write_buffer);	
	if (m_NeedNewline) 
		write_buffer[len] = '\n';
	int rc = write(m_SendFd, write_buffer, len+1);

	if(rc > -1)
		return true;
	else
	{
		rc = QMessageBox::warning(
			0,
			QObject::tr("Apollo Error"),
			QObject::tr("Apollo failed to send the command to<br>"
						"the MP3 player.  This typically means<br>"
						"that the MP3 player process is no longer<br>"
						"working correctly and needs to be restarted.<br>"
						"It might just have been a temporary problem<br>"
						"and you can try again."),
			QObject::tr("&Restart player"),
			QObject::tr("&Do nothing"));

		if(rc == 0)
		{
			init();
			return sendCommand(write_buffer);
		}
	}
	return false;
}



/*!
  Signal handler for mpg123 process.
*/

/*
  According to the C++ FAQ [1], it is not a good idea to pass
  a pointer to a member function to a C function such as signal()
  for function callbacks.  Instead, we will have to rely on a
  static member function (which acts just as a regular C function).

  [1] See 30.2 at http://www.cs.uu.nl/wais/html/na-dir/C++-faq/part9.html
*/

void
QmMpg123Player::childHandler(
	int sig)
{
	// Catch signal of exiting player.

//	signal(SIGCHLD, 0);
//	cout << "QmMpg123Player::childHandler(): Signal from mpg123" << endl;

	if( ! s_OurTermSignal)
	{
		QMessageBox::critical(
			0,
			QObject::tr("Apollo Error"),
			QObject::tr("Apollo has detected that the MP3 player<br>"
						"process failed (signal %1).  Apollo cannot function<br>"
						"without it.  You can try to restart the<br>"
						"player from the Options menu.<br>"
						"If the particular song you were trying to<br>"
						"play keeps continuing to fail the player<br>"
						"you can mark it as 'bad' from the Playlist menu.").arg(sig));
	}

	/*
    int status;
	
	if(waitpid(-1, &status, WNOHANG) == m_PlayerPid)
	{
		cout << "mpg123 process exited with code " << WEXITSTATUS(status) << endl;
		m_PlayerPid = 0;
	}
	*/
}


/*!
  Signal handler for mpg123 process.
*/

void
QmMpg123Player::failHandler(
	int /*sig*/)
{
//	signal(SIGUSR1, 0);
	
	QMessageBox::critical(
		0,
		QObject::tr("Apollo Error"),
		QObject::tr("Apollo has detected that the MP3 player<br>"
					"is not running.  Apollo cannot function<br>"
					"without it.  You can try to restart the<br>"
					"player from the Options menu."));
}


/*!
  Sets the fail flag to \a b.

  Scenario:  Apollo starts -> forks -> apollo(1), apollo(2) -> execute
  mpg123 over apollo(1) -> apollo(2) tries to communicate with mpg123 (apollo(1))
  -> mpg123 fails to execute -> process writes to cerr -> apollo(2) reads from
  cerr; thinks it's mpg123 -> problem!

  By settings the (static) fail flag to true, the input will be ignored.
*/

void
QmMpg123Player::setFailFlag(
	bool b)
{
	s_FailFlag = b;
}
