/* qmplaylist.cpp
 *
 * $Id: qmplaylist.cpp,v 1.106.2.3 2002/10/11 06:39:03 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmplaylist.h"

#include "message.h"
#include "qmconfig.h"
#include "qmdiritem.h"
#include "qmplaylistformat.h"
#include "qmpropertydialog.h"
#include "qmsonginfopropertypage.h"
#include "qmsongitem.h"
#include "util.h"
#include "qcstring.h"

#include <algorithm>
#include <iostream>
#include <time.h>
#include <cassert>

#include <qdom.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qstringlist.h>
#include <qtextstream.h>

int QmPlayList::s_PlayListCounter = 0;

/* Disables false warning in Visual Studio about dynamic_cast */

#ifdef _WS_WIN_
#pragma warning(disable: 4541)
#endif

/*!
  \file qmplaylist.cpp
  \brief The playlist widget.  Duh.
 */

/*!
  \class QmPlayList qmplaylist.h
  \brief Represents the playlist.

  This widget is the playlist in Apollo.  It provides functionality
  for loading and saving the playlist, as well as traversing and modifying
  it.

  In order to understand the class, a few definitions need to be pointed
  out:

  <ul>
  <li> Selected - An item that is selected is drawn with a filled background.
  Any number of items can be selected, including none.  Notice that if a folder
  is selected, all its children are implicitly selected too.  This is not shown
  visually.  See selectedItems() for more information. </li>

  <li> Current - The text of the current item, if any, is drawn in a
  different color than the other items.  The current item represents the
  playing item, or the item that is paused or will be played if `Play' is
  clicked. If there's no songs to be played, there won't be current
  item. </li>

  <li> Bad - Any number of items can be marked as bad.  A bad item cannot
  be played expect for one case: A playing item can be marked as bad while
  being played.  Playing will not stop. </li> </ul>

  Some methods, such as nextSong, and prevSong, are not const because there
  is no const QListView iterator.
*/

/*! \fn void QmPlayList::disableNext()
  Emitted when there's no songs below the current item, this allows
  clients of this class to disable the next button.
*/

/*! \fn void QmPlayList::enableNext()
  Emitted when there are one or more songs below the current item, this allows
  clients of this class to enable the next button.
*/

/*! \fn void QmPlayList::disablePrev()
  Emitted when there's no songs above the current item, this allows
  clients of this class to disable the previous button.
*/

/*! \fn void QmPlayList::enablePrev()
  Emitted when there's one or more songs above the current item, this allows
  clients of this class to disable the previous button.
*/

/*!
  Constructs a playlist widget.
*/

// It is necessary to keep a 'play item' variable (see above) so we can
// remove its playing status when another item is selected.
// currentItem() cannot be used as it will already be updated with
// the new song.

QmPlayList::QmPlayList(
    QWidget *parent,
    const char *name)
    : QmListView(parent, name),
      m_Dirty(false),
      m_SongIndex(),
      m_SongIndexDirty(true),
	  m_DeletingItems(false),
      m_PlayLength(0),
      m_PlayLengthNotCounted(0),
	  m_pCurrentSong(0),
      m_Filename(""),
	  m_pSongInfoPage(0),
	  m_pPropertyDlg(0)
{
	setMultiSelection(true);
	setSelectionMode(Extended);
	setAcceptDrops(true);
	setDragAutoScroll(true);
    setMinimumWidth(m_LengthWidth);
    setHScrollBarMode(AlwaysOff);
	reorganizable(true);

    m_pMenu = new QPopupMenu(this);

	connect(this, SIGNAL(mouseButtonClicked(int, QListViewItem*, const QPoint &, int)),
			this, SLOT(mouseClick(int, QListViewItem*, const QPoint &, int)));
	connect(this, SIGNAL(selectionChanged()),
			this, SLOT(newSelection()));
	connect(&m_Delay, SIGNAL(timeout()),
            this, SLOT(newSelectionHandler()));
}


/*!

 */

QmPlayList::~QmPlayList()
{
}


/*!
  Initialize the popup menu
 */

void
QmPlayList::initMenu()
{
    m_ContinueHereId = m_pMenu->insertItem(
        tr("Continue Here"),
        this,
        SLOT(continueHere()));
    m_PlayFirstId = m_pMenu->insertItem(
        tr("Play First (mid click)"),
        QmMainWindow::mainwin,
        SLOT(addToPlayFirst()));
    m_FlattenId = m_pMenu->insertItem(
        tr("Flatten"),
        this,
        SLOT(flatten()));
     m_ReGroupId = m_pMenu->insertItem(
         tr("Regroup"),
         this,
         SLOT(reGroup()));
	m_pMenu->insertSeparator();
    m_pMenu->insertItem(
        tr("Expand First"),
        this,
        SLOT(expandFirstLevel()));
    m_pMenu->insertItem(
        tr("Expand All"),
        this,
        SLOT(expandAll()));
    m_pMenu->insertItem(
        tr("Collapse All"),
        this,
        SLOT(collapseAll()));
	m_pMenu->insertSeparator();
    m_pMenu->insertItem(
        tr("Update song info for all songs"),
        this,
        SLOT(readSongInfo()));
    m_GenerateId = m_pMenu->insertItem(
        tr("Generate a playlist for each artist"),
        this,
        SLOT(generatePlayListForEachArtistAndAlbum(int)));
    m_pMenu->setItemParameter(m_GenerateId, 0);
    
    m_GenerateTreeId = m_pMenu->insertItem(
        tr("Generate a playlist for each artist - and playlist tree"),
        this,
        SLOT(generatePlayListForEachArtistAndAlbum(int)));
    m_pMenu->setItemParameter(m_GenerateTreeId, 1);

	m_pMenu->insertItem(
        "Clear",
        this,
        SLOT(clear()));
	m_pMenu->insertSeparator();
	m_pMenu->insertItem(
        tr("Properties..."),
        this,
        SLOT(showItemProperties()));
}


/*!
  Adds the playlist \a filename to the list.  The placement of the inserted
  playlist is related to \a reference.  If \a reference is null, \a filename
  will be appended if \a below is true or prepended if \a below is false.

  If \a reference is not null, \a filename will be placed below \a reference
  if \a below is true or above if \a below is false.

  \return True if successful, false otherwise.
 */

bool
QmPlayList::add(
	const QString & /*filename*/,
	QListViewItem * /*reference*/,
	bool /*below*/)
{
	// This function will be used for drag'n drop but is more general than
	// the current add() and is therefore meant to replace that one.
    
	return false;
}


/*!
  \returns the total play length of the list in seconds.
*/

long
QmPlayList::playLength() 
{
	updateSongIndex();
    return m_PlayLength;
}


/*!
  \returns the number of songs in the list with unknown length.  Songs
  that haven't been played yet and doesn't have the lenght info in the
  playlist.  (Consider adding a "stat all songs" option in the menu.)
 */

int
QmPlayList::playLengthNotCounted() 
{
	updateSongIndex();
    return m_PlayLengthNotCounted;
}


/*!
  \returns number of songs in the playlist.
*/

int QmPlayList::noSongs() 
{
	updateSongIndex();
    return m_SongIndex.size();
}


/*!
  Increases the global counter.  The counter is used for sorting the songs
  in the playlist.

  \return The previous counter value, i.e. before it was increased.
*/

int
QmPlayList::count()
{
    return s_PlayListCounter++;
}


/*!
  Wraps around to the beginning if \param wrap is true.
  \return The first non-bad song following the current song, or 0.
  \sa prevSong(bool)
*/

QmSongItem*
QmPlayList::nextSong(bool wrap) 
{
	updateSongIndex();
	if (m_SongIndex.empty())
		return 0;
	
	SongIndexIterator current = m_pCurrentSong ? std::find(m_SongIndex.begin(), m_SongIndex.end(), m_pCurrentSong) : m_SongIndex.begin();
	if (current == m_SongIndex.end())
		current = m_SongIndex.begin();

	if (*current == m_SongIndex.back())
		return wrap ? m_SongIndex.front() : 0;

	++current;
	return *current;
}


/*!
  Wraps around to the end if \param wrap is true.
  \return The first non-bad song preceding the current song, or 0.
  \sa nextSong(bool)
*/

QmSongItem*
QmPlayList::prevSong(bool wrap) 
{
	updateSongIndex();
	if (m_SongIndex.empty())
		return 0;
	
	SongIndexIterator current = m_pCurrentSong ? std::find(m_SongIndex.begin(), m_SongIndex.end(), m_pCurrentSong) : m_SongIndex.begin();
	if (current == m_SongIndex.end())
		current = m_SongIndex.begin();

	if (*current == m_SongIndex.front())
		return wrap ? m_SongIndex.back() : 0;
	--current;
	return *current;
}


/*!
  Wraps around to the beginning if \param wrap is true.
  \return The first non-bad song following the current song, or 0.
  \sa prevSong(QmSongItem*, bool)
*/

QmSongItem*
QmPlayList::nextSong(
	QmSongItem *curSong,
	bool wrap) 
{
    if (isEmpty())
		return 0;
	
	SongIndexIterator current = m_pCurrentSong ? std::find(m_SongIndex.begin(), m_SongIndex.end(), curSong) : m_SongIndex.begin();
	if (current == m_SongIndex.end())
		current = m_SongIndex.begin();
	
	if (*current == m_SongIndex.back())
		return wrap ? m_SongIndex.front() : 0;
	++current;
	return *current;
}


/*!
  Searches for a song with the given filepath.
  
  \return the song if it's found, otherwize 0
 */

QmSongItem* 
QmPlayList::findSong(
    const QString &filePath) 
{
	updateSongIndex();
	for (int i=m_SongIndex.size()-1; i>=0; --i)
		if (filePath == m_SongIndex[i]->filePath()) 
			return m_SongIndex[i];
    return 0;
}
           

/*!
  Wraps around to the end if \param wrap is true.
  \return The first non-bad song preceding the \param curSong song, or 0.
  \sa nextSong(QmSongItem*, bool)
*/

QmSongItem*
QmPlayList::prevSong(
	QmSongItem *curSong,
	bool wrap) 
{
	updateSongIndex();
	SongIndexIterator current = m_pCurrentSong ? std::find(m_SongIndex.begin(), m_SongIndex.end(), curSong) : m_SongIndex.begin();
	if (current == m_SongIndex.end())
		current = m_SongIndex.begin();

	if (*current == m_SongIndex.front())
		return wrap ? m_SongIndex.back() : 0;
	--current;
	return *current;
}


/*!
  \return A random song.
*/

QmSongItem*
QmPlayList::randomSong() 
{
	updateSongIndex();
    return m_SongIndex[static_cast<int>(m_SongIndex.size()*(double)rand()/RAND_MAX)];
}


/*!
  If \a item is a non-bad song item, give it playing status and remove the playing
  status from the song currently having it, if any.  If \a item is marked as
  a bad song or is not a song, this function will do nothing.

  This function also emits enableNext(), disableNext(), enablePrev() and
  disablePrev() signals when appropriate.
*/

void
QmPlayList::setCurrent(
	QmSongItem *song)
{
    CHECK_PTR(song);

	// rk: log this
    if(song->isBad())
        return;

	if(m_pCurrentSong)
		m_pCurrentSong->setPlaying(false);

	// rk: disableNext() and enableNext() are currently not connected
	updateSongIndex();	
	SongIndexIterator cur = std::find(m_SongIndex.begin(), m_SongIndex.end(), song);
	assert(cur != m_SongIndex.end());
	if (*cur == m_SongIndex.back() && !QmMainWindow::mainwin->loopMode())
		emit disableNext();
	else
		emit enableNext();

	if(cur == m_SongIndex.begin() && !QmMainWindow::mainwin->loopMode())
		emit disablePrev();
	else
		emit enablePrev();
	
	song->setPlaying(true);
	m_pCurrentSong = song;
    ensureItemVisible(song);
}


/*!
  \return The current song, or 0 if there is no non-bad songs.
  \todo I don't like this function.  The only reason for its existance is to save the
  name of the playing item when quitting.
*/

QmSongItem*
QmPlayList::currentSong() 
{
	updateSongIndex();

	if (m_pCurrentSong == 0)
	{
		if (m_SongIndex.empty())
			return 0;
		else
			setCurrent(m_SongIndex.front());
	}
	
	return m_pCurrentSong;
}


/*!
  \returns Song with index \a index, or playing song if \a index is -1 (default).
  0 if no song playing or invalid index.
*/

QmSongItem*
QmPlayList::song(
    int index)
{
	if (index == -1)
		return m_pCurrentSong;
	
	updateSongIndex();
	return index >= static_cast<int>(m_SongIndex.size()) || index<0 ? 0 : m_SongIndex[index];
}


/*!
  Clears the playlist.  The playing item will be cleared.
  \warning This should only be called directly by QmMainWindow.
*/

void
QmPlayList::clear()
{
	m_pCurrentSong = 0;
	QListView::clear();
    m_Dirty = false;
    m_SongIndexDirty = true;
}


/*!
  \return True if \a item is selected directly \e or if a parent of the item
  is selected.  False otherwise or if \a item is null.
*/

bool
QmPlayList::isSelected(
	QListViewItem *item) const
{
	if(item == 0)
		return false;
	
	if(item->isSelected())
		return true;

	for (QListViewItem *parent = item->parent(); parent; parent = parent->parent())
		if(parent->isSelected())
			return true;

	return false;
}


/*!
  Adjust name column width on viewport resize.
*/

void
QmPlayList::viewportResizeEvent(
    QResizeEvent *e)
{
    QmListView::viewportResizeEvent( e );
    setColumnWidth(0, e->size().width() - columnWidth(1));
//     QmListView::setGeometry(x,y,w,h);
//     QSize s = viewportSize(w, h);

// 	if(columns() == 1)
// 		setColumnWidth( 0, s.width());
// 	else
// 	{
// 		setColumnWidth( 0, s.width() - m_LengthWidth);
// 		setColumnWidth( 1, m_LengthWidth );
// 	}

//     triggerUpdate();
}


/*!
  Remove(s) the selected item(s), if any.

  If the playing item is selected either directly or indirectly (i.e. a parent is selected), the
  next song to play will be returned.  If there is no next song to play or there is no need to
  play a new song, null will be returned.

  If there is no next song to play AND the current item is the playing item, \a stop_playing will
  be set to true.  The callee should stop playing as the playing item has been deleted, but
  there was no other song to return for playing.  \a stop_playing will be false in all other
  cases.

  \return The new song to play if any, otherwise null.
*/

QmSongItem*
QmPlayList::removeSelected(
	bool *stop_playing)
{
	QmSongItem *ret = 0;
	m_DeletingItems = true;
	*stop_playing = false;
	
	// If the playing item is selected, either directly or indirectly, find the new
	// song to play.

	if(m_pCurrentSong && isSelected( m_pCurrentSong ))
	{
        // Search forward (down)
		SongIndexIterator i;
		for(i = std::find(m_SongIndex.begin(), m_SongIndex.end(), m_pCurrentSong); i != m_SongIndex.end(); ++i)
        {
			if ( ! isSelected(*i))
				break;
        }

        // If we didn't find a song in the forward search, search backward (up)
		if(i == m_SongIndex.end())
        {
			for(i = std::find(m_SongIndex.begin(), m_SongIndex.end(), m_pCurrentSong); i != m_SongIndex.begin(); --i)
            {
				if ( ! isSelected(*i))
					break;
            }
        }
		if (isSelected(*i))
        {
            m_pCurrentSong = 0;
            *stop_playing = true;
        }
		else {
			setCurrent(*i);
			ret = m_pCurrentSong;
		}
	}

	// Delete selected items.
	QList<QListViewItem> *selected = selectedItems();
	selected->setAutoDelete(true);
	delete selected;

	m_DeletingItems = false;
    setDirty();
	updateSongIndex(true);	
	return ret;
}


/*!
  \return wheter the song playing or would be playing is the first non-bad song in the list
*/

bool
QmPlayList::atBeginning() 
{
	// rk: what to do with empty list? throw exception, write a bug message in the bug log?
	// return true for now.
	if (m_SongIndex.empty())
		return true;
	
    if(m_pCurrentSong == 0)
		setCurrent(m_SongIndex.front());
	
	return m_pCurrentSong == m_SongIndex[0];
}


/*!
  \return wheter the song playing or would be playing is the last non-bad song in the list
*/

bool
QmPlayList::atEnd() 
{
	updateSongIndex(); 
	if (m_SongIndex.empty())
		return true;

    if(m_pCurrentSong == 0)
		setCurrent(m_SongIndex.front()); 

	return m_pCurrentSong == m_SongIndex.back();
}


/*!
  \return The position of the playing or would be playing item, or -1 if empty
*/

int
QmPlayList::position()
{
	updateSongIndex();
	if (m_SongIndex.empty())
		return -1;

	if (m_pCurrentSong == 0)
		setCurrent(m_SongIndex.front()); 
			
	return std::find(m_SongIndex.begin(), m_SongIndex.end(), m_pCurrentSong) - m_SongIndex.begin();
}


/*!
  Deletes the song at the specified position (0-based depth first).
*/

void
QmPlayList::deletePosition(
    uint pos)
{
	updateSongIndex();
	if (m_SongIndex.empty())
		return;
	
	if (pos < 0 || m_SongIndex.size() <= pos)
		return;

	if (m_SongIndex[pos] == m_pCurrentSong)
    {
		if (m_SongIndex.size() == 1)
		{
			if (m_pCurrentSong)
				m_pCurrentSong->setPlaying(false);
			m_pCurrentSong = 0;
		}
		else 
			setCurrent(m_SongIndex[pos+1 % m_SongIndex.size()]); 
	}

	delete m_SongIndex[pos];
}


/*!
  \return True if empty, false otherwise.
*/

bool
QmPlayList::isEmpty() const
{
	return firstChild() == 0;
}


/*!
  Appends the directory (path) \a dir to the playlist.  If,
  for some reason, \a dir cannot be read, this function fails
  silently.

  \sa prependDir(const QString&), prependList(const QString&)
*/

void
QmPlayList::appendDir(
	const QString &dir)
{
	if(dir.isEmpty())
	{
		qWarning("QmPlayList::appendDir(): Filename is empty.\n");
		return;
	}
    
    int count = 0;
	(void) new QmDirItem(this, lastChild(), dir, count, true);
    setDirty();	
	triggerUpdate();
    QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, "Loaded %d songs", count));
}


/*!
  Prepends the directory (path) \a dir to the playlist.  If,
  for some reason, \a dir cannot be read, this function fails
  silently.

  \sa appendDir(const QString&), appendList(const QString&)
 */

void
QmPlayList::prependDir(
	const QString &dir)
{
	if(dir.isEmpty())
	{
		qWarning("BUG: QmPlayList::prependDir(): Filename is empty.\n");
		return;
	}
    
    int count = 0;
	(void) new QmDirItem(this, 0, dir, count, true);
    setDirty();	
	triggerUpdate();
    QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, "Loaded %d songs", count));
}


/*!
  Appends the playlist \a filename to the playlist.  If,
  for some reason, \a filename cannot be read, this function fails
  silently.

  \sa prependList(const QString&), prependDir(const QString&)
 */

void
QmPlayList::appendList(
	const QString &filename)
{
	if(filename.isEmpty())
	{
		qWarning("BUG: QmPlayList::appendList(): Filename is empty.\n");
		return;
	}

	bool dirty = !isEmpty();
	loadPlayList(filename, dynamic_cast<QmPlayListItem*>(lastChild()));
	triggerUpdate();
	if (dirty)
		setDirty();
}


/*!
  Prepends the playlist \a filename to the playlist.  If,
  for some reason, \a filename cannot be read, this function fails
  silently.

  \sa appendList(const QString&), appendDir(const QString&)
 */

void
QmPlayList::prependList(
	const QString &filename)
{
	if(filename.isEmpty())
	{
		qWarning("BUG: QmPlayList::prependList(): Filename is empty.\n");
		return;
	}

	bool dirty = !isEmpty();
	loadPlayList(filename);
	triggerUpdate();
	if (dirty)
		setDirty();
}


/*!
  Appends the song \a filename to the playlist.  If,
  for some reason, \a filename cannot be read, this function fails
  silently.

  \sa prependList(const QString&), prependDir(const QString&)
 */

void
QmPlayList::appendSong(
	const QString &filename)
{
	if(filename.isEmpty())
	{
		qWarning("BUG: QmPlayList::appendSong(): Filename is empty.\n");
		return;
	}
    
	triggerUpdate();
	(void) new QmSongItem(this, lastChild(), filename);
	setDirty();	
}


/*!
  Prepends the song \a filename to the playlist.  If,
  for some reason, \a filename cannot be read, this function fails
  silently.

  \sa appendList(const QString&), appendDir(const QString&)
 */

void
QmPlayList::prependSong(
	const QString &filename)
{
	if(filename.isEmpty())
	{
		qWarning("BUG: QmPlayList::prependSong(): Filename is empty.\n");
		return;
	}
	
	triggerUpdate();
	(void) new QmSongItem(this, 0, filename);
	setDirty();	
}


/*!
  Saves the playlist to \a filename.  If \a filename is QString::null, the
  the default playlist (stored in the Apollo configuration directory) will be
  overwritten.

  This function will silently fail if the file cannot be saved.
 */

// rk: use saveAs*(), but keep this here for now
// void
// QmPlayList::savePlayList(
// 	const QString &filename)
// {
//     if(filename.isEmpty())
// 		qWarning("QmPlayList::savePlayList(): Filename is empty.\n");
//     else {
//         QmPlayListFormat *f = QmPlayListFormat::getFormat(filename, true);
//         f->save(this, filename);
//         delete f;
//         m_Dirty = false;
//     }
// }


/*!
  Sets the display format for each item on the playlist.
*/

void
QmPlayList::setDisplayFormat(const QString &format, const QString &formatMulti) 
{
	const char* cformat = format.latin1();
	const char* cformatMulti = formatMulti.latin1();
    for (QListViewItemIterator it(this); it.current(); ++it)
        static_cast<QmPlayListItem*>(it.current())->setDisplayFormat(cformat, cformatMulti);
}


/*!
  Loads the playlist \a filename.  If \a filename is QString::null, the
  default playlist (stored in the Apollo configuration directory) will be
  loaded.

  The \a above argument specifies where the new items will be placed.  If
  \a above is null, the will be placed at the start of the list.

  This function will silently fail if something is wrong with the playlist
  file.
 */

QmPlayListItem*
QmPlayList::loadPlayList(
	const QString &filename,
	QmPlayListItem *above)
{
    bool empty = isEmpty();
	if(filename.isEmpty())
	{
		QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                        "Error: tried to load playlist without a filename"));
		return 0;
	}
	
	QmPlayListFormat *f = QmPlayListFormat::getFormat(filename);
	QmPlayListItem *ret = f->load(this, filename, above);
    
	if (empty)
	{
        m_Filename = filename;
        m_Dirty = false;
		m_SongIndexDirty = true;
    }
	else
        setDirty();
    
	delete f;
    reCalculate();
	triggerUpdate();
	return ret;
}


/*!
  Marks the selected item as the next one to be played.
*/

void
QmPlayList::continueHere()
{
    QmMainWindow::mainwin->continueHere(dynamic_cast<QmPlayListItem *>(currentItem()));
}


/*!
  \todo just as hook for later (following qmplaylistbrowser pattern)
*/

void
QmPlayList::showPopup(QListViewItem *current, const QPoint &pos, int)
{
    m_pMenu->popup(pos);
}


/*!
  Called when either of the mouse buttons are clicked.

  \sa showPopup(QListViewItem*, const QPoint&, int)
 */

void
QmPlayList::mouseClick(
    int button,
    QListViewItem * item,
    const QPoint & pos,
    int c )
{
    if (button & Qt::MidButton)
        QmMainWindow::mainwin->addToPlayFirst();
    else if (button & Qt::RightButton)
        showPopup(item, pos, c);
//    else if ( button & Qt::LeftButton )
}


/*!

 */

void
QmPlayList::dragEnterEvent(
    QDragEnterEvent *event)
{
	event->accept(canDecode(event) || QmListView::canDecode(event));
}


/*!

 */

void
QmPlayList::dropEvent(
    QDropEvent *event)
{
    if ( ! QUriDrag::canDecode(event))
        return QmListView::dropEvent(event);

    QStringList files;
    if ( ! QUriDrag::decodeLocalFiles(event, files))
        return;

    QFileInfo fileinfo;

    int count = 0;
	QListViewItem *dropAt = 0;
    
	if (childCount() > 1) // dropindicator is 1 child
		dropAt = dynamic_cast<QListViewItem*>(dropPos());

    for (QStringList::Iterator it = files.begin(); it != files.end(); it++)
	{
        fileinfo.setFile(*it);
        if (fileinfo.isFile())
		{
			QString ext = fileinfo.fileName().right(3).lower();
            
			if (ext == "mp3")
            {
				if (dropAt && dropAt->depth())
					dropAt = new QmSongItem(reinterpret_cast<QmPlayListItem*>(dropAt->parent()),
                                            dropAt, *it);
				else
					dropAt = new QmSongItem(this, dropAt, *it);
            }
			else
            {
				dropAt = loadPlayList(fileinfo.filePath(),
									  reinterpret_cast<QmPlayListItem*>(dropAt));
            }
		}
        else
        {
			if (dropAt && dropAt->depth())
            {
				dropAt = new QmDirItem(reinterpret_cast<QmDirItem*>(dropAt->parent()),
                                       dropAt, *it, count, true);
            }
			else
            {
				dropAt = new QmDirItem(this, dropAt, *it, count, true);
            }
        }
    }

    setDirty();    
	QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, "Dropped %d songs", count));
    stopDragging();
}


/*!
  \return True if the playlist has been changed since last saved, false otherwise/
  \sa setDirty()
*/

bool
QmPlayList::isDirty() const 
{
    return m_Dirty;
}


/*!
  Sets the dirty flag.

  A function is used to provide a place to put stuff that needs to be done
  when the playlist is changed.

  \sa isDirty()
 */

void
QmPlayList::setDirty() 
{
    m_SongIndexDirty = m_Dirty = true;
}


/*!
  Remove directories so the play-tree becomes a play-list.
*/

void
QmPlayList::flatten() 
{
	setUpdatesEnabled(false);
	m_DeletingItems = true;
    // ensure songs in index
	updateSongIndex();

    // remove songs from list so they're not deleted with the directories
    // takeItem is needlessly expensive for this, consider reimplementing it
    QListViewItem *parent;
	for (uint i = 0; i < m_SongIndex.size(); ++i)
    {
		if ((parent = m_SongIndex[i]->parent()) != 0) 
			parent->takeItem(m_SongIndex[i]);
		else 
			takeItem(m_SongIndex[i]);
    }

	QListView::clear();

    // add songs from index
    QListView::insertItem(m_SongIndex[0]);
    for (uint i = 1; i< m_SongIndex.size(); ++i)
        insertItem( 0, m_SongIndex[i], m_SongIndex[i-1] );

	m_DeletingItems = false;
	setUpdatesEnabled(true);
	triggerUpdate();
}


/*!
  Searches for the string \a s (case insensitive) and highlights the item, if found.

  The last search is remembered. If the same string is passed in twice (or more), the
  search continues from where the last match was found. Otherwise, the search starts
  from the top.
*/

void
QmPlayList::search(
    const QString& s)
{
    static QString prev_search = QString::null;
    static QListViewItem *last_found = 0;
    QListViewItem *start_item = 0;

    if(prev_search == s)
    {
        if(last_found)
            start_item = last_found->itemBelow();

        if(start_item == 0)
            start_item = firstChild();
    }
    else
    {
        prev_search = s;
        start_item = firstChild();
    }

    for(QListViewItemIterator it(start_item); it.current() != 0; ++it)
    {
        if(it.current()->text(0).contains(s, false))
        {
            last_found = it.current();
            ensureItemVisible(it.current());
            QmPlayListItem::setMatchItem(reinterpret_cast<QmPlayListItem*>(it.current()));
            return;
        }
    }

    QmPlayListItem::setMatchItem(0);
}


/*!
  Re-arranges the playlist. Currently makes nodes for Artists with sub-nodes for
  album; if something else makes sense, pass the QmSongItem attributes to
  the compareSongs constructor.
 */

void
QmPlayList::reGroup() 
{
	updateSongIndex();
    if (m_SongIndex.empty())
        return;

	m_DeletingItems = true;
	setUpdatesEnabled(false);	

    std::sort(m_SongIndex.begin(), m_SongIndex.end(),
			  compareSongs<>(&QmSongItem::m_Artist, &QmSongItem::m_Album, &QmSongItem::m_CdPosition));

    // remove songs from widget
    QListViewItem *parent;
	for (SongIndexIterator i=m_SongIndex.begin(); i!=m_SongIndex.end(); ++i)
    {
		if ((parent=(*i)->parent()) != 0) 
			parent->takeItem(*i);
		else 
			takeItem(*i);
    }
    QListView::clear();
	
    // add songs, creating groups
    QString curArtist = m_SongIndex[0]->artist();
    QString curAlbum = m_SongIndex[0]->album();
    QmDirItem *artist = new QmDirItem(this, 0, curArtist);
    QmDirItem *album = new QmDirItem(artist, 0, curAlbum);
    album->insertItem(m_SongIndex[0]);
	QmSongItem *prev = m_SongIndex[0];			// insertItem doesn't take a `after' or `append' parameter, doh.

	std::vector<QmSongItem*> various;
    for (uint i = 1; i < m_SongIndex.size(); i++)
    {
		if (m_SongIndex[i]->multiArtistp())
		{
			various.push_back(m_SongIndex[i]);
			continue;
		}
		
        if (m_SongIndex[i]->album() != curAlbum)
		{
            if (m_SongIndex[i]->artist() != curArtist)
			{
                curArtist = m_SongIndex[i]->artist();
                artist = new QmDirItem(this, artist, curArtist);
				prev = 0;
            }

            curAlbum = m_SongIndex[i]->album();
            album = new QmDirItem(artist, 0, curAlbum);			
			prev = 0;
        }

        album->insertItem(m_SongIndex[i]);
        
		if (prev)
			m_SongIndex[i]->moveItem(prev);
        
		prev = m_SongIndex[i];
    }
	
	if (!various.empty())
	{
		QString tmp(tr("Various"));
		artist = new QmDirItem(this, 0, tmp);
		
		std::sort(various.begin(), various.end(), 
				  compareSongs<>(&QmSongItem::m_Album, &QmSongItem::m_CdPosition));

		QString albumName;
		for (SongIndexIteratorR i=various.rbegin(); i!=various.rend(); ++i)
		{
			if ((*i)->album() != albumName)
			{
				albumName = (*i)->album();
				album = new QmDirItem(artist, 0, albumName);
			}
			album->insertItem(*i);
		}
	}

    reCalculate();
    updateColumnWidths();

	m_DeletingItems = false;
	setUpdatesEnabled(true); 
	triggerUpdate();
}


/*!
  Expands first level directories and closes all others.

  \sa expandAll()
*/

void
QmPlayList::expandFirstLevel()
{
    collapseAll();
    
    for (QListViewItem *i = firstChild(); i; i = i->nextSibling())
        i->setOpen(true);
    
    updateColumnWidths();
}


/*!
  Expands all directories.

  \sa collapseAll(), expandFirstLevel()
*/

void
QmPlayList::expandAll()
{
    for (QListViewItemIterator i(this); i.current(); i++)
        i.current()->setOpen(true);

    updateColumnWidths();
}


/*!
  Collapses all directories.

  \sa expandAll(), expandFirstLevel()
*/

void
QmPlayList::collapseAll()
{
    for (QListViewItemIterator i(this); i.current(); i++)
        i.current()->setOpen(false);
    
    updateColumnWidths();
}


/*!
  Saves the playlist in xml format.
*/

void
QmPlayList::saveAsXml(const QString &filename) 
{
    int depth = 0;
    int song_count = 0;
    int dir_count = 0;
    long start_time = time(0);

    QmMainWindow::mainwin->setState( QmMainWindow::Busy );
    QFile save_file( filename);
    if ( !save_file.open( IO_WriteOnly ) )
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                        "Error: Could not write to \"%s\"",
                                                        save_file.name().latin1()));
        return;
    }
	
    QTextStream out( &save_file );
    if (QString("utf-8") == QmConfig::instance()->getString("stuff", "encoding"))
        out.setEncoding(QTextStream::UnicodeUTF8);

    out << "<?xml version='1.0' encoding=\"" << QmConfig::instance()->getString( "stuff", "encoding")
        << "\"?>\n"
        << "<!DOCTYPE APOLLO-PLAYLIST>\n"
        << "<apollo-playlist>\n";
    
    QmDirItem *dir;
    QmSongItem *song;
    for (QListViewItemIterator it(this); it.current(); ++it)
	{
        for (;it.current()->depth() < depth; --depth) 
            out << "</dir>\n";
        if (reinterpret_cast<QmPlayListItem*>(it.current())->isSong())
		{
            song = reinterpret_cast<QmSongItem*>(it.current());
            song->writeXml(out);

            if (++song_count%100 == 0)
            {
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Update,
                                                                "Saved %d songs in %d folders (%ld seconds)",
                                                                song_count,
                                                                dir_count,
                                                                time(0) - start_time));
            }
        }
        else
		{
            dir = reinterpret_cast<QmDirItem*>(it.current());

            dir->writeXml(out);
            
            if (++dir_count%10==0)
            {
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Update,
                                                                "Saved %d songs in %d folders (%ld seconds)",
                                                                song_count,
                                                                dir_count,
                                                                time(0)-start_time));
            }
            
            depth++;
        }
    }
    while (depth-->0)
        out << "</dir>\n";
    out << "</apollo-playlist>\n";
    
    int duration = time(0) - start_time;
    
    if (duration > 0)
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "Saved %d songs in %d folders (%ld seconds, %s)",
                                                        song_count,
                                                        dir_count,
                                                        time(0) - start_time,
                                                        filename.latin1()));
    }
    else
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "Saved %d songs in %d folders (%s)",
                                                        song_count,
                                                        dir_count,
                                                        filename.latin1()));
    }
    
    m_Dirty = false;
    QmMainWindow::mainwin->setState( QmMainWindow::Ready );
}

void
QmPlayList::saveAsM3u(const QString &filename) 
{
    QmMainWindow::mainwin->setState( QmMainWindow::Busy );
    QFile save_file( filename);
    
    if ( !save_file.open( IO_WriteOnly ) )
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                        "Error: Could not write to \"%s\"",
                                                        save_file.name().latin1()));
        return;
    }
	
    QTextStream out( &save_file );

	updateSongIndex();
    for (uint i=0; i<m_SongIndex.size(); ++i)
        out << m_SongIndex[i]->filePath() << "\n";

    QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                    "Saved %d songs in %s",
                                                    m_SongIndex.size(),
                                                    filename.latin1()));

    QmMainWindow::mainwin->setState(QmMainWindow::Ready);
}


/*!
  \return The playlist filename.
 */

const QString&
QmPlayList::filename() const
{
    return m_Filename;
}


/*!
  Calculates statistics, currently playlength, needs to be called manually
  until we come up with an acceptable performace auto-scheme.
 */

long
QmPlayList::reCalculate()
{
    long ret=0;
    for (QListViewItem *i = firstChild(); i; i=i->nextSibling())
        ret += static_cast<QmPlayListItem*>(i)->updateLength();

	updateSongIndex();
    m_PlayLength = m_PlayLengthNotCounted = 0;

    for (int i = m_SongIndex.size() - 1; i >= 0; --i)
    {
        if (m_SongIndex[i]->length() > 0) 
            m_PlayLength += m_SongIndex[i]->length();
        else
            m_PlayLengthNotCounted++;
    }

    QString tmp;
    secondsToTimeString(m_PlayLength, tmp);

    if(m_PlayLengthNotCounted > 0)
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "%d songs %s (%d not counted)",
                                                        m_SongIndex.size(),
                                                        tmp.latin1(),
                                                        m_PlayLengthNotCounted));
    }
    else
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "%d songs, %s",
                                                        m_SongIndex.size(),
                                                        tmp.latin1()));
    }

    return ret;
}


/*!
  \todo
 */

void
QmPlayList::updateColumnWidths()
{
}


/*!
  \returns True if \a e can be decoded (i.e. dropped), false otherwise.
*/

bool
QmPlayList::canDecode(
    QDragMoveEvent *e)
{
	return QUriDrag::canDecode(e) || QmListView::canDecode(e);
}


/*!
  Sets the column width of \a col to \a w.
 */

void
QmPlayList::setColumnWidth(
    int col,
    int w)
{
    QmListView::setColumnWidth(col, w);
    
    if(col == 1)
        QmListView::setColumnWidth(0, viewport()->width() - w);
}


/*!
  Reads the song info for all entries in the playlist. For MP3s, for example,
  the ID3 tags will be read.
*/

void
QmPlayList::readSongInfo()
{
	if (m_DeletingItems || QmMainWindow::mainwin->state() == QmMainWindow::Busy)
		return;
	
	QmMainWindow::mainwin->setState( QmMainWindow::Busy );
	
	QString format = QmConfig::instance()->getString("stuff", "display-format");
	QString formatMulti = QmConfig::instance()->getString("stuff", "display-format-multi");	
    const char *cformat = format.latin1();
	const char *cformatMulti = formatMulti.latin1();
	
    long len;
    long start_time = time(0);

	updateSongIndex();

    int i;
    for(unsigned i = 0; i < m_SongIndex.size(); ++i)
    {
        m_SongIndex[i]->readInfo(cformat, cformatMulti);
        
        if(i % 50 == 49)
		{
            len = time(0) - start_time;
            
            if(len)
            {
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Update,
                                                                "Read id3 info for %d songs (%ld seconds)",
                                                                i+1,
                                                                len));
            }
            else
            {
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Update,
                                                                "Read id3 info for %d songs",
                                                                i+1));
            }
        }
    }
    
    len = time(0) - start_time;
    
    if (len)
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "Read id3 info for %d songs (%ld seconds)",
                                                        i+1,
                                                        len));
    }
    else
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                        "Read id3 info for %d songs",
                                                        i+1));
    }

    triggerUpdate();
    setDirty();
    QmMainWindow::mainwin->setState(QmMainWindow::Ready);
}


/*!
  Generates a playlist for each artist and album
  Also generates a PlayListTree if alsoPlayListTree.
 */

void
QmPlayList::generatePlayListForEachArtistAndAlbum(
    int alsoPlayListTree) 
{
    QmMainWindow::mainwin->setState(QmMainWindow::Busy);

    QmConfig *conf = QmConfig::instance();
    
	updateSongIndex();
	// use a copy for sorting
	std::vector<QmSongItem*> si = m_SongIndex;
    std::sort(si.begin(), si.end(),
			  compareSongs<>(&QmSongItem::m_Artist, &QmSongItem::m_Album, &QmSongItem::m_CdPosition));

    QString pldir = QFileDialog::getExistingDirectory ( conf->getString("path", "playlist"),
                                                        this,
                                                        0,
                                                        tr("Choose directory to generate playlists in"),
                                                        true );
    if (pldir.isEmpty())
	{
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, tr("Canceled, no playlists saved")));
        QmMainWindow::mainwin->setState(QmMainWindow::Ready);
        return;
    }
    
    conf->set("path", "playlist", pldir);

    QString plt;
    QFile tree;
    QTextStream tout;
    QString enc = conf->getString("stuff", "encoding");
    bool utf8 = QString("utf-8") == enc;

    if (alsoPlayListTree)
	{
        plt = QFileDialog::getSaveFileName(conf->getString("path", "playlisttree"),
                                           QString::null, this, 0, tr("Name of Playlist tree"));
        
        if (plt.isEmpty())
            alsoPlayListTree = false;
        else
        {
            tree.setName(plt);
            
            if (!tree.open( IO_WriteOnly ) )
			{
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                                tr("Could not write playlist tree to \"%s\""),
                                                                plt.latin1()));
				alsoPlayListTree = false;
			}
            else
            {
                tout.setDevice( &tree );
                
                if (utf8)
                    tout.setEncoding(QTextStream::UnicodeUTF8);
                
                tout << "<?xml version='1.0' encoding=\"" << enc << "\"?>\n"
					 << "<!DOCTYPE APOLLO-PLAYLISTTREE>\n"
					 << "<apollo-playlisttree>\n";
            }
        }
    }

	// ar = artist, al = album
    QFile arfile;
    QFile alfile;	
    QString artist;
    QString album;
    bool first = true;
    bool firstAlbum = true;
    QTextStream arout;
    QTextStream alout;	
    int count = 0;
    QString arfilename;
    QString alfilename;
	QString arclean;			// artist name without special characters removed
	QString alclean;
    
    for (uint i = 1; i < si.size(); i++)
    {
        if (si[i]->artist().isEmpty() || si[i]->album().isEmpty())
             continue;
            
        if (si[i]->album() != album)
		{
            if (si[i]->artist() != artist)
			{
                artist = si[i]->artist();				
                if (!first) 
                {
                    if (!firstAlbum)
                        arout << "</dir>\n";
                    arout << "</dir>\n</apollo-playlist>\n";
                    
                    arfile.close();
                }
                
                firstAlbum = true;
				
				/// \todo clean this loop up and put it in a function 
				arclean.truncate(0);
				QChar c;
				for (uint j=0; j<artist.length(); ++j)
				{
					c = artist[j];
					if (c.isLetterOrNumber() || c == ' ' || c == '_' || c == "-")
						arclean += c;
				}
				arfilename = pldir + arclean + ".xml";
                arfile.setName( arfilename );
                
                if ( !arfile.open( IO_WriteOnly ) )
                {
                    QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                                    tr("Error: Could not write to \"%s\""),
                                                                    arfile.name().latin1()));
                    QmMainWindow::mainwin->setState(QmMainWindow::Ready);
					return;
                }
                
                if (alsoPlayListTree)
                {
					if (!first)
						tout << "</dir>\n";
					tout << "<dir><name><![CDATA[" << artist << "]]></name>\n";
				} 
                    
                QmMainWindow::mainwin->statusUpdate(new Message(Message::Update,
                                                                tr("Saving %s (%i)"),
                                                                si[i]->artist().latin1(),
                                                                ++count));
	
                arout.setDevice( &arfile );
                
                if (utf8)
                    arout.setEncoding(QTextStream::UnicodeUTF8);

                arout << "<?xml version='1.0' encoding=\"" << enc << "\"?>\n"
					  << "<!DOCTYPE APOLLO-PLAYLIST>\n"
					  << "<apollo-playlist>\n";

                arout << "<dir>\n" << "<name><![CDATA[" << artist << "]]></name>\n";
            }
            album = si[i]->album();
			
			alclean.truncate(0);
			QChar c;
			for (uint j=0; j<album.length(); ++j)
			{
				c = album[j];
				if (c.isLetterOrNumber() || c.isSpace() || c == '_' || c == "-")
					alclean += c;
			}
			alfilename = pldir + arclean + "  " + alclean + ".xml";
			if (!first)
			{
				alout << "</dir></dir>\n"; 
				alout << "</apollo-playlist>\n"; 
				alfile.close();
			}

			alfile.setName( alfilename );

			if ( !alfile.open( IO_WriteOnly ) )
			{
				QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
																tr("*Error: Could not write to \"%s\""),
																alfile.name().latin1()));
				QmMainWindow::mainwin->setState(QmMainWindow::Ready);
				return;
			}

			alout.setDevice( &alfile );

			if (utf8)
				alout.setEncoding(QTextStream::UnicodeUTF8);
			
			alout << "<?xml version='1.0' encoding=\"" << enc << "\"?>\n"
				  << "<!DOCTYPE APOLLO-PLAYLIST>\n"
				  << "<apollo-playlist>\n";

			if (alsoPlayListTree)
			{
				tout << "<playlist>\n"
					 << "<name><![CDATA[" << album << "]]></name>\n"
					 << "<path><![CDATA[" << alfilename << "]]></path>\n"
					 << "</playlist>\n";
			}
            
            if (!firstAlbum) 
                arout << "</dir>\n"; 
            
            arout << "<dir>\n" << "<name><![CDATA[" << album << "]]></name>\n";
			alout << "<dir>\n" << "<name><![CDATA[" << artist << "]]></name>\n";
			alout << "<dir>\n" << "<name><![CDATA[" << album << "]]></name>\n";

			first = firstAlbum = false;
        } 
        
        si[i]->writeXml(arout);
        si[i]->writeXml(alout);		
    }
    
    if (alsoPlayListTree)
        tout << "</dir>\n" << "</apollo-playlisttree>\n";
    
    QmMainWindow::mainwin->statusUpdate(new Message(Message::Status, "Generated %i playlists", count));
    QmMainWindow::mainwin->setState(QmMainWindow::Ready);
}


/*!
  Called when the selection has changed. Updates status info.
*/

void
QmPlayList::newSelection()
{
	if (m_DeletingItems || QmMainWindow::mainwin->state() == QmMainWindow::Busy)
		return;

	m_Delay.start(1000);
}

void
QmPlayList::newSelectionHandler()
{
	if (m_DeletingItems)
		return;
	
	updateSongIndex();
	long len = 0;
	int songs = 0;
	int notCounted = 0;			// songs lacking play length are not counted
	
    for (uint i = 0; i < m_SongIndex.size(); i++)
	{
		if (m_SongIndex[i]->isSong() && isSelected(m_SongIndex[i]))
		{
			if (m_SongIndex[i]->length()) 
				len += m_SongIndex[i]->length();
			else
				notCounted++;
			songs++;
		}
	}
	
	if(songs > 1)
	{
		QString time;
		secondsToTimeString(len, time);
		if (notCounted)
        {
			QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                            "Selected %d songs (%s %d not counted)",
                                                            songs,
                                                            time.latin1(),
                                                            notCounted));
        }
		else
        {
			QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                            "Selected %d songs (%s)",
                                                            songs,
                                                            time.latin1()));
        }
	}
	else
    {
        setDefaultStatus();
    }
}


/*!
  Generates the default status message showing information about the current
  playlist. This is the message that is shown when there is no "special"
  selection.
 */

void
QmPlayList::setDefaultStatus()
{
	if (m_DeletingItems || QmMainWindow::mainwin->state() == QmMainWindow::Busy)
		return;
	
    int songs = 0;
    int folders = 0;

    for(QListViewItemIterator it(this); it.current(); ++it)
	{
        if(reinterpret_cast<QmPlayListItem*>(it.current())->isSong())
            songs++;
        else
            folders++;
    }

    QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
                                                    "%d songs, %d folders",
                                                    songs,
                                                    folders));
}


/*!
  Regenerates the song index. Unless \a force is true, the index will only
  be updated if necessary.

  The song index is a std::vector of pointers to the non-bad songs to allow
  easy indexed access and iteration of non-bad songs, as well as temporary
  storage while reorganising the play list.
 */

void
QmPlayList::updateSongIndex(
    bool force) 
{
	if (m_SongIndexDirty || force)
	{
		m_SongIndex.clear();

		for (QListViewItemIterator it(this); it.current(); ++it)
        {
			if (static_cast<QmPlayListItem*>(it.current())->isSong())
				if (!static_cast<QmPlayListItem*>(it.current())->isBad())
					m_SongIndex.push_back(static_cast<QmSongItem*>(it.current()));
        }

		m_SongIndexDirty = false;
	}
	if (m_pCurrentSong==0 && !m_SongIndex.empty())
		m_pCurrentSong = m_SongIndex[0];
}


/*!
  Show properties for the selected item.
 */

void
QmPlayList::showItemProperties()
{
	QmSongItem *song = dynamic_cast<QmSongItem*>(currentItem());

	if(song == 0)
		return;

	if(m_pPropertyDlg != 0)
		delete m_pPropertyDlg;
	
	m_pPropertyDlg = new QmPropertyDialog;
	m_pPropertyDlg->addPage(tr("Info"), new QmSongInfoPropertyPage(song));
	m_pPropertyDlg->show();
}


/*!
  Get a song by it's linear position in the playlist. Automatically updates the
  song index when needed.
 */

QmSongItem*
QmPlayList::getSong(
    int index) 
{
	updateSongIndex();
    return m_SongIndex[index];
}


#ifndef DOXYGEN_SKIP_THIS

/*!
  Compare function for songs, used to sort the song index.
*/

template<typename t1, typename t2, typename t3>
bool
compareSongs<t1,t2,t3>::operator()(
    const QmSongItem *a,
    const QmSongItem *b) const
{
    CHECK_PTR(a);
    CHECK_PTR(b);

    if (depth == 2)
	{
        if (a->*sort_on1 == b->*sort_on1)
            return a->*sort_on2 < b->*sort_on2;
    }
    else if (depth == 3)
	{
        if (a->*sort_on1 == b->*sort_on1)
		{
            if (a->*sort_on2 == b->*sort_on2)
                return a->*sort_on3 < b->*sort_on3;
            else
                return a->*sort_on2 < b->*sort_on2;
        }
    }
    return a->*sort_on1 < b->*sort_on1;
}

#endif // DOXYGEN_SKIP_THIS
