/* qmplaylistbrowser.cpp
 *
 * $Id: qmplaylistbrowser.cpp,v 1.30 2002/03/31 23:52:42 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmplaylistbrowser.h"

#include "message.h"
#include "qmbrowseritem.h"
#include "qmconfig.h"
#include "qmdefaultmenuitem.h"
#include "qmdiritem.h"
#include "qmfiledialog.h"
#include "qmmainwindow.h"
#include "qmplaylist.h"
#include "qmplaylistbrowseritem.h"
#include "qmplaylistpropertypage.h"
#include "qmrecoverymanager.h"
#include "qmrenameedit.h"

#include <iostream>

#include <qdom.h>
#include <qfile.h>
#include <qfont.h>
#include <qheader.h>
#include <qnamespace.h>
#include <qpopupmenu.h>
#include <qstring.h>

/* Disables false warning in Visual Studio about dynamic_cast */

#ifdef _WS_WIN_
#pragma warning(disable: 4541)
#endif

/*!
 * \file qmplaylistbrowser.cpp
 * \brief The playlist tree widget, for browsing playlists
 */

/*!
  \class QmPlayListBrowser qmplaylistbrowser.h
  \brief Represents the playlist tree

  Provides functionality for organizing playlist in trees.  The playlists
  (apollo xml/m3u files) can be used to populate the playlist widget,
  either file by file or by adding entire tree/subtrees of playlists to the
  playlist widget.
*/


/*!
  Creates a new playlist browser.
*/

QmPlayListBrowser::QmPlayListBrowser(
	QWidget *parent,
	const char * /*name*/ )
	: QmListView(parent, "playlisttree"),
	  m_pPlayListSelected(0),
	  m_pPropertyPage(0),
	  m_pRenameEdit(0),
	  m_pRenameItem(0),
	  m_pFileDialog(0),
      m_PlayListTree()
{
	enableSelfSave(QmConfig::instance()->configPath() + "playlisttree");
	addColumn(tr("List"));
	header()->hide();
	setRootIsDecorated(true);
    setSelectionMode(Extended);
	reorganizable(true);
	connect(this, SIGNAL(mouseButtonClicked(int, QListViewItem*, const QPoint &, int)),
			this, SLOT(mouseClick(int, QListViewItem*, const QPoint &, int)));
	connect(this, SIGNAL(doubleClicked( QListViewItem* )),
			this, SLOT(mouseDoubleClick( QListViewItem* )));

	if(QmConfig::instance()->loadingEnabled(QmConfig::PlayListTree))
		loadPlayListTree();

	m_pRenameEdit = new QmRenameEdit(this);
	m_pRenameEdit->hide();
	connect(m_pRenameEdit, SIGNAL(newName(const QString &)), this, SLOT(renameItem(const QString &)));

	// -----------
	
	m_pMenu = new QPopupMenu(this);
	m_PrependPlayListId = m_pMenu->insertItem(tr("Add to top"),
											  this,
											  SLOT(prependPlayList()));
	QFont f = m_pMenu->font();
    m_AppendPlayListId = m_pMenu->insertItem(new QmDefaultMenuItem(tr("Add to end"), f));
    m_pMenu->connectItem(m_AppendPlayListId, this, SLOT(appendPlayList()));

	m_ReplacePlayListId = m_pMenu->insertItem(tr("Replace (middle click)"),
											  this,
											  SLOT(replacePlayList()));
	m_pMenu->insertSeparator();
	m_AddPlayListFolderId = m_pMenu->insertItem(tr("Add folder"),
												this,
												SLOT(addPlayListFolder()));
	m_RenamePlayListId = m_pMenu->insertItem(tr("Rename"),
											 this,
											 SLOT(renamePlayList()));
	m_PropertyPlayListId = m_pMenu->insertItem(tr("Properties..."),
											   this,
											   SLOT(propertyPlayList()));
	m_pMenu->insertSeparator();
	m_RemovePlayListId = m_pMenu->insertItem(tr("Remove"),
											 this,
											 SLOT(removeSelectedItems()));
	m_pMenu->insertItem(
        tr("Clear"),
        this,
        SLOT(clear()));
	
	m_pMenu->insertSeparator();
	m_SavePlayListId1 = m_pMenu->insertItem(tr("Add playlist file..."),
											this,
											SLOT(addOldPlayList()));
	m_SavePlayListId2 = m_pMenu->insertItem(tr("Put current playlist in tree..."),
											this,
											SLOT(addNewPlayList()));
    m_LoadFileId = m_pMenu->insertItem(tr("Load new playlist tree..."), this, SLOT(loadFile()));
    m_SaveFileId = m_pMenu->insertItem(tr("Save playlist tree"), this, SLOT(saveFile(int)));
	m_pMenu->setItemParameter(m_SaveFileId, 0);
    m_SaveFileAsId = m_pMenu->insertItem(tr("Save playlist tree As..."), this, SLOT(saveFile(int)));
	m_pMenu->setItemParameter(m_SaveFileId, 1);
}


/*!
  Saves the playlist tree.
*/

QmPlayListBrowser::~QmPlayListBrowser()
{
	if( ! fileName().isEmpty())
		save();
}


/*!
  Identical to QListView::currentItem() except it casts the item into
  a QmPlayListBrowserItem pointer.  Provided for convenience.
  
  \return The currently selected playlist or 0 if none selected.
*/

QmPlayListBrowserItem *
QmPlayListBrowser::currentPlayList()
{
	return static_cast<QmPlayListBrowserItem *>(currentItem());
}


/*!
  Called when an item is expanded.  Takes care of collapsing the previously
  expanded item, if any, so that only one item is expanded at a time.

  Currently commented out because it doesn't work (?)
  \bug Fails if there are subitems (which is not supported yet)
 */

void
QmPlayListBrowser::expanded(
	QListViewItem * /* item */)
{
	/*
	if(m_pLastExpanded != 0 && m_pLastExpanded != item)
		m_pLastExpanded->setOpen(false);

	m_pLastExpanded = item;
	*/
}


/*!
  Called when either of the mouse buttons are clicked.

  \sa showPopup(QListViewItem*, const QPoint&, int)
 */

void
QmPlayListBrowser::mouseClick(
    int button,
    QListViewItem * item,
    const QPoint & pos,
    int c )
{
    if (QmMainWindow::mainwin->state() == QmMainWindow::Ready) {
        QmMainWindow::mainwin->setState(QmMainWindow::Busy);
        if ( button & Qt::MidButton )
            replacePlayList();
        else if ( button & Qt::RightButton )
            showPopup(item, pos, c);
        else if ( button & Qt::LeftButton )
        {
            if(item == 0 && m_pRenameEdit->isVisible())
                m_pRenameEdit->hide();
        }
        
        QmMainWindow::mainwin->setState(QmMainWindow::Ready);
    }
}


/*!
  Called when the left mouse button has been double clicked.
 */

void
QmPlayListBrowser::mouseDoubleClick(
    QListViewItem * /* item */ )
{
    appendPlayList();
}


/*!
  Called by mouseClick() when the user clicks the right mouse button.

  Enables/disables the various menu items according to what item is
  currently highlighted (or no item is highlighted).

  \sa mouseClick(int, QListViewItem*, const QPoint&, int)
 */

void
QmPlayListBrowser::showPopup(
	QListViewItem *current,
	const QPoint &pos,
	int)
{
	m_pPlayListSelected = dynamic_cast<QmPlayListBrowserItem*>(current);
	
	m_pMenu->setItemEnabled(m_SavePlayListId2, !QmMainWindow::mainwin->playListIsEmpty());
    
	if(m_pPlayListSelected == 0)
		m_pMenu->setItemEnabled(m_AddPlayListFolderId, true);
	else
		m_pMenu->setItemEnabled(m_AddPlayListFolderId, m_pPlayListSelected->isFolder());

	bool enable = m_pPlayListSelected != 0;
    
	m_pMenu->setItemEnabled(m_RenamePlayListId, enable);
	m_pMenu->setItemEnabled(m_PropertyPlayListId, enable);
	m_pMenu->setItemEnabled(m_RemovePlayListId, enable);
	m_pMenu->setItemEnabled(m_PrependPlayListId, enable);
	m_pMenu->setItemEnabled(m_AppendPlayListId, enable);
	m_pMenu->setItemEnabled(m_ReplacePlayListId, enable);
		
	m_pMenu->popup(pos);
}


/*!
  Appends the playlist with the selected playlist, if any.

  \sa prependPlayList(), replacePlayList()
 */

void
QmPlayListBrowser::appendPlayList()
{
	QList<QListViewItem> *items = selectedItems();
	for (QmPlayListBrowserItem *i=reinterpret_cast<QmPlayListBrowserItem*>(items->first());
		 i;
		 i = reinterpret_cast<QmPlayListBrowserItem*>(items->next()))
    {
		if (i->isFolder())
			addFolder(i); 
		else
			QmMainWindow::mainwin->playListAppendList(i->fileName());
    }
	delete items;
}


/*!
  Prepends the playlist with the selected playlist, if any.

  \sa replacePlayList(), appendPlayList()
 */

void
QmPlayListBrowser::prependPlayList()
{
	QList<QListViewItem> *items = selectedItems();
    
	for (QmPlayListBrowserItem *i=reinterpret_cast<QmPlayListBrowserItem*>(items->last());
		 i;
		 i = reinterpret_cast<QmPlayListBrowserItem*>(items->prev()))
    {
		if (i->isFolder())
			addFolder(i, false); 
		else
			QmMainWindow::mainwin->playListPrependList(i->fileName());
    }
    
	delete items;
}


/*!
  Replaces the playlist with the selected playlist, if any.

  \sa prependPlayList(), appendPlayList()
 */

void
QmPlayListBrowser::replacePlayList()
{
	QmMainWindow::mainwin->clearPlayList();
	appendPlayList();
}


/*!
  Appends all the playlists in a folder to the playlist.
*/

void
QmPlayListBrowser::addFolder(QmPlayListBrowserItem *folder, bool append)
{
    for (QmPlayListBrowserItem *i = static_cast<QmPlayListBrowserItem *>(folder->firstChild());
         i;
         i = static_cast<QmPlayListBrowserItem *>(i->nextSibling()))
    {
        if (i->isFolder())
            addFolder(i, append);
        else
			if (append)
				QmMainWindow::mainwin->playListAppendList(i->fileName());
			else
				QmMainWindow::mainwin->playListPrependList(i->fileName());
    }
}


/*!
  Adds the current playlist to the tree, pops up a dialog asking for a name
  if it doesn't already have one.
 */

void
QmPlayListBrowser::addNewPlayList()
{
	QString filename = QmMainWindow::mainwin->playListFile();
	if (filename.isEmpty())
	{
		if(m_pFileDialog == 0)
			m_pFileDialog = new QmFileDialog();

		m_pFileDialog->setDir(QmConfig::instance()->getString("path", "playlist"));
		m_pFileDialog->setMode(QFileDialog::AnyFile);
		m_pFileDialog->setFilter("Playlists (*.m3u *.xml)");

		connect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
				 this, SLOT( doAddNewPlayList( QFileDialog *, int ) ) );

		m_pFileDialog->setCaption(tr("Apollo - Give the new playlist a name"));
		m_pFileDialog->show();
	}
	else
		reallyAddNewPlayList(filename);
}


/*!
  Called when the playlist file dialog is done.
*/

void
QmPlayListBrowser::doAddNewPlayList(
	QFileDialog *dlg,
	int val)
{
    if ( val == QDialog::Accepted )
    {
		QmConfig::instance()->set("path", "playlist", dlg->dirPath());
		reallyAddNewPlayList(dlg->selectedFile());
	}
	
    disconnect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
                this, SLOT( doAddNewPlayList( QFileDialog *, int ) ) );
}


/*!
  Actually adds \a playlist to the tree.  Called from addNewPlayList() if
  the playlist already had a name, and from doAddNewPlayList() if it didn't.
  (All as a result of the "Put current playlist in tree..." context menu)
*/

void
QmPlayListBrowser::reallyAddNewPlayList(const QString &playlist)
{
	QFileInfo fi(playlist);

	if(m_pPlayListSelected == 0)
		(void) new QmPlayListBrowserItem(this, 0, fi.baseName(), playlist);
	else if(m_pPlayListSelected->isFolder())
	{
		m_pPlayListSelected->setExpandable(true);
		m_pPlayListSelected->setOpen(true);
		clearSelection();
		
		(void) new QmPlayListBrowserItem(m_pPlayListSelected, 0, fi.baseName(), playlist);
	}
	else
		(void) new QmPlayListBrowserItem(this, 0, fi.baseName(), playlist);
}


/*!
  Pops up a dialog asking for the name (connecting doSaveFile() on close)
  if it doesn't have a name or \a forceAskp, otherwize saves the tree.
*/

void
QmPlayListBrowser::saveFile(int forceAskp)
{
	if (fileName().isEmpty() || forceAskp)
	{
		if(m_pFileDialog == 0)
			m_pFileDialog = new QmFileDialog();
		
		m_pFileDialog->setDir(QmConfig::instance()->getString("path", "playlisttree"));
		m_pFileDialog->setMode(QFileDialog::AnyFile);
		m_pFileDialog->setFilter("Playlist tree (*.xml)");
		
		connect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
             this, SLOT( doSaveFile( QFileDialog *, int ) ) );
		
		m_pFileDialog->setCaption(tr("Apollo - Enter playlist tree name"));
		m_pFileDialog->show();
	}
	else
	{
		save();
	}
}


/*!
 */

void
QmPlayListBrowser::doSaveFile(
	QFileDialog *dlg,
	int val)
{
    if ( val == QDialog::Accepted )
    {
		QmConfig::instance()->set("path", "playlisttree", dlg->dirPath());
		enableSelfSave(dlg->selectedFile());
		save();
	}
	
    disconnect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
                this, SLOT( doSaveFile( QFileDialog *, int ) ) );
}


/*!
  Pops up a dialog asking which playlist(s) to add to the tree
 */

void
QmPlayListBrowser::addOldPlayList()
{
	if(m_pFileDialog == 0)
		m_pFileDialog = new QmFileDialog();
		
	m_pFileDialog->setDir(QmConfig::instance()->getString("path", "playlist"));
	m_pFileDialog->setMode(QFileDialog::ExistingFiles);
	m_pFileDialog->setFilter("Playlists (*.m3u *.xml)");

    connect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
             this, SLOT( doAddOldPlayList( QFileDialog *, int ) ) );
	
    m_pFileDialog->setCaption(tr("Apollo - Select playlist(s)"));
    m_pFileDialog->show();
}


/*!
  Called when the playlist file dialog is done, adds the selected playlists to the tree
*/

void
QmPlayListBrowser::doAddOldPlayList(
	QFileDialog *dlg,
	int val)
{
    if ( val == QDialog::Accepted )
    {
		QmConfig::instance()->set("path", "playlist", dlg->dirPath());
        QStringList files = dlg->selectedFiles();
		QmPlayListBrowserItem *item = 0;
		QFileInfo fi;

		if (m_pPlayListSelected && m_pPlayListSelected->isFolder())
		{
			m_pPlayListSelected->setExpandable(true);
			m_pPlayListSelected->setOpen(true);
			clearSelection();
		}

		for (QStringList::Iterator file = files.begin(); file != files.end(); ++file)
		{

			fi.setFile(*file);
			
			if (m_pPlayListSelected == 0)
				item = new QmPlayListBrowserItem(this, 0, fi.baseName(), *file);
			else if (m_pPlayListSelected->isFolder())
				item = new QmPlayListBrowserItem(m_pPlayListSelected, 0, fi.baseName(), *file);
			else
				item = new QmPlayListBrowserItem(this, 0, fi.baseName(), *file);
		}
    }
	
    disconnect( m_pFileDialog, SIGNAL( dialogDone( QFileDialog *, int ) ),
                this, SLOT( doAddOldPlayList( QFileDialog *, int ) ) );
}


/*!
  Adds a playlist folder, i.e. an item in the tree to hold playlists or other folders.
*/

void
QmPlayListBrowser::addPlayListFolder()
{
	QmPlayListBrowserItem *item = 0;
	
	if(m_pPlayListSelected == 0)
		item = new QmPlayListBrowserItem(this, 0, tr("New folder"), "", true);
	else
	{
		m_pPlayListSelected->setExpandable(true);
		m_pPlayListSelected->setOpen(true);
		clearSelection();
		
		item = new QmPlayListBrowserItem(m_pPlayListSelected, 0, tr("New folder"), "", true);
	}

	renamePlayList(item);
}


/*!
  Renames the stored item to \a text.  Sorting will be applied.

  \bug Accelerator keys in toolbar override the rename edit.  Overriding
  QApplication::event() may be necessary.  Focus proxies does not seem to work.
 */

void
QmPlayListBrowser::renameItem(
	const QString &text)
{
	if(m_pRenameItem == 0)
		return;

	m_pRenameItem->setText(0, text);

	if(m_pRenameItem->parent())
		m_pRenameItem->parent()->sort();
	else
		sort();
}


/*!
  Renames the \e active playlist tree item.

  \sa renamePlayList(QListViewItem *)
 */

void
QmPlayListBrowser::renamePlayList()
{
	if (currentItem())
		renamePlayList(currentItem());
}


/*!
  Renames the tree item \a item.

  \sa renamePlayList()
*/

void
QmPlayListBrowser::renamePlayList(
	QListViewItem *item)
{
	m_pRenameItem = item;
	m_pRenameEdit->position(item);
}


/*!
  Displays the properties for the active playlist tree item.
  Currently name and path.
*/

void
QmPlayListBrowser::propertyPlayList()
{
	QmPlayListBrowserItem *current = currentPlayList();

	if(current == 0)
		return;
	
	if(m_pPropertyPage)
		delete m_pPropertyPage;
	
	m_pPropertyPage = new QmPlayListPropertyPage(current);
	m_pPropertyPage->show();
}


/** @name Load functions
	The family of functions to load the playlists tree from disk.
 */

/*@{*/


/*!
  Opens the load file dialog, connects doLoadFile() on close
*/

void
QmPlayListBrowser::loadFile() 
{
    if (QmMainWindow::mainwin->state() == QmMainWindow::Busy)
        return;

    QmConfig *conf = QmConfig::instance();

    QmFileDialog *dlg = new QmFileDialog(conf->getString("path", "playlisttree"),
                                         QString::null,
                                         0,
                                         "Apollo - Load playlist tree");
    
    connect(dlg, SIGNAL(dialogDone(QFileDialog *, int)), this, SLOT(doLoadFile(QFileDialog *, int)));
    dlg->setCaption(tr("Apollo - Load playlist tree"));
    dlg->setMode(QFileDialog::ExistingFile);

    QRect file_rect = QmConfig::instance()->getRect( "file-dialog", "geometry" );
	
    if ( file_rect.isValid() )
    {
        dlg->resize( file_rect.size() );
        dlg->move( file_rect.topLeft() );
    }
	
    dlg->show();
}


/*!
  Callback from loadFile().  Loads a playlist tree.
 */

void
QmPlayListBrowser::doLoadFile(QFileDialog *dlg, int val)
{
    QmConfig *conf = QmConfig::instance();
    conf->set("path", "playlisttree", dlg->dir()->absPath());
    conf->set("file-dialog", "geometry", QRect(dlg->pos(), dlg->size()));

    if (val == QDialog::Accepted)
    {
        QString name = dlg->selectedFile();
        if (!name.isEmpty())
		{
			enableSelfSave(name);
            loadPlayListTree(name);
		}
    }
    
    disconnect(dlg, SIGNAL(dialogDone(QFileDialog *, int)), this, SLOT(doLoadFile(QFileDialog *, int)));
    delete dlg;
}


/*!
  Loads the XML file in the .apollo setting directory that populates the
  playlist tree (this widget).
*/

void
QmPlayListBrowser::loadPlayListTree()
{
    loadPlayListTree(QmConfig::instance()->configPath() + "playlisttree" );
}


/*!
  Loads the XML file \a filename and populates the playlist tree (this widget).
*/

void
QmPlayListBrowser::loadPlayListTree(
    const QString &filename)
{
	if(QmRecoveryManager::instance()->loadingEnabled(QmConfig::PlayListTree) == false)
		return;
	QDomDocument doc( "apollo-playlisttree" );
	QFile f( filename );

	if ( ! f.open( IO_ReadOnly ) )
    {
        QmMainWindow::mainwin->statusUpdate(
			new Message(Message::Error, "Could not read %s", filename.latin1()));
		return;
    }

	if ( ! doc.setContent( &f ) )
	{
        QmMainWindow::mainwin->statusUpdate(
			new Message(Message::Error, "QmPlayListBrowser: doc.setContent failed"));
		f.close();
		return;
	}

	f.close();
 
	// Get root node (element)
	QDomNode node = doc.documentElement();

	if(node.nodeName() != "apollo-playlisttree")
	{
        QmMainWindow::mainwin->statusUpdate(
			new Message(Message::Error, "%s is not an apollo-playlisttree", filename.latin1()));
		return;
	}

    m_PlayListTree = filename;
    clear();
    QmMainWindow::mainwin->setState( QmMainWindow::Busy );

	// Get the first child in the document.  This should be a 'dir' element. 
    QDomNode child = node.firstChild();
	int lists = 0, folders = 0;
    while ( !child.isNull() )
    {
        loadPlayListTree(child, 0, lists, folders);
        child = child.nextSibling();
    }

	if (folders > 0)
		QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
													"Loaded %s.  %d playlists in %d folders",
													filename.latin1(),
													lists,
													folders));
	else
		QmMainWindow::mainwin->statusUpdate(new Message(Message::Status,
														"Loaded %s.  %d playlists.",
														filename.latin1(),
														lists));
    QmMainWindow::mainwin->setState( QmMainWindow::Ready );
}


/*!
  Helper function for recursively loading the playlist tree.

  This function is for subitems.  It has code that is quite similar to
  loadPlayListTree(), but that function is for root items (i.e. they have
  the listview as parent), while this function is for child items (i.e.
  they have some item as parent).
*/

void
QmPlayListBrowser::loadPlayListTree(
	QDomNode &node,
	QmPlayListBrowserItem *parent,
	int &lists,
	int &folders)
{
    QmPlayListBrowserItem *child = 0;
    QString path, name, open;
    bool isFolder;

    path = name = open = QString::null;

    QValueList<QDomNode> nodes;

	if(node.nodeName() != "dir" && node.nodeName() != "playlist")
	{
		qWarning("QmPlayListBrowser::loadPlayListNode(): Expected \"dir\" or \"playlist\", "
				 "found \"%s\".\n",
                 node.nodeName().latin1());
		return;
	}

	isFolder = node.nodeName() == "dir";
	QDomNodeList children = node.childNodes();
	QDomNode child_node, data;

	for(unsigned int i = 0; i < children.count(); i++)
	{
		child_node = children.item(i);
		data  = child_node.firstChild();

		if(child_node.nodeName() == "name")
			name = data.nodeValue();
		else if(child_node.nodeName() == "title")
			name = data.nodeValue();
		else if(child_node.nodeName() == "path")
			path = data.nodeValue();
		else if(child_node.nodeName() == "open")
			open = data.nodeValue();
        else
            nodes.append( child_node );
	}

    bool is_dir = false;
    if (node.nodeName() == "dir" && path.isEmpty())
    {
        folders++;
        if (parent ==  0)
            child = new QmPlayListBrowserItem(this, child, name, QString::null, true);
        else
            child = new QmPlayListBrowserItem(parent, child, name, QString::null, true);

        if(open == "1")
            child->setOpen(true);

        child->setExpandable(true);
        is_dir = true;
    }
    else if (node.nodeName() == "playlist" || ( node.nodeName() == "dir" && !path.isEmpty()) )
    {
        lists++;
        if (parent ==  0)
            child = new QmPlayListBrowserItem(this, child, name, path, false);
        else
            child = new QmPlayListBrowserItem(parent, child, name, path, false);
    }
    else
    {
        QmMainWindow::mainwin->statusUpdate(new Message(Message::Error,
                                                        "Error: expected `dir' or `playlist', found %s",
                                                        node.nodeName().latin1()));
        return;
    }

    if ( is_dir )
    {
        for ( QValueList<QDomNode>::ConstIterator it = nodes.begin(); it != nodes.end(); ++it )
        {
            child_node = *it;
            loadPlayListTree( child_node, child, lists, folders);
        }
    }
}
/*@}*/ // End of Doxygen group tag.


/*!
  \note: identical code in qmdirbrowser - modify both
*/

void
QmPlayListBrowser::startDragging()
{
	if (!reorganizable())
		return;

	QStringList files;
	QList<QListViewItem> *selected = selectedItems();
	for (QListViewItem *i=selected->first(); i; i=selected->next())
		files.append(reinterpret_cast<QmBrowserItem*>(i)->fileName());
	delete selected;

	QUriDrag *d = new QUriDrag(this);
	d->setFilenames(files);
	d->dragMove();
}
