/* qmsongitem.cpp
 *
 * $Id: qmsongitem.cpp,v 1.51.2.1 2002/10/10 21:40:52 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmsongitem.h"

#include <qbrush.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qstringlist.h>
#include <qtextstream.h>

#include "qmconfig.h"
#include "qmmarkmanager.h"
#include "qmpixmapsupplier.h"
#include "qmsonginfo.h"

/**
 * @file qmsongitem.cpp
 * @brief A song.  Widget.
 */

/*!
  \class QmSongItem qmsongitem.h
  \brief Represents a song in the play list.

  This class maintains a so-called bad list.  The bad list is a list
  of files which will be skipped during play list playing.  This is
  useful if some files are played incorrectly by the underlaying player,
  even though the files may be fine.
*/


/*!
  Constructs a new song item in the list view \a parent,
  the song being pointed to in \a filename.
*/

QmSongItem::QmSongItem(
    QListView* parent,
    QListViewItem* after,
    QString filename,
    long lengthSeconds,
    int bitrate,
    bool vbr,
    QString title,
    QString artist,
    QString album,
    QString cdPosition,
	bool multiArtist,
	int year,
	QString comment,
	QString displayString)
    : QmPlayListItem(parent, after),
	  m_File(filename),
      m_Title(title),
      m_Artist(artist),
      m_Album(album),
      m_CdPosition(cdPosition),
	  m_Comment(comment),
	  m_DisplayString(displayString),
	  m_Year(year),	  
      m_Bitrate(bitrate),
	  m_Rating(),
	  m_Played(),
      m_VariableBitrate(vbr),
	  m_Playing(false),
	  m_IsBad(false),
	  m_MultiArtistAlbum(multiArtist)
{
    init(lengthSeconds);
} 


/*!
  Destroys the song item.
*/

QmSongItem::~QmSongItem()
{
}


/*!
  Constructs a new song item as a child of the folder \a parent,
  placed below \a after, the song being pointed to in \a filename.
*/

QmSongItem::QmSongItem(
    QmPlayListItem* parent,
    QListViewItem* after,
    QString filename,
    long lengthSeconds,
    int bitrate,
    bool vbr,
    QString title,
    QString artist,
    QString album,
    QString cdPosition,
	bool multiArtist,
	int year,
	QString comment,
	QString displayString)
    : QmPlayListItem(parent, after),
	  m_File(filename),
      m_Title(title),
      m_Artist(artist),
      m_Album(album),
      m_CdPosition(cdPosition),
	  m_Comment(comment),
	  m_DisplayString(displayString),
	  m_Year(year),	  
      m_Bitrate(bitrate),
	  m_Rating(),
	  m_Played(),
      m_VariableBitrate(vbr),
	  m_Playing(false),
	  m_IsBad(false),
	  m_MultiArtistAlbum(multiArtist)
{
    init(lengthSeconds);
}


/*!
  Initializes the song item with a pixmap and sets some variables.
*/

void
QmSongItem::init(long lengthSeconds)
{
    m_Playing = false;
    setLengthSeconds(lengthSeconds);
    m_LengthSeconds = lengthSeconds;
	
}


/*!
  Sets up pixmaps, if necessary.
 */

void
QmSongItem::setup()
{
    QListViewItem::setup();
    bool packed;
    QmConfig::instance()->get( "stuff", "packed", packed);

    if (packed)
        setHeight(listView()->fontMetrics().height());
    else
	{
        QmPixmapSupplier *sup = QmPixmapSupplier::instance();
        setPixmap( 0, sup->pixmap( "icons-songitem" ) );
    }
}


/*!
  Reads song information from the file and fills out \a format with
  the values retrieved from the song, if any.
*/

void
QmSongItem::readInfo(
	const char *format,
	const char *multiFormat)
{
	QmSongInfo *si = QmSongInfo::create(m_File.filePath());
    CHECK_PTR(si);

    bool changed = false;
	setField(m_Album, si->get(QmSongInfo::Album), changed);
	setField(m_Title, si->get(QmSongInfo::Title), changed);
	setField(m_Artist, si->get(QmSongInfo::Artist), changed);
	setField(m_CdPosition, si->get(QmSongInfo::Track), changed);

	if (si->get(QmSongInfo::MultiArtist).isEmpty() == m_MultiArtistAlbum)
	{
		m_MultiArtistAlbum = !m_MultiArtistAlbum;
		changed = true;
	}

	QString len = si->get(QmSongInfo::Length);
    
	if( ! len.isEmpty())
	{
		changed = true;
		setLengthSeconds(len.toLong());
	}

    if (changed)
        setDisplayFormat(format, multiFormat);

	delete si;
}


/*!
  If \a value is non-empty and not the same as \a field, \a field will
  be updated and \a change set to true. Otherwise, this function does
  nothing.
 */

void
QmSongItem::setField(
	QString &field,
	const QString &value,
	bool &change)
{
    if(value.isEmpty())
        return;

//    cout << field << " " << value << std::endl;
    
    if (field != value)
	{
        field = value;
        change = true;
    }
}


/*!
  Sets the item as being played and repaints the item.
*/

void
QmSongItem::setPlaying(
    bool p)
{
    m_Playing = p;
	repaint();
}


/*!
  Overridden to return the correct text for the various columns.
  Column 0 returns either the filename or the song title if it has been set.
  Column 1 returns the song length or an empty string.
  All other columns return "-"
*/

QString
QmSongItem::text(
    int column) const
{
    switch(column)
    {
        case 0:
            return m_DisplayString.isEmpty() ?  m_File.fileName() : m_DisplayString;
            break;

        case 1:
            return m_Length;
            break;

        default:
            return "-";
            break;
    }
}


/*!
  \return The file path of the song item, ie. the filename including the path.
*/

QString
QmSongItem::filePath() const
{
    return m_File.filePath();
}


/*!
  Custom paint routine.
*/

void
QmSongItem::paintCell(
    QPainter *p,
    const QColorGroup &cg,
    int column,
    int width,
    int alignment)
{
//	QmPlayListItem::paintCell(p, cg, column, width, alignment);

	if(isSelected())
	{
		p->setPen(cg.highlightedText());

        if(this == matchItem())
            p->fillRect(0, 0, width, height(), QBrush(red));
		else if(isBad())
			p->fillRect(0, 0, width, height(), cg.mid());
		else if(m_Playing)
			p->fillRect(0, 0, width, height(), QBrush(blue));
		else
			p->fillRect(0, 0, width, height(), cg.highlight());
	}
	else
	{
        if(this == matchItem())
            p->setPen(red);
		else if(isBad())
			p->setPen(cg.mid());
		else if(m_Playing)
			p->setPen(blue);
		else
			p->setPen(cg.text());

		p->fillRect(0, 0, width, height(), cg.base());
	}

	if(column == 0)
	{
        bool packed;
        QmConfig::instance()->get( "stuff", "packed", packed);
		
        if(packed)
		{
            p->drawText(listView()->itemMargin() + listView()->itemMargin(),        
                        height() / 2 - p->fontMetrics().height() / 2,
                        width + 1,
                        height(),
                        alignment,
                        text(column));
        }
        else
		{    
            p->drawPixmap(listView()->itemMargin(), listView()->itemMargin(),
                          *pixmap(0));
	
            p->drawText(listView()->itemMargin() + pixmap(0)->width() + listView()->itemMargin(),
                        height() / 2 - p->fontMetrics().height() / 2,
                        width + 1,
                        height(),
                        alignment,
                        text(column));
        }
	}
	else
	{
		p->drawText(listView()->itemMargin(),
					height() / 2 - p->fontMetrics().height() / 2,
					width - 2*listView()->itemMargin(),
					height(),
					alignment,
					text(column));
	}
}


/*!
  Paints the focus for the item.
*/

void
QmSongItem::paintFocus(
    QPainter *p,
    const QColorGroup &cg,
    const QRect &r)
{
	p->setPen(cg.highlight());
	QRect rect(r.x() + 1, r.y() + 1, r.width() - 2, r.height() - 2);
	p->drawWinFocusRect(rect);
}


/*!
  Creates a "save string" which be saved to disk, and loaded at a later time
  to create an identical instance of this class.
  
  \return The save string of the item.
 */

QString
QmSongItem::save(
	const QString &rel_path) const
{
    QDir dir = m_File.dir();
    QDir rel_dir( rel_path );
    QString name = m_File.fileName();
    bool done = false;
    QString full_dir_name, dir_name;
	
    while ( !done )
    {
        if ( rel_dir == dir )
            done = true;
        else
        {
            dir_name = dir.dirName();
            if ( dir.isRoot() )
            {
                full_dir_name = QString( "/" ) + full_dir_name;
                done = true;
            }
            else
            {
                if ( !dir.cdUp() )
                    done = true;
                else
                    full_dir_name = dir_name + "/" + full_dir_name;
            }
        }
    }
	
    if ( full_dir_name.isEmpty() )
        return name;
    else
        return full_dir_name + name;
}


/*!
  This function is used to determine whether an item is a song or a folder.

  \return True for items of this class.
*/

bool
QmSongItem::isSong() const
{
	return true;
}


/*!
  Sets the bad mark if \a mark is true, clears it if false.
*/

void
QmSongItem::setMark(
	bool mark )
{
	if(mark)
		QmMarkManager::instance()->insert(filePath());
	else
		QmMarkManager::instance()->remove(filePath());
	
	repaint();
}


/*!
  Sets the bitrate to \a bitrate.
*/

void
QmSongItem::setBitrate(
	int bitrate)
{
    m_Bitrate = bitrate;
}


/*!
  Sets the rating to \a rating.
 */

void
QmSongItem::setRating(
	int rating)
{
	m_Rating = rating;	
}


/*!
  Increases the rating by one and returns the new value.

  \sa decRating()
*/

int
QmSongItem::incRating()
{
	return ++m_Rating;
}


/*!
  Decreases the rating by one and returns the new value.

  \sa incRating()
*/

int
QmSongItem::decRating()
{
	return --m_Rating;
}


/*!
  \sa resetPlayed()
 */

int
QmSongItem::played() const
{
	return m_Played;
}


/*!
  \sa played()
 */

void
QmSongItem::resetPlayed()
{
	m_Played = 0;
}


/*!

 */

void
QmSongItem::setVariableBitrate(
	bool vbr)
{
	m_VariableBitrate = vbr;
}


/*!

*/

void
QmSongItem::setDisplayFormat(
	const QString &format,
	const QString &multiFormat)
{
	setDisplayFormat(format.latin1(), multiFormat.latin1());
}


/*!
  Sets the format of the name displayed in the playlist to \a format.

  Code licensed from Espen Skoglund <esk@ira.uka.de>, and Hvard Kvlen
  <havardk@xmms.org> of http://www.xmms.org.
*/

void
QmSongItem::setDisplayFormat(
	const char* singleFormat,
	const char* multiFormat)
{
	char c, *p, outbuf[256], convert[16], *string;
	int numdigits, numpr, val, size, i;
	int width, precision;
	bool f_left, f_space, f_zero, more_flags;
	bool did_output = false;
	char digits[] = "0123456789";
	QString tmp;

	const char *format = m_MultiArtistAlbum ? multiFormat : singleFormat;
	
#define PUTCH(ch) \
	do { \
		if ( size-- <= 0 ) \
			goto Done; \
		*p++ = (ch); \
	} while (0)

#define LEFTPAD(num) \
	do { \
    	int cnt = (num); \
		if ( ! f_left && cnt > 0 ) \
            while ( cnt-- > 0 ) \
                PUTCH(f_zero ? '0' : ' '); \
	} while (0)

#define RIGHTPAD(num) \
	do { \
		int cnt = (num); \
		if ( f_left && cnt > 0 ) \
			while ( cnt-- > 0 ) \
				PUTCH( ' ' ); \
	} while (0)

	size = sizeof (outbuf) - 1;
	p = outbuf;
	
	if ( format == 0 )
		return;
	
	while (size > 0)
	{
		/* Copy characters until we encounter '%'. */
		while ((c = *format++) != '%')
		{
			if (size-- <= 0 || c == '\0')
				goto Done;
			*p++ = c;
		}

		f_left = f_space = f_zero = false;
		more_flags = true;

		/* Parse flags. */
		while (more_flags)
		{
			switch (*format)
			{
				case '-':
					f_left = true;
					format++;
					break;
				case ' ':
					f_space = true;
					format++;
					break;
				case '0':
					f_zero = true;
					format++;
					break;
				default:
					more_flags = false;
					break;
			}
		}


		/* Parse field width. */
		if ((c = *format) >= '0' && c <= '9')
		{
			width = 0;
			while ((c = *format++) >= '0' && c <= '9')
			{
				width *= 10;
				width += c - '0';
			}
			format--;
		}
		else
			width = -1;


		/* Parse precision. */
		if (*format == '.')
		{
			if ((c = *++format) >= '0' && c <= '9')
			{
				precision = 0;
				while ((c = *format++) >= '0' && c <= '9')
				{
					precision *= 10;
					precision += c - '0';
				}
				format--;
			}
			else
				precision = -1;
		}
		else
			precision = -1;


		/* Parse format conversion. */
		switch (c = *format++)
		{
			case 'a':
				string = m_Album.isEmpty() ? 0 : const_cast<char *>(m_Album.latin1());
				goto Print_string;
			case 'c':
				string = m_Comment.isEmpty() ? 0 : const_cast<char *>(m_Comment.latin1());
				goto Print_string;
// 					case 'd':
// 						string = VS(date);
// 						goto Print_string;
			case 'e':
				tmp = m_File.extension(false);
				string = tmp.isEmpty() ? 0 : const_cast<char *>(tmp.latin1());
				goto Print_string;
			case 'f':
				tmp = m_File.fileName();
				string = tmp.isEmpty() ? 0 : const_cast<char *>(tmp.latin1());
				goto Print_string;
			case 'F':
				tmp = m_File.filePath();
				string = tmp.isEmpty() ? 0 : const_cast<char *>(tmp.latin1());
				goto Print_string;
// 					case 'g':
// 						string = VS(genre);
// 						goto Print_string;
			case 'n':
				bool ok;
				val = m_CdPosition.toInt(&ok);
				if (!ok)
					val = 0;
				goto Print_number;
			case 'b':
				val = m_Bitrate;
				goto Print_number;
			case 'p':
				string = m_Artist.isEmpty() ? 0 : const_cast<char *>(m_Artist.latin1());
				goto Print_string;
			case 't':
				string = m_Title.isEmpty() ? 0 : const_cast<char *>(m_Title.latin1());

		Print_string:
				if ( string == NULL )
					break;
				did_output = true;

				numpr = 0;
				if (width > 0)
				{
					/* Calculate printed size. */
					numpr = strlen(string);
					if (precision >= 0 && precision < numpr)
						numpr = precision;

					LEFTPAD(width - numpr);
				}

				/* Insert string. */
				if (precision >= 0)
				{
					while (precision-- > 0 &&
						   (c = *string++) != '\0')
						PUTCH(c);
				}
				else
				{
					while ((c = *string++) != '\0')
						PUTCH(c);
				}

				RIGHTPAD(width - numpr);
				break;

			case 'y':
				val = m_Year;

		Print_number:
				if ( val == 0 )
					break;
				if ( c != 'N' )
					did_output = true;

				/* Create reversed number string. */
				numdigits = 0;
				do
				{
					convert[numdigits++] = digits[val % 10];
					val /= 10;
				}
				while (val > 0);

				numpr = numdigits > precision ? numdigits : precision;

				/* Insert left padding. */
				if (!f_left && width > numpr)
				{
					if (f_zero)
						numpr = width;
					else
						for (i = width - numpr; i-- > 0;)
							PUTCH(' ');
				}

				/* Insert zero padding. */
				for (i = numpr - numdigits; i-- > 0;)
					PUTCH('0');

				/* Insert number. */
				while (numdigits > 0)
					PUTCH(convert[--numdigits]);

				RIGHTPAD(width - numpr);
				break;

			case '%':
				PUTCH('%');
				break;

			default:
				PUTCH('%');
				PUTCH(c);
				break;
		}
	}
	
 Done:
	*p = '\0';

	if (did_output)
		m_DisplayString = static_cast<const char *>(outbuf);
}


/*!
  Sets the display string to \a display.

  \sa displayString()
 */

void
QmSongItem::setDisplayString(
	const QString& display)
{
	m_DisplayString = display;
}


/*!
  \return The display string.
  \sa setDisplayString(const QString&)
 */

QString
QmSongItem::displayString() const
{
	return text(0);
}


/*!
  Sets the title of this song to \a title. This used instead of the filename if
  non-empty.
*/

void
QmSongItem::setSongTitle(
	const QString &title)
{
    m_Title = title;
}


/*!
  Creates and returns a songinfo object based on this song.

  \return The songinfo object. Will never be null.
  \warning The caller is responsible for deleting the returned object.
*/

QmSongInfo*
QmSongItem::info() const
{
	return QmSongInfo::create(m_File.filePath());
}


/*!
  \return True if this song is bad, false otherwise.
  \todo Cache flag?
*/

bool
QmSongItem::isBad() const
{
    return QmMarkManager::instance()->isBad(filePath());
}


/*!
  Saves the song to \a out as XML.
 */

void
QmSongItem::writeXml(
	QTextStream &out) const
{
    out << "<song>\n";
    out << "<path><![CDATA[" << m_File.filePath() << "]]></path>\n";
    if ( ! m_CdPosition.isEmpty()) 
        out << "<cd-position>" << m_CdPosition << "</cd-position>\n";
    if (m_LengthSeconds != 0) 
        out << "<length>" << m_LengthSeconds << "</length>\n";
    if ( ! m_Artist.isEmpty())
        out << "<artist><![CDATA[" << m_Artist << "]]></artist>\n";
    if ( ! m_Title.isEmpty())
        out << "<title><![CDATA[" << m_Title << "]]></title>\n";
    if ( ! m_Album.isEmpty()) 
        out << "<album><![CDATA[" << m_Album << "]]></album>\n";
    if (m_Bitrate > 0)
        out << "<bitrate>" << m_Bitrate << "</bitrate>\n";
    if (m_VariableBitrate)
        out << "<vbr>true</vbr>\n";
	if (m_MultiArtistAlbum)
		out << "<multi-artist>true</multi-artist>\n";
	if ( ! m_Comment.isEmpty())
		out << "<comment>" << m_Comment << "</comment>\n";
	if ( ! m_DisplayString.isEmpty())
		out << "<display-string><![CDATA[" << m_DisplayString << "]]></display-string>\n";
	if (m_Year > 0)
		out << "<year>" << m_Year << "</year>\n";
    out << "</song>\n"; 
}


